package main;

import java.io.File;
import java.io.FileOutputStream;

import utils.CommonUtils;
import utils.XLocateUtils;
import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.xserver.common.CoordFormat;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RouteInfo;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RouteOnMapDemo {

	public static final String SERVER = "localhost";
	public static String xmapUrl    = "http://" + SERVER + ":50010/xmap/ws/XMap";
	public static String xlocateUrl = "http://" + SERVER + ":50020/xlocate/ws/XLocate";
	public static String xrouteUrl  = "http://" + SERVER + ":50030/xroute/ws/XRoute";

	static boolean verbose = false;
	static String routeName = "-";
	static File outDir = new File(".");
	static ImageFileFormat format = ImageFileFormat.GIF;
	static String locatingProfile = "default";
	static String routingProfile = "default";
	static String mappingProfile = "default";
	static Address[] waypoints = null;
	static int imageWidth  = 500;
	static int imageHeight = 400;
	static String geometryEncoding = "PLAIN";
	static CoordFormat coordFormat = CoordFormat.PTV_MERCATOR;
	static boolean noRoute = false;
	static boolean noMap = false;

	private static final Option OPT_HELP = new Option("-h", "--help", null, "show help") { protected void setArg(String value) { usage(0); } };
	private static final Option OPT_VERBOSE = new Option("-v", "--verbose", null, "print verbose information") { protected void setArg(String value) { verbose = true; } };
	private static final Option OPT_NAME = new Option("-n", "--name", "name", "name of the route") { protected void setArg(String value) { routeName = value; } };
	private static final Option OPT_OUTDIR = new Option("-o", "--outDir", "dir", "output directory for the image") { protected void setArg(String value) { outDir = new File(value); if (!outDir.exists() || !outDir.canWrite()) usage(1); } };
	private static final Option OPT_IMGFMT = new Option("-i", "--imageFormat", "format", "output image format - must be one of GIF, JPG, or PNG") { protected void setArg(String value) { format = ImageFileFormat.fromString(value); } };
	private static final Option OPT_IMGSIZE = new Option("-s", "--imageSize", "widthxheight", "image size, e.g. 1024x860") { protected void setArg(String value) { try { String[] size = value.split("x"); imageWidth = Integer.parseInt(size[0]); imageHeight = Integer.parseInt(size[1]); } catch (Exception e) { usage(1); } } };
	private static final Option OPT_XLOCATE = new Option("-L", "--xlocate", "url", "the url of the xlocate server") { protected void setArg(String value) { xlocateUrl = value; } };
	private static final Option OPT_GCOPROF = new Option("-l", "--locatingProfile", "profile", "the name of the profile to be used for address geocoding") { protected void setArg(String value) { locatingProfile = value; } };
	private static final Option OPT_XROUTE = new Option("-R", "--xroute", "url", "the url of the xroute server") { protected void setArg(String value) { xrouteUrl = value; } };
	private static final Option OPT_RTGPROF = new Option("-r", "--routingProfile", "profile", "the name of the profile to be used for route computation") { protected void setArg(String value) { routingProfile = value; } };
	private static final Option OPT_XMAP = new Option("-M", "--xmap", "url", "the url of the xmap server") { protected void setArg(String value) { xmapUrl = value; } };
	private static final Option OPT_MAPPROF = new Option("-m", "--mappingProfile", "profile", "the name of the profile to be used for mapping") { protected void setArg(String value) { mappingProfile = value; } };
//	private static final Option OPT_GEOM = new Option("-g", "--geometry", "geometrylist", "comma separated list of some of the values PLAIN, WKB and WKT") { protected void setArg(String value) { geometryEncoding = value; } };
	private static final Option OPT_CRDFMT = new Option("-c", "--coordFormat", "format", "coordinate format to be used, eg. PTV_MERCATOR, OG_GEODECIMAL, ...") { protected void setArg(String value) { coordFormat = CoordFormat.fromString(value); } };
	private static final Option OPT_NOROUTE = new Option("-X", "--noRoute", null, "indicates that no routing should be done") { protected void setArg(String value) { noRoute = true; } };
	private static final Option OPT_NOMAP = new Option("-N", "--noMap", null, "indicates that no images should be generated") { protected void setArg(String value) { noMap = true; } };

	private static Option[] opts = new Option[] {
		OPT_VERBOSE,
		OPT_HELP,
		OPT_NAME,
		OPT_OUTDIR,
		OPT_IMGFMT,
		OPT_IMGSIZE,
		OPT_XLOCATE,
		OPT_GCOPROF,
		OPT_XROUTE,
		OPT_RTGPROF,
		OPT_XMAP,
		OPT_MAPPROF,
//		OPT_GEOM,
		OPT_CRDFMT,
		OPT_NOROUTE,
        OPT_NOMAP
	};

	private static abstract class Option {
		String shortOpt;
		String longOpt;
		String argName;
		String descr;
		int shift = 0;
		public Option(String shortOpt, String longOpt, String argName, String descr) {
			this.shortOpt = shortOpt;
			this.longOpt = longOpt;
			this.argName = argName;
			this.shift = (this.argName == null) ? 1 : 2;
			this.descr = descr;
		}
		public boolean parseArg(String[] args, int offs) {
			if (args.length <= offs + shift - 1)
				return false;
			if (shortOpt.equals(args[offs]) || longOpt.equals(args[offs])) {
				String val;
				if (shift == 1) {
					val = "true";
				} else {
					val = args[offs+1];
				}
				setArg(val);
				if (verbose)
					System.out.println("option " + shortOpt + " set to " + val);
				return true;
			}
			return false;
		}
		protected abstract void setArg(String value);
	}

	static void usage(int exitCode) {
		System.out.println("usage: java " + RouteOnMapDemo.class.getName() + " [options] address+");
		System.out.println("  where options are:");
		for (int i = 0; i < opts.length; i++) {
			Option opt = opts[i];
			String arg = opt.argName == null ? "" : (" <" + opt.argName + ">");
			System.out.println("    " + opt.shortOpt + arg + " | " + opt.longOpt + arg);
			System.out.println("         " + opt.descr);
		}

		System.exit(exitCode);
	}

	public static void main(String[] args) {
		//System.out.println(System.getProperty("java.class.path"));
		int offs = 0;
		while (offs < args.length) {
			boolean found = false;
			for (int i = 0; i < opts.length && !found; i++) {
				found = opts[i].parseArg(args, offs);
				if (found) {
					offs += opts[i].shift;
				}
			}
			if (!found) {
				// argument handling is over
				if (args[offs].startsWith("-")) {
					System.err.println("unexpected option " + args[offs]);
					usage(1);
				}
				if (args.length - offs < 2) {
					System.err.println("must have at least two waypoints but only got " + (args.length - offs));					usage(1);
				}
				waypoints = new Address[args.length - offs];
				for (int i = offs; i < args.length; i++) {
					String addrStr = args[i];
					Address addr = XLocateUtils.makeAddress(addrStr);
					waypoints[i-offs] = addr;
				}
				offs = args.length;
			}
		}
		if (waypoints == null) {
			System.err.println("no waypoints specified");
			usage(1);
		}
		doRouting();
	}

	private static void doRouting() {

		try {

			// 1. create the access clients
			XMapRemoteInterface xmc = XMapUtils.makeClient(xmapUrl);
			XLocateRemoteInterface xlc = XLocateUtils.makeClient(xlocateUrl);
			XRouteRemoteInterface  xrc = XRouteUtils.makeClient(xrouteUrl);

			xmc.setCallerContext(CommonUtils.makeContext(mappingProfile, coordFormat, geometryEncoding, "xserver-sample", "java-client", "ptv"));
			xlc.setCallerContext(CommonUtils.makeContext(locatingProfile, coordFormat, geometryEncoding, "xserver-sample", "java-client", "ptv"));
			xrc.setCallerContext(CommonUtils.makeContext(routingProfile, coordFormat, geometryEncoding, "xserver-sample", "java-client", "ptv"));

			// 2. Geocode the given addresses
			WaypointDesc[] wpts = new WaypointDesc[waypoints.length];
			Point[] stations = new Point[waypoints.length];
			System.out.println("geocoding addresses...");
			for (int i = 0; i < waypoints.length; i++) {
				if (verbose) {
					System.out.println("geocoding " + waypoints[i] + "...");
				}
				ResultAddress[] resAddrs = XLocateUtils.geocodeAddress(xlc, waypoints[i]);
				if (verbose) {
					System.out.println("xlocate returned " + resAddrs.length + " addresses");
					for (int a = 0; a < resAddrs.length; a++) {
						System.out.println("  " + (a+1) + ": " + resAddrs[a]);
					}
				}
				if (resAddrs.length == 0) {
					System.err.println("unable to geocode address " + waypoints[i]);
					System.exit(1);
				}
				stations[i] = resAddrs[0].getCoordinates();
				wpts[i] = XRouteUtils.makeWayPointDesc(stations[i], LinkType.NEXT_SEGMENT);
				System.out.println("waypoint: " + resAddrs[0].getCountry() + "-" + resAddrs[0].getPostCode() + " " + resAddrs[0].getCity() + ", " + resAddrs[0].getStreet());
			}

			Route rt = null;
			if (!noRoute) {
			    RouteInfo ri;
                
				// 2. do the routing
                if (noMap) {
                    ri = XRouteUtils.calcRouteInfos(xrc, wpts, DetailLevel.STANDARD);
                } else {
    				System.out.println("calculating route...");
    				rt = XRouteUtils.calcRoute(xrc, wpts, DetailLevel.STANDARD);
    				ri = rt.getInfo();
                }                
                System.out.println("route information:");
				System.out.println("  distance: " + ri.getDistance() + "m");
				int time = ri.getTime();
				int hours = time / 3600;
				int mins  = (time % 3600) / 60;
				int secs  = (time % 60);
				System.out.println("  time:     " + hours + ":" + mins + ":" + secs);
			}

			if (!noMap) {
    			// 3. draw the rout on the map
    			System.out.println("drawing the map...");
    			Map overview = XMapUtils.renderCenteredRoute(xmc, format, imageWidth, imageHeight, rt != null ? rt.getPolygon() : null, XMapUtils.makeColor(0, 0, 255), stations, true);
    			File outF = new File(outDir, "route-" + routeName + "-" + System.currentTimeMillis() + "." + format.getValue());
    			FileOutputStream fos = new FileOutputStream(outF);
    			fos.write(overview.getImage().getRawImage());
    			fos.close();
    			System.out.println("overview image file is: " + outF.getAbsolutePath());
    
    			Map startmap = XMapUtils.renderCenteredRoute(xmc, format, imageWidth, imageHeight, null, XMapUtils.makeColor(0, 0, 255), new Point[] { stations[0] }, true);
    			outF = new File(outDir, "route-" + routeName + "-" + System.currentTimeMillis() + "." + format.getValue());
    			fos = new FileOutputStream(outF);
    			fos.write(startmap.getImage().getRawImage());
    			fos.close();
    			System.out.println("source image file is: " + outF.getAbsolutePath());
    
    			Map destmap = XMapUtils.renderCenteredRoute(xmc, format, imageWidth, imageHeight, null, XMapUtils.makeColor(0, 0, 255), new Point[] { stations[1] }, true);
    			outF = new File(outDir, "route-" + routeName + "-" + System.currentTimeMillis() + "." + format.getValue());
    			fos = new FileOutputStream(outF);
    			fos.write(destmap.getImage().getRawImage());
    			fos.close();
    			System.out.println("dest image file is: " + outF.getAbsolutePath());
            }
		}
		catch (Exception ex) {
			System.err.println("exception: " + ex);
		}
	}

}
