package main;

import utils.XLocateUtils;
import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

/**
 * This class contains a sample method that demonstrates a simple
 * combined use of xlocate, xroute and xmap.
 * 
 * The best is to let the sample run once to get an impression of
 * what it should do and then step into the debugger to get to know
 * the interfaces and data structures a little better.
 *  
 * @author RN
 */
public class XServerCombinedSample {

	//--------------- the URLs to look for the servers ----------------------
	
	public static final String SERVER = "localhost";
	public static final String XMAP_WS_URL    = "http://" + SERVER + ":50010/xmap/ws/XMap";
	public static final String XLOCATE_WS_URL = "http://" + SERVER + ":50020/xlocate/ws/XLocate";
	public static final String XROUTE_WS_URL  = "http://" + SERVER + ":50030/xroute/ws/XRoute";
	
	//--------------- some sample addresses in luxembourg ----------------------
	
	public static final Address ADDR_COUNTRY_AND_CITY       = XLocateUtils.makeAddress("L~~74080~~~Heilbronn~~");
	public static final Address ADDR_COUNTRY_ZIP_AND_STREET = XLocateUtils.makeAddress("L~~1121~~~Alouettes~~");

	
	public static void runCombinedDemo() {
		XMapRemoteInterface    mc = XMapUtils.makeClient(XMAP_WS_URL);
		XLocateRemoteInterface lc = XLocateUtils.makeClient(XLOCATE_WS_URL);
		XRouteRemoteInterface  rc = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		try {
			System.out.println("geocoding address 1: " + ADDR_COUNTRY_AND_CITY);
            AddressResponse resp = lc.findAddress(ADDR_COUNTRY_AND_CITY, null, null, null);
            ResultAddress[] addrs1 = resp.getResultList(); 

			System.out.println("geocoding address 2: " + ADDR_COUNTRY_ZIP_AND_STREET);
			resp = lc.findAddress(ADDR_COUNTRY_ZIP_AND_STREET, null, null, null);
            ResultAddress[] addrs2 = resp.getResultList(); 
			
			// we assume, everything went right
			ResultAddress startLoc = addrs1[0];
			ResultAddress destLoc = addrs2[0];
			
			System.out.println("routing from ");
			System.out.println("   " + startLoc);
			System.out.println("to");
			System.out.println("   " + destLoc);
			
			Route rt = XRouteUtils.calcRoute(rc, new Point[] { startLoc.getCoordinates(), destLoc.getCoordinates() }, DetailLevel.STANDARD);
			
			System.out.println("route is " + rt.getInfo().getDistance() + "m long and takes " + rt.getInfo().getTime() + "min to go");
			
			Map overview = XMapUtils.renderCenteredRoute(mc, ImageFileFormat.GIF, 800, 600, rt.getPolygon(), XMapUtils.makeColor(0, 0, 255), null, false);
			
			System.out.println("overview map url is: " + overview.getImage().getUrl());
			
		}
		catch (SystemException re) {
			System.err.println("Network or URL problem");
		}
		catch (XRouteException xre) {
			System.err.println("Problem while routing");
		}
		catch (XLocateException xle) {
			System.err.println("Problem rendering map");
		}
		catch (XMapException xme) {
			System.err.println("Problem rendering map");
		}
		catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}

	}

	public static void main(String[] args) {
		runCombinedDemo();
	}

}
