package samples.xcluster;

import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XClusterUtils;
import utils.XMapUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xcluster.DayCluster;
import com.ptvag.xserver.xcluster.Location;
import com.ptvag.xserver.xcluster.MultiWeeksOrder;
import com.ptvag.xserver.xcluster.MultiWeeksParams;
import com.ptvag.xserver.xcluster.MultiWeeksPlan;
import com.ptvag.xserver.xcluster.MultiWeeksRules;
import com.ptvag.xserver.xcluster.MultiWeeksVisit;
import com.ptvag.xserver.xcluster.OutputMultiWeeksOrder;
import com.ptvag.xserver.xcluster.OutputMultiWeeksVisit;
import com.ptvag.xserver.xcluster.WeekValidity;
import com.ptvag.xserver.xcluster.WeekWeight;
import com.ptvag.xserver.xcluster.XClusterException;
import com.ptvag.xserver.xcluster.XClusterRemoteInterface;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative purposes only. These
 * examples have not been thoroughly tested under all conditions. PTV AG,
 * therefore, cannot guarantee or imply reliability, service ability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xCluster Server - Multi-Week Planning
 * 
 * (c) 2014 PTV AG
 */

public class ClusteringPlanMultiWeek {

	public static MultiWeeksPlan runSample(MultiWeeksPlan multiWeeksPlan) {

		// Step 1: Instantiate remote interface for xCluster
		final String url = "http://localhost:50080/xcluster/ws/XCluster";
		XClusterRemoteInterface client = XClusterUtils.makeClient(url);

		// Step 2.1: Set CallerContext for adding log information into your
		// basic
		// response logs.
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Plan Week");
		// Step 2.2 Set CallerContext properties to define profile and
		// CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] {
				props1, props2 };
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 3: create object orders which contain a list of multi-week
		// orders
		Vector<String> orders = new Vector<String>();
		orders = loadOrder();
		int max = orders.size();
		MultiWeeksOrder[] multiWeeksOrders = new MultiWeeksOrder[max];

		Enumeration<String> e = orders.elements();
		int index = 0;
		while (e.hasMoreElements()) {
			String arg0 = e.nextElement();

			Vector c = XClusterUtils.getSubStrings(arg0);
			Enumeration<String> eOrder = c.elements();

			int oid = XClusterUtils.stringtoint(eOrder.nextElement());
			int x = XClusterUtils.stringtoint(eOrder.nextElement());
			int y = XClusterUtils.stringtoint(eOrder.nextElement());
			int wmax = XClusterUtils.stringtoint(eOrder.nextElement());
			Vector<Integer> vecVisits = XClusterUtils.getQuantities(eOrder
					.nextElement());

			// Define location
			Location location = new Location();
			location.setX(x);
			location.setY(y);

			WeekValidity validity1 = new WeekValidity();
			// Define validity1 for week1
			validity1.setDay1(false);
			validity1.setDay2(true);
			validity1.setDay3(true);
			validity1.setDay4(true);
			validity1.setDay5(true);
			validity1.setDay6(true);
			validity1.setDay7(false);

			WeekValidity validity2 = new WeekValidity();
			// Define validity2 for week2
			validity2.setDay1(true);
			validity2.setDay2(true);
			validity2.setDay3(true);
			validity2.setDay4(true);
			validity2.setDay5(true);
			validity2.setDay6(false);
			validity2.setDay7(false);

			WeekValidity[] validities = new WeekValidity[] { validity1,
					validity2 };

			// Define all possible weekday patterns according to the visit
			// rhythm
			WeekValidity[] weekdayPatterns = setWeekdayPatterns(wmax);

			// Set assignmentRules
			MultiWeeksRules assignmentRules = new MultiWeeksRules();
			assignmentRules.setWeekRhythm(2);
			assignmentRules.setValidities(validities);
			assignmentRules.setVisitsPerWeek(wmax);
			assignmentRules.setWeekdayPatterns(weekdayPatterns);
			assignmentRules.setFirstWeekFix(false);

			// Set quantity to multi-week visits
			MultiWeeksVisit[] multiWeeksVisits = new MultiWeeksVisit[wmax];
			Enumeration<Integer> eVisits = vecVisits.elements();
			int oindex = 0;

			while (eVisits.hasMoreElements()) {
				int quantity = eVisits.nextElement();
				MultiWeeksVisit multiWeeksVisit = new MultiWeeksVisit();
				multiWeeksVisit.setQuantity(quantity);
				multiWeeksVisits[oindex] = multiWeeksVisit;
				oindex++;
			}

			// Set multi-week orders
			MultiWeeksOrder order = new MultiWeeksOrder();
			order.setOrderID(oid);
			order.setLoc(location);
			order.setOrderFixed(false);
			order.setVisits(multiWeeksVisits);
			order.setAssignmentRules(assignmentRules);

			multiWeeksOrders[index] = order;
			index++;
		}

		// Step 4: Define number of clusters and specific parameters for both
		// weeks
		MultiWeeksParams multiWeeksParams = new MultiWeeksParams();
		multiWeeksParams.setWeekCount(2);
		multiWeeksParams.setWeekdayCount(6);
		multiWeeksParams.setCostPeriodMinute(400);
		multiWeeksParams.setCostDistanceKm(100);
		multiWeeksParams.setConstructionStep(true);
		multiWeeksParams.setImprovementStep(true);
		multiWeeksParams.setPostOptStep(true);

		WeekWeight weight1 = new WeekWeight();
		weight1.setDay1(20);
		weight1.setDay2(20);
		weight1.setDay3(20);
		weight1.setDay4(20);
		weight1.setDay5(20);
		weight1.setDay6(20);
		weight1.setDay7(0);

		WeekWeight weight2 = new WeekWeight();
		weight2.setDay1(20);
		weight2.setDay2(20);
		weight2.setDay3(20);
		weight2.setDay4(20);
		weight2.setDay5(20);
		weight2.setDay6(20);
		weight2.setDay7(0);

		WeekWeight[] weekWeights = new WeekWeight[] { weight1, weight2 };
		multiWeeksParams.setWeights(weekWeights);

		try {
			// Step 5: Call method planMultiWeeks
			multiWeeksPlan = client.planMultiWeeks(multiWeeksOrders,
					multiWeeksParams);

			createTableOutputMultiWeeksOrder(multiWeeksPlan);
			createTableDayCluster(multiWeeksPlan);

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XClusterException xce) {
			System.err.println("Problem with method planMultiWeeks()");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}

		return multiWeeksPlan;
	}

	private static WeekValidity[] setWeekdayPatterns(int wmax) {
		WeekValidity[] weekdayPatterns = null;

		WeekValidity weekdayPattern1 = new WeekValidity();
		weekdayPattern1.setDay1(true);
		weekdayPattern1.setDay2(false);
		weekdayPattern1.setDay3(false);
		weekdayPattern1.setDay4(false);
		weekdayPattern1.setDay5(false);
		weekdayPattern1.setDay6(false);
		weekdayPattern1.setDay7(false);

		WeekValidity weekdayPattern2 = new WeekValidity();
		weekdayPattern2.setDay1(false);
		weekdayPattern2.setDay2(true);
		weekdayPattern2.setDay3(false);
		weekdayPattern2.setDay4(false);
		weekdayPattern2.setDay5(false);
		weekdayPattern2.setDay6(false);
		weekdayPattern2.setDay7(false);

		WeekValidity weekdayPattern3 = new WeekValidity();
		weekdayPattern3.setDay1(false);
		weekdayPattern3.setDay2(false);
		weekdayPattern3.setDay3(true);
		weekdayPattern3.setDay4(false);
		weekdayPattern3.setDay5(false);
		weekdayPattern3.setDay6(false);
		weekdayPattern3.setDay7(false);

		WeekValidity weekdayPattern4 = new WeekValidity();
		weekdayPattern4.setDay1(false);
		weekdayPattern4.setDay2(false);
		weekdayPattern4.setDay3(false);
		weekdayPattern4.setDay4(true);
		weekdayPattern4.setDay5(false);
		weekdayPattern4.setDay6(false);
		weekdayPattern4.setDay7(false);

		WeekValidity weekdayPattern5 = new WeekValidity();
		weekdayPattern5.setDay1(false);
		weekdayPattern5.setDay2(false);
		weekdayPattern5.setDay3(false);
		weekdayPattern5.setDay4(false);
		weekdayPattern5.setDay5(true);
		weekdayPattern5.setDay6(false);
		weekdayPattern5.setDay7(false);

		WeekValidity weekdayPattern6 = new WeekValidity();
		weekdayPattern6.setDay1(false);
		weekdayPattern6.setDay2(false);
		weekdayPattern6.setDay3(false);
		weekdayPattern6.setDay4(false);
		weekdayPattern6.setDay5(false);
		weekdayPattern6.setDay6(true);
		weekdayPattern6.setDay7(false);

		WeekValidity weekdayPattern7 = new WeekValidity();
		weekdayPattern7.setDay1(true);
		weekdayPattern7.setDay2(false);
		weekdayPattern7.setDay3(true);
		weekdayPattern7.setDay4(false);
		weekdayPattern7.setDay5(false);
		weekdayPattern7.setDay6(false);
		weekdayPattern7.setDay7(false);

		WeekValidity weekdayPattern8 = new WeekValidity();
		weekdayPattern8.setDay1(false);
		weekdayPattern8.setDay2(true);
		weekdayPattern8.setDay3(false);
		weekdayPattern8.setDay4(true);
		weekdayPattern8.setDay5(false);
		weekdayPattern8.setDay6(false);
		weekdayPattern8.setDay7(false);

		WeekValidity weekdayPattern9 = new WeekValidity();
		weekdayPattern9.setDay1(false);
		weekdayPattern9.setDay2(false);
		weekdayPattern9.setDay3(true);
		weekdayPattern9.setDay4(false);
		weekdayPattern9.setDay5(true);
		weekdayPattern9.setDay6(false);
		weekdayPattern9.setDay7(false);

		WeekValidity weekdayPattern10 = new WeekValidity();
		weekdayPattern10.setDay1(false);
		weekdayPattern10.setDay2(false);
		weekdayPattern10.setDay3(false);
		weekdayPattern10.setDay4(true);
		weekdayPattern10.setDay5(false);
		weekdayPattern10.setDay6(true);
		weekdayPattern10.setDay7(false);

		WeekValidity weekdayPattern11 = new WeekValidity();
		weekdayPattern11.setDay1(true);
		weekdayPattern11.setDay2(false);
		weekdayPattern11.setDay3(false);
		weekdayPattern11.setDay4(true);
		weekdayPattern11.setDay5(false);
		weekdayPattern11.setDay6(false);
		weekdayPattern11.setDay7(false);

		WeekValidity weekdayPattern12 = new WeekValidity();
		weekdayPattern12.setDay1(false);
		weekdayPattern12.setDay2(true);
		weekdayPattern12.setDay3(false);
		weekdayPattern12.setDay4(false);
		weekdayPattern12.setDay5(true);
		weekdayPattern12.setDay6(false);
		weekdayPattern12.setDay7(false);

		WeekValidity weekdayPattern13 = new WeekValidity();
		weekdayPattern13.setDay1(false);
		weekdayPattern13.setDay2(false);
		weekdayPattern13.setDay3(true);
		weekdayPattern13.setDay4(false);
		weekdayPattern13.setDay5(false);
		weekdayPattern13.setDay6(true);
		weekdayPattern13.setDay7(false);

		WeekValidity weekdayPattern14 = new WeekValidity();
		weekdayPattern14.setDay1(true);
		weekdayPattern14.setDay2(false);
		weekdayPattern14.setDay3(false);
		weekdayPattern14.setDay4(false);
		weekdayPattern14.setDay5(true);
		weekdayPattern14.setDay6(false);
		weekdayPattern14.setDay7(false);

		WeekValidity weekdayPattern15 = new WeekValidity();
		weekdayPattern15.setDay1(false);
		weekdayPattern15.setDay2(true);
		weekdayPattern15.setDay3(false);
		weekdayPattern15.setDay4(false);
		weekdayPattern15.setDay5(false);
		weekdayPattern15.setDay6(true);
		weekdayPattern15.setDay7(false);

		if (wmax == 1) {
			weekdayPatterns = new WeekValidity[] { weekdayPattern1,
					weekdayPattern2, weekdayPattern3, weekdayPattern4,
					weekdayPattern5, weekdayPattern6 };
		} else {
			weekdayPatterns = new WeekValidity[] { weekdayPattern7,
					weekdayPattern8, weekdayPattern9, weekdayPattern10,
					weekdayPattern11, weekdayPattern12, weekdayPattern13,
					weekdayPattern14, weekdayPattern15 };
		}

		return weekdayPatterns;
	}

	private static void createTableOutputMultiWeeksOrder(
			MultiWeeksPlan multiWeeksPlan) {
		Vector<Object[][]> jMultiWeeksOrder = new Vector<Object[][]>();
		Object[][] rows = new Object[multiWeeksPlan.getOrders().length][3];

		int clindex = 0;

		for (OutputMultiWeeksOrder order : multiWeeksPlan.getOrders()) {
			rows[clindex][0] = "" + order.getOrderID();
			String weekday = "";
			String week = "";

			for (OutputMultiWeeksVisit visit : order.getVisits()) {
				int w = visit.getWeek();
				if (w == 1) {
					week = week + "WW 1";
				} else {
					week = week + "WW 2";
				}
				week = week + "; ";

				int wd = visit.getWeekday();
				if (wd == 1) {
					weekday = weekday + "Monday";
				} else if (wd == 2) {
					weekday = weekday + "Tuesday";
				} else if (wd == 3) {
					weekday = weekday + "Wednesday";
				} else if (wd == 4) {
					weekday = weekday + "Thursday";
				} else if (wd == 5) {
					weekday = weekday + "Friday";
				} else {
					weekday = weekday + "Saturday";
				}
				weekday = weekday + "; ";
			}

			rows[clindex][1] = week;
			rows[clindex][2] = weekday;
			clindex++;
		}

		jMultiWeeksOrder.add(rows);

		// visualize orders assigned to their clusters
		JTable jOutputMultiWeeksOrderTable = new JTable(
				jMultiWeeksOrder.firstElement(), new Object[] { "Order ID",
						"Week", "Weekday" });
		JScrollPane jScrollPaneMultiWeeksOrder = new JScrollPane(
				jOutputMultiWeeksOrderTable);

		JFrame jFrameMultiWeeksOrders = new JFrame("Multi-week orders");
		jFrameMultiWeeksOrders.getContentPane().add(jScrollPaneMultiWeeksOrder);
		jFrameMultiWeeksOrders.setVisible(true);
		jFrameMultiWeeksOrders.setSize(425, 550);
		jFrameMultiWeeksOrders.setLocation(500, 0);
		jFrameMultiWeeksOrders.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static void createTableDayCluster(MultiWeeksPlan multiWeeksPlan) {
		Vector<Object[][]> jDayClusters = new Vector<Object[][]>();
		Object[][] rows = new Object[multiWeeksPlan.getClusters().length][6];

		int clindex = 0;

		for (DayCluster cluster : multiWeeksPlan.getClusters()) {
			rows[clindex][0] = "" + String.format("%.2f", cluster.getLoad());
			rows[clindex][1] = "" + cluster.getQuantity();
			rows[clindex][2] = "" + cluster.getTarget();
			rows[clindex][3] = "" + cluster.getVisitCount();

			int wd = cluster.getWeekday();
			String weekday = "";
			if (wd == 1) {
				weekday = "Monday";
			} else if (wd == 2) {
				weekday = "Tuesday";
			} else if (wd == 3) {
				weekday = "Wednesday";
			} else if (wd == 4) {
				weekday = "Thursday";
			} else if (wd == 5) {
				weekday = "Friday";
			} else {
				weekday = "Saturday";
			}
			rows[clindex][4] = weekday;

			int w = cluster.getWeek();
			String week = "";
			if (w == 1) {
				week = "Week Number 1";
			} else {
				week = "Week Number 2";
			}
			rows[clindex][5] = week;

			clindex++;
		}

		jDayClusters.add(rows);

		// visualize orders assigned to their clusters
		JTable jDayClusterTable = new JTable(jDayClusters.firstElement(),
				new Object[] { "Load [%]", "Total Service Period [min]",
						"Target [min]", "Visit Count", "Weekday", "Week" });
		JScrollPane jScrollPaneDayCluster = new JScrollPane(jDayClusterTable);

		JFrame jFrameDayCluster = new JFrame("Day Clusters");
		jFrameDayCluster.getContentPane().add(jScrollPaneDayCluster);
		jFrameDayCluster.setVisible(true);
		jFrameDayCluster.setSize(600, 255);
		jFrameDayCluster.setLocation(925, 0);
		jFrameDayCluster.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

	private static Vector<String> loadOrder() {
		Vector<String> vector = new Vector<String>();
		// orderId, X, Y, number of visits, potential values (service
		// period(s)
		// and day(s))
		vector.addElement("101;671006;6382294;1;90");
		vector.addElement("102;681812;6375071;1;120");
		vector.addElement("103;682676;6388154;1;90");
		vector.addElement("104;675000;6380000;1;70");
		vector.addElement("105;679143;6374455;2;90~110");
		vector.addElement("106;684621;6374703;1;180");
		vector.addElement("107;684659;6372947;1;90");
		vector.addElement("108;683151;6371248;1;50");
		vector.addElement("109;685669;6371954;1;60");
		vector.addElement("110;683974;6369193;2;60~80");
		vector.addElement("111;683000;6366903;1;90");
		vector.addElement("112;678763;6367800;1;120");
		vector.addElement("113;681397;6369670;1;110");
		vector.addElement("114;678400;6370854;1;90");
		vector.addElement("115;674779;6373188;2;60~45");
		vector.addElement("116;678149;6380180;1;45");
		vector.addElement("117;680440;6381326;1;30");
		vector.addElement("118;665627;6378195;1;90");
		vector.addElement("119;687472;6386122;1;180");
		vector.addElement("120;666773;6381020;2;120~110");
		vector.addElement("121;687136;6382563;1;60");
		vector.addElement("122;665642;6382471;1;30");
		vector.addElement("123;665627;6386365;1;45");
		vector.addElement("124;672957;6388121;1;60");
		vector.addElement("125;677004;6388121;2;80~100");
		vector.addElement("126;682578;6384074;1;90");
		vector.addElement("127;687540;6379111;1;100");
		vector.addElement("128;681661;6384838;1;120");
		vector.addElement("129;687388;6385754;1;130");
		vector.addElement("130;685937;6388121;2;150~130");
		vector.addElement("131;683000;6366903;1;90");
		vector.addElement("132;679763;6369800;1;120");
		vector.addElement("133;683397;6366670;1;110");
		vector.addElement("134;672400;6378854;1;90");
		vector.addElement("135;679779;6370188;2;60~45");
		vector.addElement("136;688149;6382180;1;45");
		vector.addElement("137;681440;6380326;1;30");
		vector.addElement("138;668627;6368195;1;90");
		vector.addElement("139;677472;6384122;1;180");
		vector.addElement("140;676773;6382020;2;120~110");
		vector.addElement("141;689136;6384563;1;60");
		vector.addElement("142;665042;6392471;1;30");
		vector.addElement("143;662627;6382365;1;45");
		vector.addElement("144;675957;6380121;1;60");
		vector.addElement("145;676004;6384121;2;80~100");
		vector.addElement("146;680578;6389074;1;90");
		vector.addElement("147;682540;6359111;1;100");
		vector.addElement("148;688661;6389838;1;120");
		vector.addElement("149;680388;6380754;1;130");
		vector.addElement("150;680937;6380121;2;150~130");

		return vector;
	}

	private static void setCallerContext(XMapRemoteInterface xmapClient) {
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Plan Week");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] {
				props1, props2 };
		cxt.setProperties(properties);
		xmapClient.setCallerContext(cxt);
	}

	private static ImageInfo setImageInfo() {
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(250);
		imageInfo.setWidth(250);
		imageInfo.setImageParameter("");

		return imageInfo;
	}

	private static MapParams setMapParams() {
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		return mapParams;
	}

	private static CustomLayer setCustomLayer() {
		CustomLayer customLayer = new CustomLayer();
		customLayer.setCenterObjects(true);
		customLayer.setDrawPriority(25000);
		customLayer.setName("ObjectLayer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);

		return customLayer;
	}

	private static void runMap(MultiWeeksPlan multiWeeksPlan) {
		// Step 1: Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Set CallerContext
		setCallerContext(xmapClient);

		// Step 3: Create imageInfo and set relevant attributes
		ImageInfo imageInfo = setImageInfo();

		// Step 4: Realize object mapParams showing scale without use of miles.
		MapParams mapParams = setMapParams();

		// Step 5: Define boolean value including image in response
		boolean includeImageInResponse = false;

		// Step 6: Initialize customLayer with the relevant settings
		CustomLayer customLayer = setCustomLayer();

		// Step 7.1: Create bitmap arrays per weekday
		int vertical = 0;
		int horizontal = 0;
		int num = multiWeeksPlan.getClusters().length;

		for (int i = 0; i < num; i++) {
			int max = multiWeeksPlan.getClusters()[i].getVisitCount();
			if (max == 0) {
				continue;
			}
			Bitmap[] bitmapArray = new Bitmap[max];
			Vector<String> orders = loadOrder();
			int index = 0;
			int weeknumber;
			int countClusterDay;
			String weekDay = "";
			if ((i == 0)) {
				weekDay = "Monday-1";
				weeknumber = 1;
				countClusterDay = 1;
			} else if ((i == 1)) {
				weekDay = "Tuesday-1";
				weeknumber = 1;
				countClusterDay = 2;
			} else if ((i == 2)) {
				weekDay = "Wednesday-1";
				weeknumber = 1;
				countClusterDay = 3;
			} else if ((i == 3)) {
				weekDay = "Thursday-1";
				weeknumber = 1;
				countClusterDay = 4;
			} else if ((i == 4)) {
				weekDay = "Friday-1";
				weeknumber = 1;
				countClusterDay = 5;
			} else if ((i == 5)) {
				weekDay = "Saturday-1";
				weeknumber = 1;
				countClusterDay = 6;
			} else if ((i == 6)) {
				weekDay = "Monday-2";
				weeknumber = 2;
				countClusterDay = 1;
			} else if ((i == 7)) {
				weekDay = "Tuesday-2";
				weeknumber = 2;
				countClusterDay = 2;
			} else if ((i == 8)) {
				weekDay = "Wednesday-2";
				weeknumber = 2;
				countClusterDay = 3;
			} else if ((i == 9)) {
				weekDay = "Thursday-2";
				weeknumber = 2;
				countClusterDay = 4;
			} else if ((i == 10)) {
				weekDay = "Friday-2";
				weeknumber = 2;
				countClusterDay = 5;
			} else {
				weekDay = "Saturday-2";
				countClusterDay = 6;
				weeknumber = 2;
			}

			for (OutputMultiWeeksOrder multiWeeksOrder : multiWeeksPlan
					.getOrders()) {

				// Step 7.2: Plot multi-week visits
				for (OutputMultiWeeksVisit outputMultiWeeksVisit : multiWeeksOrder
						.getVisits()) {
					int ww = outputMultiWeeksVisit.getWeek();

					// Step 7.3: Get location from order and fill
					// bitmapArray
					if ((outputMultiWeeksVisit.getWeekday() == countClusterDay)
							&& (ww == weeknumber)) {
						String oid = "" + multiWeeksOrder.getOrderID();
						Enumeration<String> e = orders.elements();

						double x = 0.0;
						double y = 0.0;
						Point point = new Point();
						PlainPoint plainPoint = new PlainPoint();

						while (e.hasMoreElements()) {
							String arg0 = e.nextElement();

							Vector c = XClusterUtils.getSubStrings(arg0);
							Enumeration<String> eOrder = c.elements();
							String orderIndex = eOrder.nextElement();
							if (orderIndex.equals(oid)) {
								x = XClusterUtils.stringtoint(eOrder
										.nextElement());
								y = XClusterUtils.stringtoint(eOrder
										.nextElement());
								plainPoint.setX(x);
								plainPoint.setY(y);
								point.setPoint(plainPoint);
								bitmapArray[index] = new Bitmap();
								bitmapArray[index].setPosition(point);

								if ((countClusterDay == 1)
										|| (countClusterDay == 7)) {
									bitmapArray[index].setName("flagblue.bmp");
									bitmapArray[index].setDescr(oid);
								} else if ((countClusterDay == 2)
										|| (countClusterDay == 8)) {
									bitmapArray[index].setName("flagred.bmp");
									bitmapArray[index].setDescr(oid);
								} else if ((countClusterDay == 3)
										|| (countClusterDay == 9)) {
									bitmapArray[index].setName("flaggreen.bmp");
									bitmapArray[index].setDescr(oid);
								} else if ((countClusterDay == 4)
										|| (countClusterDay == 10)) {
									bitmapArray[index].setName("n_marker1.bmp");
									bitmapArray[index].setDescr(oid);
								} else if ((countClusterDay == 5)
										|| (countClusterDay == 11)) {
									bitmapArray[index].setName("n_marker2.bmp");
									bitmapArray[index].setDescr(oid);
								} else {
									bitmapArray[index].setName("location.bmp");
									bitmapArray[index].setDescr(oid);
								}

								index++;
								// System.out.println("Order: " + orderIndex
								// +
								// " X: " + x +
								// " Y: " + y);
								break;
							}
						}
					}
				}
			}
			// Validate bitmapArray
			System.out.println("Day cluster[" + i + "]: " + bitmapArray.length);
			for (int k = 0; k < max; k++) {
				System.out.println("\tOrder-Id: " + bitmapArray[k].getDescr()
						+ "   X: "
						+ bitmapArray[k].getPosition().getPoint().getX()
						+ "   Y: "
						+ bitmapArray[k].getPosition().getPoint().getY());
			}

			// Step 8.1: Set bitmapOptions
			BitmapOptions bitmapOptions = new BitmapOptions();
			PlainPoint referencePoint = new PlainPoint(20, 20, null);
			bitmapOptions.setReferencePoint(referencePoint);
			bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

			// Step 8.2: Add bitmapArray and bitmapOptions into bitmaps
			Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapArray,
					bitmapOptions) };
			customLayer.setBitmaps(bitmaps);

			// Step 9: Put customLayer into layers
			Layer[] layers = new Layer[] { customLayer };

			try {
				// Step 10: Return map object with its characteristics
				Map map = xmapClient.renderMap(null, mapParams, imageInfo,
						layers, includeImageInResponse);

				// Step 11: Retrieve icon
				String imageUrl = "http://" + map.getImage().getUrl();
				ImageIcon icon = new ImageIcon(new URL(imageUrl));

				// Step 12: Display image in window
				JFrame frame = new JFrame(weekDay);
				frame.setSize(icon.getIconWidth() + 20,
						icon.getIconHeight() + 30);
				frame.getContentPane().add(new JLabel(icon));
				frame.setVisible(true);
				frame.setLocation(horizontal, vertical);
				horizontal = horizontal + 80;
				vertical = vertical + 80;
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			} catch (Exception exc) {
				// proper exception handling: see API for PTV's exception
				// handling!
				exc.printStackTrace();
			}

		}
	}

	public static void main(String[] args) {
		MultiWeeksPlan multiWeeksPlan = null;
		multiWeeksPlan = runSample(multiWeeksPlan);

		if (multiWeeksPlan != null) {
			runMap(multiWeeksPlan);
		}
	}

}