package samples.xcluster;

import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XClusterUtils;
import utils.XMapUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xcluster.Location;
import com.ptvag.xserver.xcluster.OutputWeekOrder;
import com.ptvag.xserver.xcluster.OutputWeekVisit;
import com.ptvag.xserver.xcluster.WeekOrder;
import com.ptvag.xserver.xcluster.WeekParams;
import com.ptvag.xserver.xcluster.WeekPlan;
import com.ptvag.xserver.xcluster.WeekRules;
import com.ptvag.xserver.xcluster.WeekValidity;
import com.ptvag.xserver.xcluster.WeekVisit;
import com.ptvag.xserver.xcluster.WeekdayCluster;
import com.ptvag.xserver.xcluster.XClusterException;
import com.ptvag.xserver.xcluster.XClusterRemoteInterface;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative purposes only. These
 * examples have not been thoroughly tested under all conditions. PTV AG,
 * therefore, cannot guarantee or imply reliability, service ability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xCluster Server - Plan Week
 * 
 * (c) 2014 PTV AG
 */

public class ClusteringPlanWeek {

	public static WeekPlan runSample(WeekPlan weekPlan) {

		// Step 1: Instantiate remote interface for xCluster
		final String url = "http://localhost:50080/xcluster/ws/XCluster";
		XClusterRemoteInterface client = XClusterUtils.makeClient(url);

		// Step 2.1: Set CallerContext for adding log information into your basic
		// response logs.
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Plan Week");
		// Step 2.2 Set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 3: create object orders which contain a list of cluster orders
		// (corresponds to one visit)
		Vector<String> orders = new Vector<String>();
		orders = loadOrder();
		int max = orders.size();
		WeekOrder[] weekOrders = new WeekOrder[max];

		Enumeration<String> e = orders.elements();
		int index = 0;
		while (e.hasMoreElements()) {
			String arg0 = e.nextElement();

			Vector c = XClusterUtils.getSubStrings(arg0);
			Enumeration<String> eOrder = c.elements();

			int oid = XClusterUtils.stringtoint(eOrder.nextElement());
			int x = XClusterUtils.stringtoint(eOrder.nextElement());
			int y = XClusterUtils.stringtoint(eOrder.nextElement());
			int wmax = XClusterUtils.stringtoint(eOrder.nextElement());
			Vector<Integer> vecVisits = XClusterUtils.getQuantities(eOrder.nextElement());

			// Define location
			Location location = new Location();
			location.setX(x);
			location.setY(y);

			// Define validity
			WeekValidity validities = new WeekValidity();
			validities.setDay1(true);
			validities.setDay2(true);
			validities.setDay3(true);
			validities.setDay4(true);
			validities.setDay5(true);
			validities.setDay6(false);
			validities.setDay7(false);

			// Define weekday patterns

			WeekValidity[] weekdayPatterns;

			if (wmax == 1) {
				WeekValidity weekdayPattern1 = new WeekValidity();
				weekdayPattern1.setDay1(true);
				weekdayPattern1.setDay2(false);
				weekdayPattern1.setDay3(false);
				weekdayPattern1.setDay4(false);
				weekdayPattern1.setDay5(false);
				weekdayPattern1.setDay6(false);
				weekdayPattern1.setDay7(false);

				WeekValidity weekdayPattern2 = new WeekValidity();
				weekdayPattern2.setDay1(false);
				weekdayPattern2.setDay2(true);
				weekdayPattern2.setDay3(false);
				weekdayPattern2.setDay4(false);
				weekdayPattern2.setDay5(false);
				weekdayPattern2.setDay6(false);
				weekdayPattern2.setDay7(false);

				WeekValidity weekdayPattern3 = new WeekValidity();
				weekdayPattern3.setDay1(false);
				weekdayPattern3.setDay2(false);
				weekdayPattern3.setDay3(true);
				weekdayPattern3.setDay4(false);
				weekdayPattern3.setDay5(false);
				weekdayPattern3.setDay6(false);
				weekdayPattern3.setDay7(false);

				WeekValidity weekdayPattern4 = new WeekValidity();
				weekdayPattern4.setDay1(false);
				weekdayPattern4.setDay2(false);
				weekdayPattern4.setDay3(false);
				weekdayPattern4.setDay4(true);
				weekdayPattern4.setDay5(false);
				weekdayPattern4.setDay6(false);
				weekdayPattern4.setDay7(false);

				WeekValidity weekdayPattern5 = new WeekValidity();
				weekdayPattern5.setDay1(false);
				weekdayPattern5.setDay2(false);
				weekdayPattern5.setDay3(false);
				weekdayPattern5.setDay4(false);
				weekdayPattern5.setDay5(true);
				weekdayPattern5.setDay6(false);
				weekdayPattern5.setDay7(false);

				weekdayPatterns = new WeekValidity[] { weekdayPattern1, weekdayPattern2, weekdayPattern3, weekdayPattern4,
						weekdayPattern5 };
			} else {
				WeekValidity weekdayPattern1 = new WeekValidity();
				weekdayPattern1.setDay1(true);
				weekdayPattern1.setDay2(false);
				weekdayPattern1.setDay3(true);
				weekdayPattern1.setDay4(false);
				weekdayPattern1.setDay5(false);
				weekdayPattern1.setDay6(false);
				weekdayPattern1.setDay7(false);

				WeekValidity weekdayPattern2 = new WeekValidity();
				weekdayPattern2.setDay1(false);
				weekdayPattern2.setDay2(true);
				weekdayPattern2.setDay3(false);
				weekdayPattern2.setDay4(true);
				weekdayPattern2.setDay5(false);
				weekdayPattern2.setDay6(false);
				weekdayPattern2.setDay7(false);

				WeekValidity weekdayPattern3 = new WeekValidity();
				weekdayPattern3.setDay1(false);
				weekdayPattern3.setDay2(false);
				weekdayPattern3.setDay3(true);
				weekdayPattern3.setDay4(false);
				weekdayPattern3.setDay5(true);
				weekdayPattern3.setDay6(false);
				weekdayPattern3.setDay7(false);

				WeekValidity weekdayPattern4 = new WeekValidity();
				weekdayPattern4.setDay1(false);
				weekdayPattern4.setDay2(true);
				weekdayPattern4.setDay3(false);
				weekdayPattern4.setDay4(false);
				weekdayPattern4.setDay5(true);
				weekdayPattern4.setDay6(false);
				weekdayPattern4.setDay7(false);

				WeekValidity weekdayPattern5 = new WeekValidity();
				weekdayPattern5.setDay1(true);
				weekdayPattern5.setDay2(false);
				weekdayPattern5.setDay3(false);
				weekdayPattern5.setDay4(true);
				weekdayPattern5.setDay5(false);
				weekdayPattern5.setDay6(false);
				weekdayPattern5.setDay7(false);

				WeekValidity weekdayPattern6 = new WeekValidity();
				weekdayPattern6.setDay1(true);
				weekdayPattern6.setDay2(false);
				weekdayPattern6.setDay3(false);
				weekdayPattern6.setDay4(false);
				weekdayPattern6.setDay5(true);
				weekdayPattern6.setDay6(false);
				weekdayPattern6.setDay7(false);

				weekdayPatterns = new WeekValidity[] { weekdayPattern1, weekdayPattern2, weekdayPattern3, weekdayPattern4,
						weekdayPattern5, weekdayPattern6 };
			}

			// Set assignmentRules
			WeekRules assignmentRules = new WeekRules();
			assignmentRules.setValidities(validities);
			assignmentRules.setVisitsPerWeek(wmax);
			assignmentRules.setWeekdayPatterns(weekdayPatterns);

			// Set quantity to weekVisits
			WeekVisit[] weekVisits = new WeekVisit[wmax];
			Enumeration<Integer> eVisits = vecVisits.elements();
			int oindex = 0;
			while (eVisits.hasMoreElements()) {
				int quantity = eVisits.nextElement();
				WeekVisit weekVisit = new WeekVisit();
				weekVisit.setQuantity(quantity);
				weekVisits[oindex] = weekVisit;
				oindex++;
			}

			WeekOrder order = new WeekOrder();
			order.setOrderID(oid);
			order.setLoc(location);
			order.setOrderFixed(false);
			order.setVisits(weekVisits);
			order.setVisitSplits(2);
			order.setAssignmentRules(assignmentRules);

			weekOrders[index] = order;
			index++;
		}

		// Step 4: Define number of clusters and specific parameters
		WeekParams weekParams = new WeekParams();
		weekParams.setCostPeriodMinute(120);
		weekParams.setCostDistanceKm(360);
		weekParams.setConstructionStep(true);
		weekParams.setImprovementStep(true);
		weekParams.setPostOptStep(true);
		weekParams.setWeekdayCount(5);

		try {
			// Step 5: Call method planWeek
			weekPlan = client.planWeek(weekOrders, weekParams);

			createTableOutputWeekOrder(weekPlan);
			createTableWeekdayCluster(weekPlan);

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XClusterException xce) {
			System.err.println("Problem with method planWeek()");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}

		return weekPlan;
	}

	private static void createTableOutputWeekOrder(WeekPlan weekPlan) {
		Vector<Object[][]> jWeekOrders = new Vector<Object[][]>();
		Object[][] rows = new Object[weekPlan.getOrders().length][2];

		int clindex = 0;

		for (OutputWeekOrder order : weekPlan.getOrders()) {
			rows[clindex][0] = "" + order.getOrderID();
			String visits = "";

			for (OutputWeekVisit visit : order.getVisits()) {
				int wd = visit.getWeekday();
				String weekday = "";

				if (wd == 1) {
					weekday = "Monday";
				} else if (wd == 2) {
					weekday = "Tuesday";
				} else if (wd == 3) {
					weekday = "Wednesday";
				} else if (wd == 4) {
					weekday = "Thursday";
				} else {
					weekday = "Friday";
				}
				visits = visits + weekday + ";";
			}

			rows[clindex][1] = visits;
			clindex++;
		}

		jWeekOrders.add(rows);

		// visualize orders assigned to their clusters
		JTable jOutputWeekOrderTable = new JTable(jWeekOrders.firstElement(), new Object[] { "Order ID", "Visits" });
		JScrollPane jScrollPaneClusterOrders = new JScrollPane(jOutputWeekOrderTable);

		JFrame jFrameClusterOrders = new JFrame("Orders");
		jFrameClusterOrders.getContentPane().add(jScrollPaneClusterOrders);
		jFrameClusterOrders.setVisible(true);
		jFrameClusterOrders.setSize(300, 550);
		jFrameClusterOrders.setLocation(625, 0);
		jFrameClusterOrders.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static void createTableWeekdayCluster(WeekPlan weekPlan) {
		Vector<Object[][]> jWeekClusters = new Vector<Object[][]>();
		Object[][] rows = new Object[weekPlan.getClusters().length][5];

		int clindex = 0;

		for (WeekdayCluster cluster : weekPlan.getClusters()) {
			rows[clindex][0] = "" + String.format("%.2f", cluster.getLoad());
			rows[clindex][1] = "" + cluster.getQuantity();
			rows[clindex][2] = "" + cluster.getTarget();
			rows[clindex][3] = "" + cluster.getVisitCount();
			rows[clindex][4] = "" + cluster.getWeekday();

			clindex++;
		}

		jWeekClusters.add(rows);

		// visualize orders assigned to their clusters
		JTable jWeekdayClusterTable = new JTable(jWeekClusters.firstElement(), new Object[] { "Load [%]",
				"Total Service Period [min]", "Target [min]", "Visit Count", "Weekday" });
		JScrollPane jScrollPaneWeekdayCluster = new JScrollPane(jWeekdayClusterTable);

		JFrame jFrameWeekdayCluster = new JFrame("Weekday Clusters");
		jFrameWeekdayCluster.getContentPane().add(jScrollPaneWeekdayCluster);
		jFrameWeekdayCluster.setVisible(true);
		jFrameWeekdayCluster.setSize(400, 150);
		jFrameWeekdayCluster.setLocation(925, 0);
		jFrameWeekdayCluster.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

	private static Vector<String> loadOrder() {
		Vector<String> vector = new Vector<String>();
		// orderId, X, Y, number of week visits, visits values (service period(s)
		// and day(s))
		vector.addElement("101;671006;6382294;1;90");
		vector.addElement("102;681812;6375071;1;120");
		vector.addElement("103;682676;6388154;2;90~80");
		vector.addElement("104;675000;6380000;1;60");
		vector.addElement("105;679143;6374455;1;90");
		vector.addElement("106;684621;6374703;2;180~160");
		vector.addElement("107;684659;6372947;1;90");
		vector.addElement("108;683151;6371248;1;30");
		vector.addElement("109;685669;6371954;2;45~70");
		vector.addElement("110;683974;6369193;1;60");
		vector.addElement("111;683000;6366903;1;90");
		vector.addElement("112;678763;6367800;2;120~90");
		vector.addElement("113;681397;6369670;1;105");
		vector.addElement("114;678400;6370854;1;90");
		vector.addElement("115;674779;6373188;2;60~30");
		vector.addElement("116;678149;6380180;1;45");
		vector.addElement("117;680440;6381326;1;30");
		vector.addElement("118;665627;6378195;2;90~100");
		vector.addElement("119;687472;6386122;1;180");
		vector.addElement("120;666773;6381020;1;120");
		vector.addElement("121;687136;6382563;2;60~50");
		vector.addElement("122;665642;6382471;1;30");
		vector.addElement("123;665627;6386365;1;45");
		vector.addElement("124;672957;6388121;2;60~20");
		vector.addElement("125;677004;6388121;1;75");
		vector.addElement("126;682578;6384074;1;90");
		vector.addElement("127;687540;6379111;2;105~120");
		vector.addElement("128;681661;6384838;1;120");
		vector.addElement("129;687388;6385754;1;135");
		vector.addElement("130;685937;6388121;2;150~110");

		return vector;
	}

	private static void setCallerContext(XMapRemoteInterface xmapClient) {
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Plan Week");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		xmapClient.setCallerContext(cxt);
	}

	private static ImageInfo setImageInfo() {
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(300);
		imageInfo.setWidth(300);
		imageInfo.setImageParameter("");

		return imageInfo;
	}

	private static MapParams setMapParams() {
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		return mapParams;
	}

	private static CustomLayer setCustomLayer() {
		CustomLayer customLayer = new CustomLayer();
		customLayer.setCenterObjects(true);
		customLayer.setDrawPriority(25000);
		customLayer.setName("ObjectLayer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);

		return customLayer;
	}

	private static void runMap(WeekPlan weekPlan) {
		// Step 1: Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Set CallerContext
		setCallerContext(xmapClient);

		// Step 3: Create imageInfo and set relevant attributes
		ImageInfo imageInfo = setImageInfo();

		// Step 4: Realize object mapParams showing scale without use of miles.
		MapParams mapParams = setMapParams();

		// Step 5: Define boolean value including image in response
		boolean includeImageInResponse = false;

		// Step 6: Initialize customLayer with the relevant settings
		CustomLayer customLayer = setCustomLayer();

		// Step 7.1: Create bitmap arrays per weekday
		int vertical = 0;
		int horizontal = 0;
		for (int i = 0; i < 5; i++) {
			int max = weekPlan.getClusters()[i].getVisitCount();
			Bitmap[] bitmapArray = new Bitmap[max];
			Vector<String> orders = loadOrder();
			int index = 0;
			String weekDay = "";
			if (i == 0) {
				weekDay = "Monday";
			} else if (i == 1) {
				weekDay = "Tuesday";
			} else if (i == 2) {
				weekDay = "Wednesday";
			} else if (i == 3) {
				weekDay = "Thursday";
			} else {
				weekDay = "Friday";
			}

			for (OutputWeekOrder weekOrder : weekPlan.getOrders()) {

				// Step 7.2: Define weekdays
				for (OutputWeekVisit outputWeekVisit : weekOrder.getVisits()) {

					// Step 7.3: Get location from order and fill bitmapArray
					if (outputWeekVisit.getWeekday() == i + 1) {
						String oid = "" + weekOrder.getOrderID();
						Enumeration<String> e = orders.elements();
						double x = 0.0;
						double y = 0.0;
						Point point = new Point();
						PlainPoint plainPoint = new PlainPoint();

						while (e.hasMoreElements()) {
							String arg0 = e.nextElement();

							Vector c = XClusterUtils.getSubStrings(arg0);
							Enumeration<String> eOrder = c.elements();
							String orderIndex = eOrder.nextElement();
							if (orderIndex.equals(oid)) {

								x = XClusterUtils.stringtoint(eOrder.nextElement());
								y = XClusterUtils.stringtoint(eOrder.nextElement());
								plainPoint.setX(x);
								plainPoint.setY(y);
								point.setPoint(plainPoint);
								bitmapArray[index] = new Bitmap();
								bitmapArray[index].setPosition(point);

								if (i == 0) {
									bitmapArray[index].setName("flagblue.bmp");
									bitmapArray[index].setDescr("First cluster");
								} else if (i == 1) {
									bitmapArray[index].setName("flagred.bmp");
									bitmapArray[index].setDescr("Second cluster");
								} else if (i == 2) {
									bitmapArray[index].setName("flaggreen.bmp");
									bitmapArray[index].setDescr("Third cluster");
								} else if (i == 3) {
									bitmapArray[index].setName("n_marker1.bmp");
									bitmapArray[index].setDescr("Forth cluster");
								} else {
									bitmapArray[index].setName("n_marker2.bmp");
									bitmapArray[index].setDescr("Fifth cluster");
								}

								index++;
								// System.out.println("Order: " + orderIndex + " X: " + x +
								// " Y: " + y);
								break;
							}
						}
					}
				}
			}
			// Validate bitmapArray
			System.out.println("BitmapArray[" + i + "]: " + bitmapArray.length);
			for (int k = 0; k < max; k++) {
				System.out.println("\tX: " + bitmapArray[k].getPosition().getPoint().getX() + "   Y: "
						+ bitmapArray[k].getPosition().getPoint().getY());
			}

			// Step 8.1: Set bitmapOptions
			BitmapOptions bitmapOptions = new BitmapOptions();
			PlainPoint referencePoint = new PlainPoint(20, 20, null);
			bitmapOptions.setReferencePoint(referencePoint);
			bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

			// Step 8.2: Add bitmapArray and bitmapOptions into bitmaps
			Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapArray, bitmapOptions) };
			customLayer.setBitmaps(bitmaps);

			// Step 9: Put customLayer into layers
			Layer[] layers = new Layer[] { customLayer };

			try {
				// Step 10: Return map object with its characteristics
				Map map = xmapClient.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse);

				// Step 11: Retrieve icon
				String imageUrl = "http://" + map.getImage().getUrl();
				ImageIcon icon = new ImageIcon(new URL(imageUrl));

				// Step 12: Display image in window
				JFrame frame = new JFrame("Cluster " + weekDay);
				frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
				frame.getContentPane().add(new JLabel(icon));
				frame.setVisible(true);
				frame.setLocation(horizontal, vertical);
				horizontal = horizontal + 100;
				vertical = vertical + 100;
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			} catch (Exception exc) {
				// proper exception handling: see API for PTV's exception handling!
				exc.printStackTrace();
			}

		}
	}

	public static void main(String[] args) {
		WeekPlan weekPlan = null;
		weekPlan = runSample(weekPlan);

		if (weekPlan != null) {
			runMap(weekPlan);
		}
	}

}