package samples.xcluster;

import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XClusterUtils;
import utils.XMapUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xcluster.Cluster;
import com.ptvag.xserver.xcluster.ClusterOrder;
import com.ptvag.xserver.xcluster.ClusterParams;
import com.ptvag.xserver.xcluster.ClusterPlan;
import com.ptvag.xserver.xcluster.Location;
import com.ptvag.xserver.xcluster.OutputClusterOrder;
import com.ptvag.xserver.xcluster.XClusterException;
import com.ptvag.xserver.xcluster.XClusterRemoteInterface;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative purposes only. These
 * examples have not been thoroughly tested under all conditions. PTV AG,
 * therefore, cannot guarantee or imply reliability, service ability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xCluster Server - Clustering with quantity
 * 
 * (c) 2014 PTV AG
 */

public class ClusteringWithQuantity {
	// Number of clusters is fixed and used as input plan!

	public static ClusterPlan runSample(ClusterPlan clusterPlan) {

		// Step 1: Instantiate remote interface for xCluster
		final String url = "http://localhost:50080/xcluster/ws/XCluster";
		XClusterRemoteInterface client = XClusterUtils.makeClient(url);

		// Step 2.1: Set CallerContext for adding log information into your basic
		// response logs.
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Clustering with Quantity");
		// Step 2.2 Set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 3: create object orders which contain a list of cluster orders
		// (corresponds to one visit)
		Vector<String> orders = new Vector<String>();
		orders = loadOrder();
		int max = orders.size();
		ClusterOrder[] clusterOrders = new ClusterOrder[max];

		Enumeration<String> e = orders.elements();
		int index = 0;
		while (e.hasMoreElements()) {
			String arg0 = e.nextElement();

			Vector c = XClusterUtils.getSubStrings(arg0);
			Enumeration<String> eOrder = c.elements();

			int oid = XClusterUtils.stringtoint(eOrder.nextElement());
			int x = XClusterUtils.stringtoint(eOrder.nextElement());
			int y = XClusterUtils.stringtoint(eOrder.nextElement());
			int turnover = XClusterUtils.stringtoint(eOrder.nextElement());
			Integer clindex = XClusterUtils.stringtoInteger(eOrder.nextElement());

			ClusterOrder order = new ClusterOrder();
			order.setOrderID(oid);
			Location location = new Location();
			location.setX(x);
			location.setY(y);
			order.setLoc(location);
			order.setOrderFixed(false);
			order.setClusterIndex(clindex); 
			order.setQuantity(turnover); // set quantity such as service period, turnover or specific potential values 

			clusterOrders[index] = order;
			index++;
		}

		// Step 4: Define number of clusters and specific parameters
		ClusterParams clusterParams = new ClusterParams();
		clusterParams.setClusterCount(3); // number of clusters is fixed according to the input
		clusterParams.setCostPeriodMinute(120);
		clusterParams.setCostDistanceKm(360);
		clusterParams.setConstructionStep(false); // previous cluster plan is used
		clusterParams.setImprovementStep(true);
		clusterParams.setPostOptStep(true);

		try {
			// Step 5: Call method planClusters
			clusterPlan = client.planClusters(clusterOrders, clusterParams);
			
			Vector<Object[][]> jVectorClusters = new Vector<Object[][]>();
			Object[][] rows = new Object[clusterPlan.getClusters().length][5];
			
			int clindex = 0;
			
			for (Cluster cluster : clusterPlan.getClusters()) {
				rows[clindex][0] = "" + cluster.getClusterIndex();
				rows[clindex][1] = String.format("%.2f", cluster.getLoad());
				rows[clindex][2] = "" + cluster.getQuantity();
				rows[clindex][3] = "" + cluster.getTarget();
				rows[clindex][4] = "" + cluster.getVisitCount();
				clindex++;
			}
			
			jVectorClusters.add(rows);
			
			// visualize orders assigned to their clusters
			JTable jOutputClusterOrderTable = new JTable(jVectorClusters.firstElement(), new Object[] { "Cluster Index", "Load [%]", "Total []", "Target []", "Visit Count"});
			JScrollPane jScrollPaneClusterOrders = new JScrollPane(jOutputClusterOrderTable);

			JFrame jFrameClusterOrders = new JFrame("Clustering with Quantity");
			jFrameClusterOrders.getContentPane().add(jScrollPaneClusterOrders);
			jFrameClusterOrders.setVisible(true);
			jFrameClusterOrders.setSize(500, 120);
			jFrameClusterOrders.setLocation(625, 0);
			jFrameClusterOrders.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XClusterException xce) {
			System.err.println("Problem with clustering [wrong number]");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}

		return clusterPlan;
	}

	private static Vector<String> loadOrder() {
		Vector<String> vector = new Vector<String>(); // orderId, X, Y, turnover, cluster index
		vector.addElement("101;671006;6382294;90000;1");
		vector.addElement("102;681812;6375071;10000;3");
		vector.addElement("103;682676;6388154;20000;2");
		vector.addElement("104;675000;6380000;10000;1");
		vector.addElement("105;679143;6374455;50000;3");
		vector.addElement("106;684621;6374703;80000;3");
		vector.addElement("107;684659;6372947;20000;3");
		vector.addElement("108;683151;6371248;30000;3");
		vector.addElement("109;685669;6371954;90000;3");
		vector.addElement("110;683974;6369193;40000;3");
		vector.addElement("111;683000;6366903;50000;3");
		vector.addElement("112;678763;6367800;30000;3");
		vector.addElement("113;681397;6369670;10000;3");
		vector.addElement("114;678400;6370854;20000;3");
		vector.addElement("115;674779;6373188;30000;3");
		vector.addElement("116;678149;6380180;50000;2");
		vector.addElement("117;680440;6381326;30000;2");
		vector.addElement("118;665627;6378195;50000;1");
		vector.addElement("119;687472;6386122;30000;2");
		vector.addElement("120;666773;6381020;20000;1");
		vector.addElement("121;687136;6382563;50000;2");
		vector.addElement("122;665642;6382471;30000;1");
		vector.addElement("123;665627;6386365;70000;1");
		vector.addElement("124;672957;6388121;10000;1");
		vector.addElement("125;677004;6388121;20000;1");
		vector.addElement("126;682578;6384074;30000;2");
		vector.addElement("127;687540;6379111;60000;3");
		vector.addElement("128;681661;6384838;20000;2");
		vector.addElement("129;687388;6385754;10000;2");
		vector.addElement("130;685937;6388121;40000;2");

		return vector;
	}

	private static void runMap(ClusterPlan clusterPlan) {
		// Step 1: Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Set CallerContext
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Clustering with Quantity");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		xmapClient.setCallerContext(cxt);

		// Step 3: Create imageInfo and set relevant attributes
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(600);
		imageInfo.setWidth(600);
		imageInfo.setImageParameter("");

		// Step 4: Realize object mapParams showing scale without use of miles.
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 5: Create a bitmap array per cluster
		int max = clusterPlan.getOrders().length;

		Bitmap[] bitmapArray = new Bitmap[max];
		int index = 0;
		Vector<String> orders = loadOrder();

		for (OutputClusterOrder clusterOrder : clusterPlan.getOrders()) {
			String oid = "" + clusterOrder.getOrderID();
			// System.out.println("Indeces: " + cid + " ; " + oid);

			Enumeration<String> e = orders.elements();
			double x = 0.0;
			double y = 0.0;
			Point point = new Point();
			PlainPoint plainPoint = new PlainPoint();

			while (e.hasMoreElements()) {
				String arg0 = e.nextElement();

				Vector c = XClusterUtils.getSubStrings(arg0);
				Enumeration<String> eOrder = c.elements();
				String orderIndex = eOrder.nextElement();

				if (orderIndex.equals(oid)) {
					// System.out.println(orderIndex + " ; " + oid);
					x = XClusterUtils.stringtoint(eOrder.nextElement());
					y = XClusterUtils.stringtoint(eOrder.nextElement());
					plainPoint.setX(x);
					plainPoint.setY(y);
					point.setPoint(plainPoint);
					bitmapArray[index] = new Bitmap();
					bitmapArray[index].setPosition(point);

					int cli = clusterOrder.getClusterIndex();
					if (cli == 1) {
						bitmapArray[index].setName("flagblue.bmp");
						bitmapArray[index].setDescr("First cluster");
					} else if (cli == 2) {
						bitmapArray[index].setName("flagred.bmp");
						bitmapArray[index].setDescr("Second cluster");
					} else if (cli == 3) {
						bitmapArray[index].setName("flaggreen.bmp");
						bitmapArray[index].setDescr("Third cluster");
					} else if (cli == 4) {
						bitmapArray[index].setName("n_marker1.bmp");
						bitmapArray[index].setDescr("Forth cluster");
					} else if (cli == 5) {
						bitmapArray[index].setName("n_marker2.bmp");
						bitmapArray[index].setDescr("Fifth cluster");
					} else {
						bitmapArray[index].setName("location.bmp");
						bitmapArray[index].setDescr("Not defined");
					}

					index++;
					// System.out.println("Order: " + orderIndex + " X: " + x + " Y: " +
					// y);
					break;
				}
			}
		}

		// Step 6: Set bitmapOptions
		BitmapOptions bitmapOptions = new BitmapOptions();
		// change the reference point for the pixel position of the bitmap
		// by default the bitmap center will be used.
		// here: bitmap displacement of 20 pixels.
		PlainPoint referencePoint = new PlainPoint(20, 20, null);
		bitmapOptions.setReferencePoint(referencePoint);
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 7: Add bitmapArray and bitmapOptions into bitmaps
		Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapArray, bitmapOptions) };

		// Step 8: Initialize customLayer with the relevant settings
		CustomLayer customLayer = new CustomLayer();
		customLayer.setCenterObjects(true);
		customLayer.setDrawPriority(25000);
		customLayer.setName("ObjectLayer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setBitmaps(bitmaps);

		// Step 9: Put customLayer into layers
		Layer[] layers = new Layer[] { customLayer };

		// Step 10: Define boolean value including image in response
		boolean includeImageInResponse = false;

		try {

			// Step 11: Return map object with its characteristics
			Map map = xmapClient.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse);

			// Step 12: Retrieve icon
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));

			// Step 13: Display image in window
			JFrame frame = new JFrame("Sample Clustering with Quantity");
			frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (Exception exc) {
			// proper exception handling: see API for PTV's exception handling!
			exc.printStackTrace();
		}

	}

	public static void main(String[] args) {
		ClusterPlan clusterPlan = null;
		clusterPlan = runSample(clusterPlan);

		if (clusterPlan != null) {
			runMap(clusterPlan);
		}
	}

}