/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xdima;

/*
 * PTV xServer Code Samples
 * 
 * PT xDima Server - Calculate distance table
 * 
 * (c) 2012 PTV AG
 */
import utils.CommonUtils;
import utils.XDimaUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xdima.Location;
import com.ptvag.xserver.xdima.XDimaException;
import com.ptvag.xserver.xdima.XDimaRemoteInterface;

public class DimaCalculateDistanceTable {

	private static void runSample() {

		// Step 1: Initialize PTV xDima client using the client factory in package
		// utils.
		String URL = "http://localhost:50070/xdima/ws/XDima";
		XDimaRemoteInterface xDimaClient = XDimaUtils.makeClient(URL);

		// Step 2: Define caller context and also use the package utils.
		CallerContext ctx = CommonUtils.makeContext("default", "PTV GROUP", "PTV xDima Server", "calculateDistanceTable()");
		xDimaClient.setCallerContext(ctx);

		// Step 3: Set the unique dima ID to catch the correct dima profile.
		int dimaID = 1001;

		// Step 4.1: Define 5 locations according to the dima profile.
		// Location A
		Location location_1 = new Location();
		int x1 = 681108;
		int y1 = 6371953;
		location_1.setX(x1);
		location_1.setY(y1);

		// Location B
		Location location_2 = new Location();
		int x2 = 681379;
		int y2 = 6372672;
		location_2.setX(x2);
		location_2.setY(y2);

		// Location C
		Location location_3 = new Location();
		int x3 = 681722;
		int y3 = 6372326;
		location_3.setX(x3);
		location_3.setY(y3);

		// Location D
		Location location_4 = new Location();
		int x4 = 682355;
		int y4 = 6372266;
		location_4.setX(x4);
		location_4.setY(y4);

		// Location E
		Location location_5 = new Location();
		int x5 = 681936;
		int y5 = 6371800;
		location_5.setX(x5);
		location_5.setY(y5);

		// Step 4.2: Add all locations into array locations
		Location[] locations = new Location[] { location_1, location_2, location_3, location_4, location_5 };

		try {
			// Step 5: Call method calculateDistanceTable()
			System.out.println("Calculating Dima is in progress...");
			xDimaClient.calculateDistanceTable(dimaID, locations);
			System.out
					.println("Dima <ID=1001> is calculated. Export Dima afterwards to receive the corressponding distances and periods.");
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XDimaException xse) {
			System.err.println("Problem creating dima profile");
			System.out.println(xse.getMessage());
		} catch (XServiceException xsse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xsse);
		}
	}

	public static void main(String[] args) {
		runSample();
	}
}
