/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xdima;

/*
 * PTV xServer Code Samples
 * 
 * PT xDima Server - Create Dima
 * 
 * (c) 2012 PTV AG
 */
import utils.CommonUtils;
import utils.XDimaUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xdima.CountryInfo;
import com.ptvag.xserver.xdima.DimaCoordFormat;
import com.ptvag.xserver.xdima.DimaProfile;
import com.ptvag.xserver.xdima.LinkType;
import com.ptvag.xserver.xdima.RoutingType;
import com.ptvag.xserver.xdima.VehicleType;
import com.ptvag.xserver.xdima.XDimaException;
import com.ptvag.xserver.xdima.XDimaRemoteInterface;

public class DimaCreate {

	private static void runSample() {

		// Step 1: Initialize PTV xDima client using the client factory in package
		// utils.
		String URL = "http://localhost:50070/xdima/ws/XDima";
		XDimaRemoteInterface xDimaClient = XDimaUtils.makeClient(URL);

		// Step 2: Define caller context and also use the package utils.
		CallerContext ctx = CommonUtils.makeContext("default", "PTV GROUP", "PTV xDima Server", "createDima()");
		xDimaClient.setCallerContext(ctx);

		// Step 3: Create an unique dima ID
		int dimaID = 1001;

		// Step 4.1: Define a new dima profile and set a bundle of parameters
		DimaProfile dimaProfile = new DimaProfile();
		String description = "PTV code sample";
		dimaProfile.setDescription(description);

		// Step 4.2: Set countryCodes
		short[] countryCodes = new short[1];
		countryCodes[0] = 52;
		CountryInfo countryInfo = new CountryInfo();
		countryInfo.setCountryCodes(countryCodes);
		dimaProfile.setCountryInfo(countryInfo);

		// Step 4.3: Set the correct coordinate format
		dimaProfile.setDimaCoordFormat(DimaCoordFormat.MERCATOR);

		// Step 4.4: Avoid or prefer some elements
		short ferryMalus = 100;
		dimaProfile.setFerryMalus(ferryMalus);
		short highwayMalus = 0;
		dimaProfile.setHighwayMalus(highwayMalus);
		short sideRoadMalus = 0;
		dimaProfile.setSideRoadMalus(sideRoadMalus);
		short urbanAreaMalus = 500;
		dimaProfile.setUrbanAreaMalus(urbanAreaMalus);
		short residentsOnlyMalus = 2000;
		dimaProfile.setResidentsOnlyMalus(residentsOnlyMalus);
		short rampMalus = 10;
		dimaProfile.setRampMalus(rampMalus);
		short tollRoadMalus = 0;
		dimaProfile.setTollRoadMalus(tollRoadMalus);

		// Step 4.5: Set the existing vehicle type
		dimaProfile.setVehicleType(VehicleType.VTCAR);
		
		// Step 4.6: Set the maximum number of locations
		short maxLocationCount = 5;
		dimaProfile.setMaxLocationCount(maxLocationCount);
		
		// Step 4.7: Select the linking type
		dimaProfile.setLinkType(LinkType.LTSEGMENT);
	
		// Step 4.8: Select the routing type
		dimaProfile.setRoutingType(RoutingType.RT1TON);
		
		// Step 4.9: Add the speed profile as 8 pairs for maximum and minimum speed of 8 network classes.
		String speedTable = "135,70,125,35,85,25,60,25,50,20,40,18,16,9,6,4";
		dimaProfile.setSpeedTable(speedTable);
		
		// Step 4.10 Set the value for the optimization factor
		short optimization = 80;
		dimaProfile.setOptimization(optimization);

		try {
			// Step 5: Call method createDima()
			System.out.println("Start to create Dima...");
			xDimaClient.createDima(dimaID, dimaProfile);
			
			// Step 6: Dima is created and lies in the corresponding folder of PTV xDima Server.  
			System.out.println("Dima <ID=1001> is created. Check your dima folder for input.");
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XDimaException xse) {
			System.err.println("Problem creating dima profile");
			System.out.println(xse.getMessage());
		} catch (XServiceException xsse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xsse);
		}
	}

	public static void main(String[] args) {
		runSample();
	}
}
