/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xdima;

/*
 * PTV xServer Code Samples
 * 
 * PT xDima Server - Get Dima profile
 * 
 * (c) 2012 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.CommonUtils;
import utils.XDimaUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xdima.DimaProfileInfo;
import com.ptvag.xserver.xdima.XDimaException;
import com.ptvag.xserver.xdima.XDimaRemoteInterface;

public class DimaGetProfile {

	private static Object[][] runSample(Object rows[][]) {
		// Step 1: Instantiate xDima client.
		String URL = "http://127.0.0.1:50070/xdima/ws/XDima";
		XDimaRemoteInterface xDimaClient = XDimaUtils.makeClient(URL);

		// Step 2: Define caller context. Use package utils to add the existing elements properly.
		CallerContext ctx = CommonUtils.makeContext("default", "PTV GROUP", "PTV xDima Server",
				"getDimaProfile()");
		xDimaClient.setCallerContext(ctx);

		// Step 3: Set existing dimaID or zero
		int dimaID = 0;
		//int dimaID = 1001;

		try {
			// Step 4: Call method getDimaProfile()
			DimaProfileInfo[] dimaProfileInfos = xDimaClient.getDimaProfile(dimaID);

			// Step 5: Create result table with the corresponding output.
			rows = new Object[dimaProfileInfos.length][10];
			int rowIdx = 0;

			if (dimaProfileInfos[0] != null) {
				System.out.println("Output of dima profile");
				for (int i = 0; i < dimaProfileInfos.length; i++) {

					rows[rowIdx++] = new Object[] { dimaProfileInfos[i].getDimaID(),
							dimaProfileInfos[i].getDimaProfile().getVehicleType().getValue(),
							dimaProfileInfos[i].getDimaProfile().getMaxLocationCount(),
							dimaProfileInfos[i].getDimaProfile().getSpeedTable(),
							dimaProfileInfos[i].getDimaProfile().getOptimization(),
							dimaProfileInfos[i].getDimaProfile().getFerryMalus(),
							dimaProfileInfos[i].getDimaProfile().getTollRoadMalus(),
							dimaProfileInfos[i].getDimaProfile().getRampMalus(),
							dimaProfileInfos[i].getDimaProfile().getLinkType().getValue(),
							dimaProfileInfos[i].getDimaProfile().getRoutingType().getValue() };
				}
			} else {
				System.out.println("Getting dimaProfileInfos failed.");
			}
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XDimaException xse) {
			System.err.println("Problem getting dima profile");
			System.out.println(xse.getMessage());
		} catch (XServiceException xsse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xsse);
		}
		return rows;
	}

	public static void main(String[] args) {
		Object rows[][] = new Object[1][1];
		rows = runSample(rows);

		// display table in window
		JFrame frame = new JFrame("Sample Emsission Calculation");
		frame.setSize(800, 165);
		JTable table = new JTable(rows, new Object[] { "DimaID", "Vehicle Type", "MAX Location", "Speed Table",
				"Optimization", "Ferry Malus", "Toll Road Malus", "Ramp Malus", "Linking Type", "Routing Type" });
		JScrollPane scrollPane1 = new JScrollPane(table);
		frame.getContentPane().add(scrollPane1);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
}
