/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xdima;

/*
 * PTV xServer Code Samples
 * 
 * PT xDima Server - Set distance and period
 * 
 * (c) 2012 PTV AG
 */
import utils.CommonUtils;
import utils.XDimaUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xdima.DistPeriod;
import com.ptvag.xserver.xdima.Location;
import com.ptvag.xserver.xdima.Relation;
import com.ptvag.xserver.xdima.RelationValue;
import com.ptvag.xserver.xdima.XDimaException;
import com.ptvag.xserver.xdima.XDimaRemoteInterface;

public class DimaSetDistPeriod {

	private static void runSample() {

		// Step 1: Initialize PTV xDima client using the client factory in package
		// utils.
		String URL = "http://localhost:50070/xdima/ws/XDima";
		XDimaRemoteInterface xDimaClient = XDimaUtils.makeClient(URL);

		// Step 2: Define caller context and also use the package utils.
		CallerContext ctx = CommonUtils.makeContext("default", "PTV GROUP", "PTV xDima Server", "setDistPeriod()");
		xDimaClient.setCallerContext(ctx);

		// Step 3: Set the unique dima ID to catch the correct dima profile.
		int dimaID = 1001;

		// Step 4: Define 2 relations according to the existing location lists. Add them to array relationValues.
		Location startLocation = new Location();
		int x1 = 681108;
		int y1 = 6371953;
		startLocation.setX(x1);
		startLocation.setY(y1);

		Location destinationLocation = new Location();
		int x2 = 681379;
		int y2 = 6372672;
		destinationLocation.setX(x2);
		destinationLocation.setY(y2);

		RelationValue relationValue1 = new RelationValue();
		DistPeriod distPeriod1 = new DistPeriod();
		distPeriod1.setDist(99999);
		distPeriod1.setPeriod(5555);
		relationValue1.setDistPeriod(distPeriod1);
		Relation relation_1 = new Relation();
		relation_1.setStartLocation(startLocation);
		relation_1.setDestinationLocation(destinationLocation);
		relationValue1.setRelation(relation_1);
				
		RelationValue relationValue2 = new RelationValue();
		DistPeriod distPeriod2 = new DistPeriod();
		distPeriod2.setDist(88888);
		distPeriod2.setPeriod(4444);
		relationValue2.setDistPeriod(distPeriod2);
		Relation relation_2 = new Relation();
		relation_2.setStartLocation(destinationLocation);
		relation_2.setDestinationLocation(startLocation);
		relationValue2.setRelation(relation_2);

		RelationValue[] relationValues = new RelationValue[] {relationValue1, relationValue2};
		
		try {
			// Step 5: Call method setDistPeriod()
			System.out.println("Start to modify distance and period of relations...");
			xDimaClient.setDistPeriod(dimaID, relationValues);
			System.out
					.println("Dima <ID=1001> is used for modification. Export Dima afterwards to evaluate the altered distances and periods.");
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XDimaException xse) {
			System.err.println("Problem creating dima profile");
			System.out.println(xse.getMessage());
		} catch (XServiceException xsse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xsse);
		}
	}

	public static void main(String[] args) {
		runSample();
	}
}
