package samples.xlocate.advanced;

import utils.XLocateUtils;

import com.ptvag.xserver.xlocate.AdditionalField;
import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.ResultField;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchOptionBase;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find address using additional fields
 * 
 * (c) 2014 PTV AG
 */
public class GeocodeWithAdditionalFields {

	public static void runSample() {
		
		final String URL = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(URL);

		// define new Address
		final Address ZIP_ADDR_STREET = XLocateUtils.makeAddress("~~1856~Luxembourg~~Rue Evrard Ketten~");
		
		// define SearchOption (set COUNTRY_CODETYPE -> 2)
		// makes 3-letter ISO codes default 
		SearchOption soCountryCodeType = new SearchOption();
		soCountryCodeType.setParam(SearchParameter.COUNTRY_CODETYPE);
		soCountryCodeType.setValue("2");	
		
		// set result language to English 
		SearchOption soResultLanguage = new SearchOption();
		soResultLanguage.setParam(SearchParameter.RESULT_LANGUAGE);
		soResultLanguage.setValue("GB");	
		
		// create a new array of SearchOptions that contains our SearchOption
		SearchOptionBase[] options = new SearchOptionBase[]{ soCountryCodeType, soResultLanguage};

		AdditionalField addfield = new AdditionalField(ResultField.DETAILLEVEL, "");
		ResultField resfield1 = addfield.getField();
		addfield = new AdditionalField(ResultField.DETAILLEVEL_DESCRIPTION, "");
		ResultField resfield2 = addfield.getField();
		addfield = new AdditionalField(ResultField.SCORE_TOTALSCORE, "");
		ResultField resfield3 = addfield.getField();
		addfield = new AdditionalField(ResultField.CLASSIFICATION, "");
		ResultField resfield4 = addfield.getField();
		addfield = new AdditionalField(ResultField.CLASSIFICATION_DESCRIPTION, "");
		ResultField resfield5 = addfield.getField();

		ResultField[] resfield = {resfield1,
				resfield2, 	resfield3,
				resfield4,	resfield5};
		
		
		try {
			AddressResponse response =  xLocateClient.findAddress(ZIP_ADDR_STREET, options, null, resfield);

			for (ResultAddress resAddr : response.getResultList()) {
				System.out.println( String.format
				("Country: %s, City: %s, Street: %s,\n \tDetail Level: %s, Detail Level Description: %s, Total Score: %s, Classification: %s, Classification Description: %s", 
								resAddr.getCountry(),
								resAddr.getCity(),
								resAddr.getStreet(),
								resAddr.getAdditionalFields()[0].getValue(),
								resAddr.getAdditionalFields()[1].getValue(),
								resAddr.getAdditionalFields()[2].getValue(),
								resAddr.getAdditionalFields()[3].getValue(),
								resAddr.getAdditionalFields()[4].getValue()
						)
					);
			}

		} catch (Exception e) {
			// TODO proper exception handling 
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		runSample();
	}

}
;
