package samples.xlocate.advanced;

import java.util.Locale;

import utils.XLocateUtils;

import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchOptionBase;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find address selected by country
 * 
 * (c) 2014 PTV AG
 */
public class GeocodeWithCountryCodes {

	public static void runSample() {
		
		final String XLOCATE_URL = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(XLOCATE_URL);
		
		// define new Address
		final Address ZIP_ADDR_STREET = XLocateUtils.makeAddress("~~1856~Luxembourg~~Rue Evrard Ketten~");
		
		// define SearchOption (set COUNTRY_CODETYPE -> 2)
		// makes 3-letter ISO codes default 
		
		SearchOption soCountryCodeType = new SearchOption();
		soCountryCodeType.setParam(SearchParameter.COUNTRY_CODETYPE);
		soCountryCodeType.setValue("2");	
		
		// set result language to Swedish  
		SearchOption soResultLanguage = new SearchOption();
		soResultLanguage.setParam(SearchParameter.RESULT_LANGUAGE);
		soResultLanguage.setValue("SWE");	
		
		// create a new array of SearchOptions that contains our SearchOption
		SearchOptionBase[] options = new SearchOptionBase[]{soCountryCodeType, soResultLanguage};

		try {
			AddressResponse response =  xLocateClient.findAddress(ZIP_ADDR_STREET, options, null, null);

			for (ResultAddress resAddr : response.getResultList()) {
				
				System.out.println( String.format(Locale.ENGLISH, "%s,  %s,  %s, x=%.2f, y=%.2f", 		
								resAddr.getCountry(),
								resAddr.getCity(),
								resAddr.getStreet(),
								resAddr.getCoordinates().getPoint().getX(),
								resAddr.getCoordinates().getPoint().getY()
						)
					);
			}

		} catch (Exception e) {
			// TODO proper exception handling 
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		runSample();
	}

}
;
