package samples.xlocate.advanced;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.jabba.service.baseservices.ClientFactory;
import com.ptvag.jabba.service.baseservices.RemoteType;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find address using various search options
 * 
 * (c) 2014 PTV AG
 */
public class GeocodingWithSearchOptions {

	public static void runSample() {

		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface client = (XLocateRemoteInterface) ClientFactory.createClient(XLocateRemoteInterface.class,
				RemoteType.DOCSTYLE_CXF, "", "", url);

		// Step 2.1: Set CallerContext for adding log information into your basic response logs.
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Basic Geocoding");
		// Step 2.2: Set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[]{props1, props2};
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 3: Create address object
		Address addr = new Address();
		addr.setCountry("LUX");
		addr.setPostCode("1856");
		addr.setCity("Luxemburg");
		addr.setStreet("Rue Evrard Ketten");
		addr.setHouseNumber("40");
		
		// Step 4: Create SearchOption and use corresponding parameters
		SearchOption soCountryCodeType = new SearchOption();
		soCountryCodeType.setParam(SearchParameter.COUNTRY_CODETYPE);
		soCountryCodeType.setValue("2");
		
		SearchOption soResultLanguage = new SearchOption();
		soResultLanguage.setParam(SearchParameter.RESULT_LANGUAGE);
		soResultLanguage.setValue("FRE");
		
		SearchOption[] options = new SearchOption[]{soCountryCodeType, soResultLanguage};

		try {
			// Step 5: Get address response
			AddressResponse response = client.findAddress(addr, options, null, null);

			for (ResultAddress resAddr : response.getResultList()) {
				System.out.println(String.format("%s~%s~%s~%s~%s~location:(%.2f,%.2f)~total score:%s", resAddr.getCountry(),
						resAddr.getPostCode(), resAddr.getCity(), resAddr.getStreet(), resAddr.getHouseNumber(), resAddr
								.getCoordinates().getPoint().getX(), resAddr.getCoordinates().getPoint().getY(),
						resAddr.getTotalScore()));
			}

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem geocoding address");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
	}

	public static void main(String[] args) {
		runSample();
	}

}