package samples.xlocate.basic;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.AdditionalField;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.ResultField;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find address candidates by single field search
 * 
 * (c) 2014 PTV AG
 */

public class FindAddressByText {

	public static void main(String[] args) {
		AddressResponse resultList = findAddressByText();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "Country", "Postcode", "City", "District", "Street", "Location", "Count",
				"Unmatched Words" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find Address by Text");
		frame.setSize(600, 100);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(AddressResponse resultList) {
		int max = resultList.getResultList().length;
		Object rows[][] = new Object[max][8];

		int index = 0;
		for (ResultAddress address : resultList.getResultList()) {
			rows[index][0] = address.getCountry();
			rows[index][1] = address.getPostCode();
			rows[index][2] = address.getCity();
			rows[index][3] = address.getCity2();

			String street = address.getStreet();
			street = street + " " + address.getHouseNumber();
			rows[index][4] = street;

			String coords = "" + String.format("%.0f", address.getCoordinates().getPoint().getX());
			coords = coords + ";" + String.format("%.0f", address.getCoordinates().getPoint().getY());
			rows[index][5] = coords;
			rows[index][6] = address.getAdditionalFields()[0].getValue();
			rows[index][7] = address.getAdditionalFields()[1].getValue();
			index++;
		}

		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Geocoding");
		ctx.setProperties(properties);

		return ctx;
	}

	private static SearchOption[] getSearchOptions() {
		SearchOption separator = new SearchOption();
		separator.setParam(SearchParameter.SINGLE_FIELD_SEPARATORS);
		separator.setValue("");
		// separator.setValue(";,/~");

		SearchOption[] options = new SearchOption[] { separator };

		return options;
	}

	private static ResultField[] getResultFields() {
		AdditionalField unmatchedWordsCount = new AdditionalField();
		unmatchedWordsCount.setField(ResultField.UNMATCHED_WORDS_COUNT);
		unmatchedWordsCount.setValue("");

		AdditionalField unmatchedWords = new AdditionalField();
		unmatchedWords.setField(ResultField.UNMATCHED_WORDS);
		unmatchedWords.setValue("");

		ResultField resField1 = unmatchedWordsCount.getField();
		ResultField resField2 = unmatchedWords.getField();

		ResultField[] resultFields = new ResultField[] { resField1, resField2 };
		return resultFields;
	}

	private static AddressResponse findAddressByText() {
		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set Address
		String address = "Luxemburg 2310 Avenue Pasteur 42-64";
		// Please note: It is not permitted to set a separator between City and
		// postal code.
		// String address = "Luxemburg 2310; Avenue Pasteur; 42-64";
		String country = "L";

		// Step 4: Specific search options
		SearchOption[] options = getSearchOptions();

		// Step 5: Define parameters for the final list
		ResultField[] resultFields = getResultFields();

		// Step 6: Call findAddressByText()
		AddressResponse resultList = null;
		try {
			resultList = xLocateClient.findAddressByText(address, country, options, null, resultFields);
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with geocoding");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return resultList;
	}
}