package samples.xlocate.basic;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.Address;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find address candidates
 * 
 * (c) 2014 PTV AG
 */

public class FindAddresses {

	public static void main(String[] args) {
		AddressResponse[] resultList = findAddresses();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "Index", "Country", "Postcode", "City", "District", "Street", "Location" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find Addresses");
		frame.setSize(700, 200);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(AddressResponse[] resultList) {
		int max = resultList.length;
		int kmax = 0;
		for (int i = 0; i < max; i++) {
			kmax = kmax + resultList[i].getResultList().length;
		}
		Object rows[][] = new Object[kmax][7];

		int index = 0;
		for (int i = 0; i < max; i++) {
			for (ResultAddress address : resultList[i].getResultList()) {
				rows[index][0] = "" + (i+1);
				rows[index][1] = address.getCountry();
				rows[index][2] = address.getPostCode();
				rows[index][3] = address.getCity();
				rows[index][4] = address.getCity2();

				String street = address.getStreet();
				street = street + " " + address.getHouseNumber();
				rows[index][5] = street;

				String coords = "" + String.format("%.0f", address.getCoordinates().getPoint().getX());
				coords = coords + ";" + String.format("%.0f", address.getCoordinates().getPoint().getY());
				rows[index][6] = coords;
				index++;
			}
		}
		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Geocoding");
		ctx.setProperties(properties);

		return ctx;
	}

	private static Address setAddress(String country, String postcode, String city, String street, String hno) {
		Address addr = new Address();
		addr.setCountry(country);
		addr.setPostCode(postcode);
		addr.setCity(city);
		addr.setStreet(street);
		addr.setHouseNumber(hno);

		return addr;
	}

	private static AddressResponse[] findAddresses() {
		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set addresses
		Address addr1 = setAddress("L", "2310", "Luxemburg", "Avenue Pasteur", "42-64");
		Address addr2 = setAddress("L", "2310", "Luxemburg", "Avenue Pasteur", "59-79");
		Address addr3 = setAddress("L", "2310", "Luxemburg", "Avenue Pasteur", "130-136");
		Address addr4 = setAddress("L", "2310", "Luxemburg", "Avenue Pasteur", "140-152");

		Address[] addresses = new Address[] { addr1, addr2, addr3, addr4 };

		// Step 4: Call findAddresses()
		AddressResponse[] resultList = null;
		try {
			resultList = xLocateClient.findAddresses(addresses, null, null, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with geocoding");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return resultList;
	}
}