package samples.xlocate.basic;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.CommonUtils;
import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.AdditionalField;
import com.ptvag.xserver.xlocate.CombinedTransportResponse;
import com.ptvag.xserver.xlocate.Location;
import com.ptvag.xserver.xlocate.PoiLocation;
import com.ptvag.xserver.xlocate.ResultCombinedTransport;
import com.ptvag.xserver.xlocate.ResultField;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find combined transport by location
 * 
 * (c) 2014 PTV AG
 */

public class FindCombinedTransportByLocation {

	public static void main(String[] args) {
		CombinedTransportResponse resultList = findCombinedTransportByLocation();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "ID", "Name", "Duration", "Type", "Start", "Destination", "Additional Fields" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find Combined Transport by Location");
		frame.setSize(750, 100);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(CombinedTransportResponse resultList) {
		int max = resultList.getResultList().length;
		Object rows[][] = new Object[max][7];

		for (int i = 0; i < max; i++) {
			for (ResultCombinedTransport address : resultList.getResultList()) {
				rows[i][0] = i;
				rows[i][1] = address.getName();
				rows[i][2] = address.getDuration();
				rows[i][3] = address.getType();

				String startCountry = address.getStart().getCountry();
				String startName = address.getStart().getName();
				String startCoordX = "" + String.format("%.0f", address.getStart().getCoordinate().getPoint().getX());
				String startCoordY = "" + String.format("%.0f", address.getStart().getCoordinate().getPoint().getY());
				rows[i][4] = startCountry + "; " + startName + "; Location:[" + startCoordX + ":" + startCoordY + "]";

				String destCountry = address.getDestination().getCountry();
				String destName = address.getDestination().getName();
				String destCoordX = "" + String.format("%.0f", address.getDestination().getCoordinate().getPoint().getX());
				String destCoordY = "" + String.format("%.0f", address.getDestination().getCoordinate().getPoint().getY());
				rows[i][5] = destCountry + "; " + destName + "; Location:[" + destCoordX + ":" + destCoordY + "]";

				String additionalFields = "";
				int kmax = address.getCombinedTransportFields().length;
				for (int k = 0; k < kmax; k++) {
					additionalFields = additionalFields + address.getCombinedTransportFields()[k].getValue() + "; ";
				}
				rows[i][6] = additionalFields;
			}
		}

		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Combined Transport Search");
		ctx.setProperties(properties);

		return ctx;
	}

	private static Location setLocation(double x, double y) {
		PoiLocation loc = new PoiLocation();
		Point coords = new Point();
		coords = CommonUtils.makePoint(x, y);
		loc.setCoordinate(coords);

		return loc;
	}

	private static ResultField[] getResultFields() {
		AdditionalField field1 = new AdditionalField(ResultField.START_LOCATION_COUNTRYCODE, "");
		ResultField resfield1 = field1.getField();

		AdditionalField field2 = new AdditionalField(ResultField.DESTINATION_LOCATION_COUNTRYCODE, "");
		ResultField resfield2 = field2.getField();

		AdditionalField field3 = new AdditionalField(ResultField.START_LOCATION_TILE_X, "");
		ResultField resfield3 = field3.getField();

		AdditionalField field4 = new AdditionalField(ResultField.DESTINATION_LOCATION_TILE_X, "");
		ResultField resfield4 = field4.getField();

		AdditionalField field5 = new AdditionalField(ResultField.START_LOCATION_TILE_Y, "");
		ResultField resfield5 = field5.getField();

		AdditionalField field6 = new AdditionalField(ResultField.DESTINATION_LOCATION_TILE_Y, "");
		ResultField resfield6 = field6.getField();

		ResultField[] resultFields = new ResultField[] { resfield1, resfield2, resfield3, resfield4, resfield5, resfield6 };
		return resultFields;
	}

	private static CombinedTransportResponse findCombinedTransportByLocation() {
		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set location for searching combined transports
		// Please notices: The PTV Europe City Map Premium is required!
		Location location = setLocation(1087160.0, 7074100.0);

		// Step 4: Create array for result field
		ResultField[] results = getResultFields();

		// Step 5: Call findCombinedTransportByLocation()
		CombinedTransportResponse resultList = null;
		try {
			resultList = xLocateClient.findCombinedTransportByLocation(location, null, results);
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with finding POIs");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return resultList;
	}
}