package samples.xlocate.basic;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find locations
 * 
 * (c) 2014 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.AddressResponse;
import com.ptvag.xserver.xlocate.Location;
import com.ptvag.xserver.xlocate.ResultAddress;
import com.ptvag.xserver.xlocate.ReverseSearchOption;
import com.ptvag.xserver.xlocate.ReverseSearchParameter;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

public class FindLocations {

	public static void main(String[] args) {
		AddressResponse[] resultList = findLocations();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "Index", "Country", "Postcode", "City", "District", "Street", "Location" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find Locations");
		frame.setSize(700, 140);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(AddressResponse[] resultList) {
		int max = resultList.length;
		int kmax = 0;
		for (int i = 0; i < max; i++) {
			kmax = kmax + resultList[i].getResultList().length;
		}
		Object rows[][] = new Object[kmax][7];

		int index = 0;
		for (int i = 0; i < max; i++) {
			for (ResultAddress address : resultList[i].getResultList()) {
				rows[index][0] = "" + (i + 1);
				rows[index][1] = address.getCountry();
				rows[index][2] = address.getPostCode();
				rows[index][3] = address.getCity();
				rows[index][4] = address.getCity2();

				String street = address.getStreet();
				street = street + " " + address.getHouseNumber();
				rows[index][5] = street;

				String coords = "" + String.format("%.0f", address.getCoordinates().getPoint().getX());
				coords = coords + ";" + String.format("%.0f", address.getCoordinates().getPoint().getY());
				rows[index][6] = coords;
				index++;
			}
		}
		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Reverse Geocoding");
		ctx.setProperties(properties);

		return ctx;
	}

	private static Location setLocation(double x, double y) {
		Location location = new Location();

		PlainPoint plainPoint = new PlainPoint();
		plainPoint.setX(x);
		plainPoint.setY(y);
		Point point = new Point();
		point.setPoint(plainPoint);

		location.setCoordinate(point);
		return location;
	}

	public static ReverseSearchOption[] getReverseSearchOptions() {
		ReverseSearchOption soEngineTargetSize = new ReverseSearchOption();
		soEngineTargetSize.setValue("5");
		soEngineTargetSize.setParam(ReverseSearchParameter.ENGINE_TARGETSIZE);
		// Candidates around the location will be listed until tolerance is reached.
		ReverseSearchOption soEngineTolerance = new ReverseSearchOption();
		soEngineTolerance.setValue("160");
		soEngineTolerance.setParam(ReverseSearchParameter.ENGINE_TOLERANCE);

		ReverseSearchOption[] options = new ReverseSearchOption[] { soEngineTargetSize, soEngineTolerance };

		return options;
	}

	public static AddressResponse[] findLocations() {

		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set locations
		Location loc1 = setLocation(680546.0, 6373544.0);
		Location loc2 = setLocation(680581.0, 6373573.0);
		Location loc3 = setLocation(680599.0, 6373521.0);

		Location[] locations = new Location[] { loc1, loc2, loc3 };

		// Step 4: Define ReverseSearchOption
		ReverseSearchOption[] options = getReverseSearchOptions();

		// Step 5: Call findLocations()
		AddressResponse[] response = null;
		try {
			response = xLocateClient.findLocations(locations, options, null, null);

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with reverse geocoding");
			System.out.println("Exception:\n" + xle);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return response;
	}
}
