package samples.xlocate.basic;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.AdditionalField;
import com.ptvag.xserver.xlocate.ObjectResponse;
import com.ptvag.xserver.xlocate.ResultCombinedTransport;
import com.ptvag.xserver.xlocate.ResultField;
import com.ptvag.xserver.xlocate.ResultObject;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find object by single field search
 * 
 * (c) 2014 PTV AG
 */

public class FindObjectByText {

	public static void main(String[] args) {
		ObjectResponse resultList = findObjectByText();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "ID", "Name", "Duration", "Type", "Start", "Destination" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find Object by Text");
		frame.setSize(750, 150);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(ObjectResponse resultList) {
		ResultObject[] objects = resultList.getResultList();
		int max = objects.length;
		Object rows[][] = new Object[max][8];

		for (int i = 0; i < max; i++) {
			ResultCombinedTransport address = objects[i].getCombinedTransport();
			if (address != null) {
				rows[i][0] = i;
				rows[i][1] = address.getName();
				rows[i][2] = address.getDuration();
				rows[i][3] = address.getType();
				String startCountry = address.getStart().getCountry();
				String startName = address.getStart().getName();
				String startCoordX = "" + String.format("%.0f", address.getStart().getCoordinate().getPoint().getX());
				String startCoordY = "" + String.format("%.0f", address.getStart().getCoordinate().getPoint().getY());
				rows[i][4] = startCountry + "; " + startName + "; Location:[" + startCoordX + ":" + startCoordY + "]";

				String destCountry = address.getDestination().getCountry();
				String destName = address.getDestination().getName();
				String destCoordX = "" + String.format("%.0f", address.getDestination().getCoordinate().getPoint().getX());
				String destCoordY = "" + String.format("%.0f", address.getDestination().getCoordinate().getPoint().getY());
				rows[i][5] = destCountry + "; " + destName + "; Location:[" + destCoordX + ":" + destCoordY + "]";
			}
		}

		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Geocoding");
		ctx.setProperties(properties);

		return ctx;
	}

	private static SearchOption[] getSearchOptions() {
		SearchOption separator = new SearchOption();
		separator.setParam(SearchParameter.SINGLE_FIELD_SEPARATORS);
		separator.setValue("");
		SearchOption addressSearch = new SearchOption();
		addressSearch.setParam(SearchParameter.ENGINE_ADDRESSSEARCH_ENABLE);
		addressSearch.setValue("false");
		SearchOption combinedTransportSearch = new SearchOption();
		combinedTransportSearch.setParam(SearchParameter.ENGINE_COMBINEDTRANSPORTSEARCH_ENABLE);
		combinedTransportSearch.setValue("true");

		SearchOption[] options = new SearchOption[] { separator, addressSearch, combinedTransportSearch };

		return options;
	}

	private static ResultField[] getResultFields() {
		AdditionalField unmatchedWordsCount = new AdditionalField(ResultField.UNMATCHED_WORDS_COUNT, "");
		ResultField resfield1 = unmatchedWordsCount.getField();
		AdditionalField unmatchedWords = new AdditionalField(ResultField.UNMATCHED_WORDS, "");
		ResultField resfield2 = unmatchedWords.getField();

		ResultField[] resultFields = new ResultField[] { resfield1, resfield2, };
		return resultFields;
	}

	private static ObjectResponse findObjectByText() {
		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set Address
		// Please notice: The PTV Europe City Map Premium is required!
		String object = "Fhre Hamburg";
		String country = "D";

		// Step 4: Specific search options
		SearchOption[] options = getSearchOptions();

		// Step 5: Define parameters for the final list
		ResultField[] resultFields = getResultFields();

		// Step 6: Call findAddressByText()
		ObjectResponse resultList = null;
		try {
			resultList = xLocateClient.findObjectByText(object, country, options, null, resultFields);
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with geocoding");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return resultList;
	}
}