package samples.xlocate.basic;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XLocateUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xlocate.PoiAddress;
import com.ptvag.xserver.xlocate.PoiAddressResponse;
import com.ptvag.xserver.xlocate.PoiResultAddress;
import com.ptvag.xserver.xlocate.SearchOption;
import com.ptvag.xserver.xlocate.SearchParameter;
import com.ptvag.xserver.xlocate.XLocateException;
import com.ptvag.xserver.xlocate.XLocateRemoteInterface;

/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license to use all programming
 * code examples from which you can generate similar function tailored to your
 * own specific needs.
 * 
 * All sample codes are provided by PTV Group for illustrative purposes only.
 * These examples have not been thoroughly tested under all conditions.
 * Therefore, PTV Group cannot guarantee or imply reliability, service ability
 * or function of these programs.
 * 
 * All programs contained herein are provided to you "AS IS" without any
 * warranties of any kind. The implied warranties of non-infringement, merchant
 * ability and fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/

/*
 * PTV xServer Code Samples
 * 
 * PT xLocate Server - find POIs by addresses
 * 
 * (c) 2014 PTV AG
 */

public class FindPoiByAddresses {

	public static void main(String[] args) {
		PoiAddressResponse[] resultList = findPoiByAddresses();

		if (resultList == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(resultList);
		Object[] cols = new Object[] { "Index", "Country", "Postcode", "City", "Street", "POI Type", "POI Name", "Location" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Find POIs by Addresses");
		frame.setSize(750, 450);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(PoiAddressResponse[] resultList) {
		int max = resultList.length;
		int kmax = 0;
		for (int i = 0; i < max; i++) {
			kmax = kmax + resultList[i].getResultList().length;
		}
		Object rows[][] = new Object[kmax][8];

		int index = 0;
		for (int i = 0; i < max; i++) {
			for (PoiResultAddress address : resultList[i].getResultList()) {
				rows[index][0] = "" + (i+1);
				rows[index][1] = address.getCountry();
				rows[index][2] = address.getPostCode();
				rows[index][3] = address.getCity();
				String street = address.getStreet();
				street = street + " " + address.getHouseNumber();
				rows[index][4] = street;
				rows[index][5] = address.getPoiType();
				rows[index][6] = address.getPoiName();
				String coords = "" + String.format("%.0f", address.getCoordinates().getPoint().getX());
				coords = coords + ";" + String.format("%.0f", address.getCoordinates().getPoint().getY());
				rows[index][7] = coords;
				index++;
			}
		}
		return rows;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };

		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic POI Search");
		ctx.setProperties(properties);

		return ctx;
	}

	private static PoiAddress setPoiAddress(String country, String postCode, String poiType, String poiName,
			String geoDataSourceLayer) {
		PoiAddress addr = new PoiAddress();
		addr.setCountry(country);
		addr.setPostCode(postCode);
		addr.setPoiType(poiType);
		addr.setPoiName(poiName);
		addr.setGeoDataSourceLayer(geoDataSourceLayer);

		return addr;
	}

	public static SearchOption[] getSearchOptions() {
		SearchOption engineTargetSize = new SearchOption();
		engineTargetSize.setValue("250");
		engineTargetSize.setParam(SearchParameter.MAX_RESULT);

		SearchOption[] options = new SearchOption[] { engineTargetSize };

		return options;
	}

	private static PoiAddressResponse[] findPoiByAddresses() {
		// Step 1: Instantiate remote interface for xLocate
		final String url = "http://localhost:50020/xlocate/ws/XLocate";
		XLocateRemoteInterface xLocateClient = XLocateUtils.makeClient(url);

		// Step 2: Set CallerContext
		xLocateClient.setCallerContext(getCallerContext());

		// Step 3: Set POI addresses
		PoiAddress addr1 = setPoiAddress("L", "2", "7311", "", "default.points-of-interest.default");
		PoiAddress addr2 = setPoiAddress("L", "2", "7312", "", "default.points-of-interest.default");
		PoiAddress addr3 = setPoiAddress("L", "2", "9910", "", "default.points-of-interest.default");

		PoiAddress[] addresses = new PoiAddress[] { addr1, addr2, addr3 };

		// Step 4: Define search options
		SearchOption[] options = getSearchOptions();

		// Step 6: Call findPoiByAddresses()
		PoiAddressResponse[] resultList = null;
		try {
			resultList = xLocateClient.findPoiByAddresses(addresses, options, null, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XLocateException xle) {
			System.err.println("Problem with finding addresses");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return resultList;
	}
}