/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.Coordinates_Transformation;
import utils.XMapUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

@SuppressWarnings("serial")
public class CoordinateTransformation extends JPanel {
	public static Dimension imageSize = new Dimension(340, 340);
	public static JLabel label = new JLabel("Loading ...");
	public static ImageIcon icon;

	// public static Map map;

	private void runSample() {
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(685407.751736, 6372537.965244, null));
		mapSection.setCenter(point);
		mapSection.setScale(10);

		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(imageSize.height);
		imageInfo.setWidth(imageSize.width);
		imageInfo.setImageParameter("");

		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		boolean includeImageInResponse = false;

		try {
			Map map = xmapClient.renderMap(mapSection, mapParams, imageInfo,
					null, includeImageInResponse);

			/*
			 * Example: how to access information available in a Map
			 * 
			 * BoundingBox bbox = map.getVisibleSection().getBoundingBox();
			 * double leftTopX = bbox.getLeftTop().getPoint().getX(); double
			 * leftTopY = bbox.getLeftTop().getPoint().getY(); double
			 * rightBottomX = bbox.getLeftTop().getPoint().getX(); double
			 * rightBottomY = bbox.getLeftTop().getPoint().getY();
			 */

			// retrieve the image
			String imageUrl = "http://" + map.getImage().getUrl();
			icon = new ImageIcon(new URL(imageUrl));

			label.setIcon(icon);
			label.setText("");

			getVisibleSection(map);

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}

	}

	public void startSample() {
		this.add(label);

		Thread thread = new Thread() {

			public void run() {
				runSample();
			}
		};
		thread.start();
	}

	public static void main(String[] args) {

		// display image in window
		JFrame frame = new JFrame("Coordinate Transformation");
		CoordinateTransformation cT = new CoordinateTransformation();
		cT.startSample();
		frame.setLayout(new FlowLayout(FlowLayout.CENTER));
		frame.getContentPane().add(label);
		frame.setVisible(true);
		frame.pack();
		frame.setSize(imageSize.width + 20, imageSize.height + 40);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

	public void getVisibleSection(Map map) {
		// center coordinates (default: PTV MERCATOR)
		double xCoord = map.getVisibleSection().getCenter().getPoint().getX();
		double yCoord = map.getVisibleSection().getCenter().getPoint().getY();
		// scale
		int scale = map.getVisibleSection().getScale();
		// leftTop- and rightBottom-coordinates
		double leftTopX = map.getVisibleSection().getBoundingBox().getLeftTop()
				.getPoint().getX();
		double leftTopY = map.getVisibleSection().getBoundingBox().getLeftTop()
				.getPoint().getY();
		double rightBottomX = map.getVisibleSection().getBoundingBox()
				.getRightBottom().getPoint().getX();
		double rightBottomY = map.getVisibleSection().getBoundingBox()
				.getRightBottom().getPoint().getY();

		// The function below uses a formula which gives an approximation of the
		// distance in meters. Hence, the result is not exact, but for distances
		// up to 600km and 80 latitude the error is sufficiently small.
		double distance = Coordinates_Transformation.distanceFromMercator(
				leftTopX, leftTopY, rightBottomX, rightBottomY);

		System.out.println("Results of VisibleSection");
		System.out.println(String.format("center:\t\t %.0f\t %.0f", xCoord,
				yCoord));
		System.out.println("scale:\t\t " + scale);

		System.out.println("\nPTV MERCATOR");
		System.out.println(String.format("LeftTop:\t %.0f\t %.0f", leftTopX,
				leftTopY));
		System.out.println(String.format("RightBottom:\t %.0f\t %.0f",
				rightBottomX, rightBottomY));
		System.out.println(String.format("distance:\t %.3f", distance / 1000.0)
				+ " km");

		// Geodecimal
		double[] outlT = new double[2];
		double[] outrB = new double[2];

		outlT = Coordinates_Transformation.Mercator_2_GeoDecimal(leftTopX,
				leftTopY);
		outrB = Coordinates_Transformation.Mercator_2_GeoDecimal(rightBottomX,
				rightBottomY);

		System.out.println("\nPTV GEODECIMAL");
		System.out.println(String.format("LeftTop:\t %.0f\t %.0f", outlT[0],
				outlT[1]));
		System.out.println(String.format("RightBottom:\t %.0f\t %.0f",
				outrB[0], outrB[1]));

		// GeoMinSec
		outlT = Coordinates_Transformation.Mercator_2_GeoMinSec(leftTopX,
				leftTopY);
		outrB = Coordinates_Transformation.Mercator_2_GeoMinSec(rightBottomX,
				rightBottomY);

		System.out.println("\nPTV GEOMINSEC");
		System.out.println(String.format("LeftTop:\t %.0f\t %.0f", outlT[0],
				outlT[1]));
		System.out.println(String.format("RightBottom:\t %.0f\t %.0f",
				outrB[0], outrB[1]));
		// SmartUnits
		outlT = Coordinates_Transformation.Mercator_2_SmartUnits(leftTopX,
				leftTopY);
		outrB = Coordinates_Transformation.Mercator_2_SmartUnits(rightBottomX,
				rightBottomY);
		
		System.out.println("\nPTV SmartUnits");
		System.out.println(String.format("LeftTop:\t %.0f\t %.0f", outlT[0],
				outlT[1]));
		System.out.println(String.format("RightBottom:\t %.0f\t %.0f",
				outrB[0], outrB[1]));

	}

}
