/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with drawing a line
 * 
 * (c) 2012 PTV AG
 */
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;

import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainLineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithCustomLayer {
	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// Step 1: Instantiate xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Define boundingBox
		BoundingBox boundingBox = new BoundingBox();	
		// Step 2.1: Define upper left coordinate of the bounding box
		Point leftTop = new Point();
		leftTop.setPoint(new PlainPoint(678722, 6375300, null)); // point = (x,y)
		// Step 2.2: Define lower right coordinate of the bounding box
		Point rightBottom = new Point();
		rightBottom.setPoint(new PlainPoint(684350, 6368807, null));	
		boundingBox.setLeftTop(leftTop);
		boundingBox.setRightBottom(rightBottom);
		
		// Step 3: Define parameters for call using object imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(480);
		imageInfo.setWidth(640);
		imageInfo.setImageParameter("");
	
		// Step 4: Create mapParams
		MapParams mapParams = new  MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);
		
		// Step 5: Set value to false
		boolean includeImageInResponse = false;
		
		// Step 6.1: Prepare plainPoints
		PlainPoint[] plainPoints = new PlainPoint[5];
		plainPoints[0] = new PlainPoint(681854.0, 6372751.0, null);
		plainPoints[1] = new PlainPoint(680941.0, 6370524.0, null);
		plainPoints[2] = new PlainPoint(683598.0, 6371416.0, null);
		plainPoints[3] = new PlainPoint(683165.0, 6374963.0, null);
		plainPoints[4] = new PlainPoint(679546.0, 6373244.0, null);
	
		// Step 6.2: Create plainLineString
		PlainLineString plainLineString = new PlainLineString(plainPoints);
		
		// Step 6.3: Set lineStrings
		LineString[] lineStrings = new LineString[1];
		lineStrings[0] = new LineString();
		lineStrings[0].setLineString(plainLineString);
		
		// Step 6.4 Add main line combined with part options	
		LineOptions options =  new LineOptions();
		LinePartOptions partoptions = new LinePartOptions();
		partoptions.setVisible(true);
		partoptions.setColor(new Color(255,50,20));
		partoptions.setWidth(7);
		options.setMainLine(partoptions);

		// Step 6.5: Define lines and put lineStrings and options togehter
		Lines[] lines = new Lines[1];
		lines[0] = new Lines(lineStrings, options);
		
		// Step 7.1: Define customLayer that contains lines and further attributes
		CustomLayer customLayer = new CustomLayer();
		customLayer.setVisible(true);
		customLayer.setDrawPriority(25000);
		customLayer.setLines(lines);
		customLayer.setObjectInfos(ObjectInfoType.NONE);
		
		// Step 7.2: Set customLayer
		Layer[] layers = new Layer[]{ customLayer };
		
		try {
			// Step 8: return map object
			Map map = xmapClient
				.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse);
			
			// Step 9: Retrieve the image
			String imageUrl = "http://"+map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			
			// Step 10: Display image using class frame
			JFrame frame = new JFrame("Sample Map");
			frame.setSize(icon.getIconWidth()+20, icon.getIconHeight()+30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			
		}catch(Exception e){
			// TODO proper exception handling
			e.printStackTrace();
		}
		
	}

}
