/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with footprints
 * 
 * (c) 2012 PTV AG
 */
import java.awt.Dimension;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.jabba.service.baseservices.ClientFactory;
import com.ptvag.jabba.service.baseservices.RemoteType;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithFootprints extends JPanel {
	Dimension imageSize = new Dimension(240, 320);
	JLabel label = new JLabel("Loading ...");

	private void runSample() {

		// Step 1: Instantiate remote interface for xMap
		final String url = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface client = (XMapRemoteInterface) ClientFactory.createClient(XMapRemoteInterface.class,
				RemoteType.DOCSTYLE_CXF, "", "", url);

		// Step 2: Create object MapSection
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(681490.0, 6372500.0, null));
		mapSection.setCenter(point);
		mapSection.setScale(10);

		// Step 3: Create object CallerContext
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Basic Mapping");
		// Step 3.1: Set CallerContext properties to define profile "footprints"
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("footprints");
		// Step 3.2: Set CallerContext properties to define coordinate format "PTV Mercator"
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		// Step 3.3: Put properties into class CallerContextProperty[]
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		// Step 3.4: set CallerContext
		client.setCallerContext(cxt);

		// Step 4: Create object ImageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(240);
		imageInfo.setWidth(320);
		imageInfo.setImageParameter("");

		// Step 5: Create object MapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 6: Set attribute includeImageInResponse
		boolean includeImageInResponse = false;

		try {
			// Step 7: Render map
			Map map = client.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse);

			// Step 8: Get the image from object map
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			// Step 8.1: set icon into label
			label.setIcon(icon);
			label.setText("");
			this.setMinimumSize(new Dimension(icon.getIconWidth() + 20, icon.getIconHeight() + 30));

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}

	}

	public MappingWithFootprints() {
		this.add(label);
		Thread thread = new Thread() {
			@Override
			public void run() {
				runSample();
			}
		};
		thread.start();
	}

	public static void main(String[] args) {
		// display image in window
		JFrame frame = new JFrame("Sample Map");
		JPanel sample = new MappingWithFootprints();
		frame.getContentPane().add(sample);
		frame.setVisible(true);
		frame.pack();
		frame.setSize(340, 280);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

}
