/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;
/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping using GeometryLayer
 * 
 * (c) 2012 PTV AG
 */


import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.Polygon;
import com.ptvag.xserver.xmap.Geometries;
import com.ptvag.xserver.xmap.Geometry;
import com.ptvag.xserver.xmap.GeometryLayer;
import com.ptvag.xserver.xmap.GeometryOption;
import com.ptvag.xserver.xmap.GeometryOptions;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithGeometryLayer {
	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
	
		// Step 1: Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);
		
		// Note: the German map is required as minimum in this example!
		// Step 2: Set center
		Point point = new Point();
		point.setPoint( new PlainPoint(850000, 6900000, null) );
		
		// Step 3: Set the attributes for mapSection
		MapSection mapSection = new MapSection();
		mapSection.setCenter(point);
		mapSection.setScale(20000);
		
		// Step 4: Set the attributes for mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);
		
		// Step 5: Set the attributes for imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(240);
		imageInfo.setWidth(320);
		imageInfo.setImageParameter("");

		// Step 6: Set value to false
		boolean includeImageInResponse = false;
		
		// Step 7: Define both polygons in WKT-format
		Polygon polygon1 = new Polygon();
		polygon1.setWkt("POLYGON((800000 6860000, 800000 6880000, 900000 6880000, 900000 6860000, 800000 6860000)" +
				"              ,(820000 6862000, 820000 6868000, 880000 6868000, 880000 6862000, 820000 6862000))");
		
		Polygon polygon2 = new Polygon();
		polygon2.setWkt("POLYGON((800000 6900000, 900000 6900000, 850000 7000000, 800000 6900000)" +
		"              ,(830000 6910000, 870000 6910000, 850000 6950000, 830000 6910000))");

		// An alternative to using WKT is to use the available data structures
		// For polygon2 this would look like
		/*Polygon polygon2 = new Polygon(null,null,
		new PlainPolygon(
				new PlainLinearRing[]{
						new PlainLinearRing(
								new PlainPoint[]{
										new PlainPoint(800000, 6900000),
										new PlainPoint(900000, 6900000),
										new PlainPoint(850000, 7000000),
										new PlainPoint(800000, 6900000)
								}
						),
						new PlainLinearRing(
								new PlainPoint[]{
										new PlainPoint(830000, 6910000),
										new PlainPoint(870000, 6910000),
										new PlainPoint(850000, 6950000),
										new PlainPoint(830000, 6910000)
								}
						)
				}
		)
		);*/
		
		// Step 8: Set some geometry options to alter the appearance of the shapes
		GeometryOption[] geometryOptions = new GeometryOption[]{
			new GeometryOption(GeometryOptions.BORDERCOLOR, rgbToColorString(0,128,255)),
			new GeometryOption(GeometryOptions.FILLCOLOR, rgbToColorString(255,0,0)),
			new GeometryOption(GeometryOptions.FILLALPHA, "200"), // 0-255
			new GeometryOption(GeometryOptions.AUTOCENTEROBJECTS, "1") // 0-255
		};
			
		// Step 9: Create geometries from the shapes
		Geometry geometry1 = new Geometry();
		geometry1.setDescription("firstPolygon");
		geometry1.setId(0);
		geometry1.setGeometry(polygon1);
		geometry1.setReferencePoint(point);
		
		Geometry geometry2 = new Geometry();
		geometry2.setDescription("secondPolygon");
		geometry2.setId(1);
		geometry2.setGeometry(polygon2);
		geometry2.setReferencePoint(point);
		
		// Step 10: Bundle geometries with geometry options 
		Geometries geometries = new Geometries(
				new Geometry[]{ geometry1, geometry2},
				geometryOptions
				);
		
		// Step 11: Put the geometries into the specific geometry layer
		GeometryLayer layer = new GeometryLayer();
		layer.setName("geomLayer");
		layer.setDrawPriority(DrawPriorities.PRIORSTREET);
		layer.setVisible(true);
		layer.setGeometries(new Geometries[]{geometries});
		layer.setObjectInfos(ObjectInfoType.GEOMETRY);
		
		try {
			// Step 12: Return object map with the specific information
			Map map = xmapClient.renderMap(
					mapSection, mapParams, imageInfo,
					new Layer[]{layer}, // include custom layer in request
					includeImageInResponse);
			
			// Step 13: Retrieve the image from the specific URL
			String imageUrl = "http://"+map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			
			// Step 14: Display image in your frame
			JFrame frame = new JFrame("Sample Map");
			frame.setSize(icon.getIconWidth()+20, icon.getIconHeight()+30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	private static String rgbToColorString(int red, int green, int blue) {
		int color = (blue<<16)+(green<<8)+(red);
		return ""+color;
	}

	/**
	 *  Class with draw priority constants.
	 *  
	 *  Values taken from the PTV xMap documentation 
	 */
	public class DrawPriorities {
		public static final int PRIORMIN                  =0;       //Minimum priority
		public static final int PRIORBRIDGE               =10;      //Bridges
		public static final int PRIORHEIGHT               =20;      //Height
		public static final int PRIOROTHER                =30;      //Other
		public static final int PRIORRAIL                 =40;      //Rail
		public static final int PRIORTUNNEL               =50;      //Tunnels
		public static final int PRIORWATER                =60;      //Water
		public static final int PRIORBACKGROUND           =80;      //Background
		public static final int PRIORROUTE                =100;     //Routes
		public static final int PRIORTOLL                 =110;     //Toll data
		public static final int PRIORSCENICROUTE          =115;     //Scenic routes
		public static final int PRIORSTREET               =150;     //Streets
		public static final int PRIORTOWNPOINTBASE        =400;     //Town points
		public static final int PRIORONEWAYSIGN           =450;     //One-way symbol
		public static final int PRIORTOWNNAMEBASE         =500;     //City names
		public static final int PRIORTOWNNAME1            =501;     //City names category 14 (>500000)
		public static final int PRIORTOWNNAME2            =502;     //City names category 13 (250000-500000)
		public static final int PRIORTOWNNAME3            =503;     //City names category 12 (100000-250000)
		public static final int PRIORTOWNNAME4            =504;     //City names category 11 (50000-100000)
		public static final int PRIORTOWNNAME5            =505;     //City names category 10 (20000-50000)
		public static final int PRIORTOWNNAME6            =506;     //City names category 9 (10000-20000)
		public static final int PRIORTOWNNAME7            =507;     //City names category 8 (5000-10000)
		public static final int PRIORTOWNNAME8            =508;     //City names category 7 (3000-5000)
		public static final int PRIORTOWNNAME9            =509;     //City names category 6 (2000-3000)
		public static final int PRIORTOWNNAME10           =510;     //City names category 5 (1000-2000)
		public static final int PRIORTOWNNAME11           =511;     //City names category 4 (500-1000)
		public static final int PRIORTOWNNAME12           =512;     //City names category 3 (200-500)
		public static final int PRIORTOWNNAME13           =513;     //City names category 2 (100-200)
		public static final int PRIORTOWNNAME14           =514;     //City names category 1 (100)
		public static final int PRIORSTREETSIGNSBASE      =550;     //Street signs
		public static final int PRIORSTREETSIGNSMOTORWAY  =551;     //Street signs motorways
		public static final int PRIORSTREETSIGNSCOUNTRY   =552;     //Street signs trunk roads
		public static final int PRIORSTREETNAMEBASE       =600;     //Street names
		public static final int PRIORSTREETNAMEMOTORWAY   =601;     //Street names motorways
		public static final int PRIORSTREETNAMEEUROPE     =602;     //Street names Europe roads
		public static final int PRIORSTREETNAMENATIONAL   =603;     //Street names trunk roads
		public static final int PRIORSTREETNAMECOUNTRY    =604;     //Street names country roads
		public static final int PRIORSTREETNAMEREGION     =605;     //Street names county roads
		public static final int PRIORSTREETNAMETOWN       =606;     //Street names town streets
		public static final int PRIORSTREETNAMEZONE30     =607;     //Street names residential area streets
		public static final int PRIORSTREETNAMEPLAY       =608;     //Street names play streets
		public static final int PRIORADAKSIGN             =800;     //Signs of intersections
		public static final int PRIORHOUSENUMBER          =11000;    //House number
		public static final int PRIORTEXT                 =12000;   //General text
		public static final int PRIORPOIDEFAULT           =25000;   //Points of interest (on top)
		public static final int PRIORMAX                  =50000;   //Maximum priority
	}
}

