/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, service ability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG, Karlsruhe Germany
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * xServer code samples
 * 
 * xMap: Changing the language of maps
 *
 * (c) 2012 PTV AG
 */

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithLanguage extends JPanel {

	JLabel iconLabel;
	JComboBox<String> profileComboBox;

	/**
	 * Array for available profiles selecting different languages.
	 * 
	 * NOTE: these must be available on the target server (not by default)
	 */
	private static final String[] AVAILABLE_PROFILES = new String[] { "italian", "french", "german" };

	/**
	 * Retrieves a map from PTV xMap Server with the specific settings of the
	 * selected profile
	 */
	private void updateMap() {

		// Step 1: Instantiate the map client
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Create object mapSection
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(681600, 6372500, null));
		mapSection.setCenter(point);
		mapSection.setScale(20000);

		// Step 3: Set the parameters for imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(320);
		imageInfo.setWidth(480);
		imageInfo.setImageParameter("");

		// Step 4: Define class mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 5: Use class CallerContext to define the specific profile
		CallerContext ctx = new CallerContext();
		CallerContextProperty ctx_property = new CallerContextProperty();
		ctx_property.setKey("Profile");
		// Note: Did you have added the new profiles before and restarted the PTV
		// xMap Server?
		// Step 6: Select the profile via application
		ctx_property.setValue(this.profileComboBox.getSelectedItem().toString());
		CallerContextProperty[] properties = { ctx_property };
		ctx.setProperties(properties);
		xmapClient.setCallerContext(ctx);

		// Step 7: Set boolean value to false for attribute includeImageInResponse
		boolean includeImageInResponse = false;

		try {
			// Step 8: Return map object with the specific information
			Map map = xmapClient.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse);

			// Step 9: Plot the image
			String imageUrl = "http://" + map.getImage().getUrl();
			this.setImageUrl(imageUrl);

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}
	}

	/**
	 * Constructor that sets up the UI
	 */
	public MappingWithLanguage() {
		iconLabel = new JLabel();
		profileComboBox = new JComboBox<String>(AVAILABLE_PROFILES);

		this.setLayout(new FlowLayout());
		this.add(profileComboBox);
		this.add(iconLabel);

		profileComboBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				updateMap();
			}
		});
		updateMap();
	}

	/**
	 * Sets the image to the image available at the given URL
	 * 
	 * @param imageUrl
	 *          URL of the image to display
	 */
	private void setImageUrl(String imageUrl) {
		try {
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			iconLabel.setIcon(icon);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		JFrame frame;

		frame = new JFrame("PTV Codesample - Mapping with different languages");
		frame.setSize(500, 380);
		frame.getContentPane().add(new MappingWithLanguage());
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

}
