/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with POIs
 * 
 * (c) 2012 PTV AG
 */

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.ObjectInfos;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithPoints {
	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// Step 1: Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Define mapSection and set center and scale
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(685407.751736, 6372537.965244, null));
		mapSection.setCenter(point);
		mapSection.setScale(1000);

		// Step 3: Create imageInfo and set relevant attributes
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(240);
		imageInfo.setWidth(320);
		imageInfo.setImageParameter("");

		// Step 4: Realize object mapParams showing scale without use of miles.
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 5: Define points for bitmaps
		PlainPoint plainPoint1 = new PlainPoint(681254.0, 6372399.0, null);
		Point point1 = new Point();
		point1.setPoint(plainPoint1);
		PlainPoint plainPoint2 = new PlainPoint(682875.0, 6369972.0, null);
		Point point2 = new Point();
		point2.setPoint(plainPoint2);

		// Step 6: Fill bitmapArray with start and destination flag
		Bitmap[] bitmapArray = new Bitmap[2];
		bitmapArray[0] = new Bitmap();
		bitmapArray[0].setDescr("MyFirstBitmap");
		bitmapArray[0].setPosition(point1);
		bitmapArray[0].setName("flaggreen.bmp");
		bitmapArray[1] = new Bitmap();
		bitmapArray[1].setDescr("MySecondBitmap");
		bitmapArray[1].setPosition(point2);
		bitmapArray[1].setName("flagred.bmp");

		// Step 7: Set bitmapOptions
		BitmapOptions bitmapOptions = new BitmapOptions();
		// change the reference point for the pixel position of the bitmap
		// by default the bitmap center will be used.
		// here: bitmap displacement of 20 pixels.
		PlainPoint referencePoint = new PlainPoint(20,20, null);
		bitmapOptions.setReferencePoint(referencePoint);
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 8: Add bitmapArray and bitmapOptions into bitmaps
		Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapArray, bitmapOptions) };

		// Step 9: Initialize customLayer with the relevant settings
		CustomLayer customLayer = new CustomLayer();
		customLayer.setCenterObjects(true);
		customLayer.setDrawPriority(25000);
		customLayer.setName("ObjectLayer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setBitmaps(bitmaps);

		// Step 10: Put customLayer into layers
		Layer[] layers = new Layer[] { customLayer };

		// Step 11: Define boolean value including image in response
		boolean includeImageInResponse = false;
		
		try {
			// Step 12: Return map object with its characteristics
			Map map = xmapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse);
			ObjectInfos[] objInfo = map.getObjects();

			// Step 13: Print result
			System.out.println(String.format("%s (%.0f, %.0f)", "Pixel position of first point:", objInfo[0].getObjects()[0]
					.getPixel().getX(), objInfo[0].getObjects()[0].getPixel().getY()));
		
			// Step 14: Retrieve icon
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));

			// Step 15: Display image in window
			JFrame frame = new JFrame("Sample Map");
			frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (Exception e) {
			// proper exception handling: see API for PTV's exception handling!
			e.printStackTrace();
		}

	}

}
