/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with points and lines
 * 
 * (c) 2012 PTV AG
 */
import java.awt.FlowLayout;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;

import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainLineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithPointsAndLines extends JPanel {
	private static final long serialVersionUID = 1L;

	JLabel mapimageLabel = new JLabel();

	private void runSample() {
		// Step 1: initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: define start and destination points as type of PlainPoint
		PlainPoint startPoint = new PlainPoint(681602.0, 6372219.0, null);
		double[][] pointCoordinates = new double[][] { { 680361.0, 6370092.0 }, { 680868.0, 6369835.0 },
				{ 683232.0, 6374948.0 }, { 679510.0, 6373244.0 }, { 682756.0, 6368686.0 }, { 684163.0, 6372005.0 },
				{ 682071.0, 6376094.0 }, { 679305.0, 6371392.0 }, };
		int numberOfDestinationPoints = pointCoordinates.length;

		PlainPoint[] destinationPlainPoints = new PlainPoint[numberOfDestinationPoints];
		for (int i = 0; i < numberOfDestinationPoints; i++) {
			destinationPlainPoints[i] = new PlainPoint();
			destinationPlainPoints[i].setX(pointCoordinates[i][0]);
			destinationPlainPoints[i].setY(pointCoordinates[i][1]);
		}

		// Step 2.1: set color values for each line
		int[][] colorValues = new int[][] {
				// {blue, green, red}
				{ 255, 0, 0 }, { 0, 0, 255 }, { 0, 255, 0 }, { 238, 134, 28 }, { 139, 123, 108 }, { 79, 79, 47 },
				{ 131, 139, 131 }, { 0, 290, 139 } };

		// Step 3: define an array of lines
		Lines[] lines = new Lines[numberOfDestinationPoints];

		for (int i = 0; i < numberOfDestinationPoints; i++) {
			// Step 3.1: create plainLineString
			PlainLineString plainLineString = new PlainLineString();
			plainLineString.setPoints(new PlainPoint[] { startPoint, destinationPlainPoints[i] });
			// Step 3.2: set object into lineStrings[0]. Note: object lineStrings has to be reinitialized in every loop.
			LineString[] lineStrings = new LineString[1];
			lineStrings[0] = new LineString();
			lineStrings[0].setLineString(plainLineString);
			// Step 3.3: create new lines[i] and fill array
			lines[i] = new Lines();
			lines[i].setLines(lineStrings);
			// Step 3.4: set lineOptions transparency
			LineOptions lineOptions = new LineOptions();
			lineOptions.setTransparent(true);
			// Step 3.5: set linePartOptions visible, define width of line and
			// specific color
			LinePartOptions linePartOptions = new LinePartOptions();
			linePartOptions.setVisible(true);
			linePartOptions.setWidth(7);
			linePartOptions.setColor(new Color(colorValues[i][0], colorValues[i][1], colorValues[i][2]));
			// Step 3.6: set main line
			lineOptions.setMainLine(linePartOptions);
			// Step 3.7: set all options for specific line
			lines[i].setOptions(lineOptions);
		}

		// Step 4: generate a new object named arrBitmap as type of Bitmap[]
		Bitmap[] arrBitmap = new Bitmap[numberOfDestinationPoints + 1];
		for (int i = 0; i < numberOfDestinationPoints; i++) {
			Point bitmapPointDestination = new Point();
			bitmapPointDestination.setPoint(destinationPlainPoints[i]);
			String description = (i + 2) + ". bitmap: destination";
			// Step 4.1: set destination point, description and select the
			// corresponding flag
			arrBitmap[i] = new Bitmap();
			arrBitmap[i].setDescr(description);
			arrBitmap[i].setName("flagred.bmp");
			arrBitmap[i].setPosition(bitmapPointDestination);
		}

		Point bitmapPointStart = new Point();
		bitmapPointStart.setPoint(startPoint);
		// Step 4.2: set start point, description and select the corresponding flag
		arrBitmap[8] = new Bitmap();
		arrBitmap[8].setDescr("1. bitmap: start");
		arrBitmap[8].setName("location.bmp");
		arrBitmap[8].setPosition(bitmapPointStart);

		// Step 5: create bitmapOptions and set color for transparency
		BitmapOptions bitmapOptions = new BitmapOptions();
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 6: create bitmaps and put arrBitmap and bitmapOptions into this
		// object
		Bitmaps[] bitmaps = new Bitmaps[1];
		bitmaps[0] = new Bitmaps();
		bitmaps[0].setBitmaps(arrBitmap);
		bitmaps[0].setOptions(bitmapOptions);

		// Step 7: define customLayer and set the relevant attributes
		CustomLayer customLayer = new CustomLayer();
		customLayer.setDrawPriority(25000);
		customLayer.setName("Text layer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setBitmaps(bitmaps);
		customLayer.setLines(lines);
		// Step 7.1: add customLayer into layers
		Layer[] layers = new Layer[] { customLayer };

		// Step 8: define mapSection and set center and scale
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(startPoint);
		mapSection.setCenter(point);
		mapSection.setScale(400);

		// Step 9: create object imageInfo, define the relevant attributes
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(480);
		imageInfo.setWidth(640);
		imageInfo.setImageParameter("");

		// Step 10: set values as specified
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 11: set value to false.
		boolean includeImageInResponse = false;
		
		try {
			// Step 12: return object map with the comprehensive information
			Map map = xmapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse);

			// Step 13: retrieve the image
			String imageUrl = "http://" + map.getImage().getUrl();
			mapimageLabel.setIcon(new ImageIcon(new URL(imageUrl)));

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}
	}

	public MappingWithPointsAndLines() {
		this.setLayout(new FlowLayout());
		this.add(mapimageLabel);
		runSample();
	}

	public static void main(String[] args) {
		JFrame frame;

		frame = new JFrame("Point & Lines in map example");
		frame.setSize(640, 480);
		frame.getContentPane().add(new MappingWithPointsAndLines());
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

}
