/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with text blocks
 * 
 * (c) 2012 PTV AG
 */
import java.awt.FlowLayout;
import java.net.URL;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import utils.XMapUtils;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.Font;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.Text;
import com.ptvag.xserver.xmap.TextAlignment;
import com.ptvag.xserver.xmap.TextOptions;
import com.ptvag.xserver.xmap.Texts;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithTextToMap extends JPanel {
	JLabel mapImageLabel = new JLabel();
	
	public MappingWithTextToMap() {
		this.setLayout(new FlowLayout());
		this.add(mapImageLabel);
		runSample();
	}

	public static void main(String[] args) {
		JFrame frame;
		frame = new JFrame("PTV code sample - set text blocks");
		frame.setSize(640, 480);
		frame.getContentPane().add(new MappingWithTextToMap());
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	private void runSample() {
		// Step 1: Instantiate xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Define mapSection. Set center and scale.
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(685407.751736, 6372537.965244, null));
		mapSection.setCenter(point);
		mapSection.setScale(1500);

		// Step 3: Create object imageInfo and set the relevant parameters
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(480);
		imageInfo.setWidth(640);
		imageInfo.setImageParameter("");

		// Step 4: Add new class mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 5: Set text blocks and place object via defined location
		Point point1 = new Point();
		point1.setWkt("POINT(681254.0 6378399.0)");
		Text text1 = new Text();
		text1.setPosition(point1);
		text1.setText("Start point");

		Point point2 = new Point();
		point2.setWkt("POINT(688875.0 6369972.0)");
		Text text2 = new Text();
		text2.setText("Destination point");
		text2.setPosition(point2);

		// Step 6: Set text options
		Font font = new Font();
		font.setName("Arial");
		font.setSize(24);
		font.setBold(true);
		
		TextOptions textOptions = new TextOptions();
		textOptions.setFont(font);
		textOptions.setTextColor(new Color(0, 0, 0));
		textOptions.setFillBg(true);
		textOptions.setBgColor(new Color(255, 255, 255));
		textOptions.setShowFrame(true);
		textOptions.setFrameColor(new Color(0, 0, 255));
		textOptions.setAlignment(TextAlignment.CENTER);
		textOptions.setPixelX(0);
		textOptions.setPixelY(0);
		textOptions.setRelY(0);
		textOptions.setRelY(0);

		// Step 7: If you want to style each text object differently
		// you should create an object "Texts" for each text.
		// This allows you to use different text options.
		Texts texts = new Texts();
		texts.setOptions(textOptions);
		texts.setTexts(new Text[] { text1, text2 });

		// Step 8: Define customLayer and set the texts visible
		CustomLayer customLayer = new CustomLayer();
		customLayer.setDrawPriority(25000);
		customLayer.setName("Text layer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setTexts(new Texts[] { texts });
		Layer[] layers = new Layer[] { customLayer };

		// Step 9: Set value to false
		boolean includeImageInResponse = false;
		
		try {
			// Step 10: Return object map
			Map map = xmapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse);

			// Step 11: Retrieve the image
			String imageUrl = "http://" + map.getImage().getUrl();
			mapImageLabel.setIcon(new ImageIcon(new URL(imageUrl)));

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}

	}
}
