/*******************************************************************************
 * PTV xServer - Code Samples
 * Please note: this example is only executable whether you have correctly 
 * configured the RoadEditor configuration parameters.
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Mapping with Truck Attributes
 * 
 * (c) 2014 PTV AG
 */
import java.awt.Dimension;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.RoadEditorLayer;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class MappingWithTruckAttributes extends JPanel {
	private static Dimension imageSize = new Dimension(640, 480);
	JLabel label = new JLabel("Loading ...");

	private void runSample() {
		// Step 1: Instantiate client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface client = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Create CallerContext and use profile "roadattributes"
		// create object CallerContext
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Mapping with Truck Attributes");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("Profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 3: Define class mapSection
		MapSection mapSection = new MapSection();
		Point point = new Point();
		point.setPoint(new PlainPoint(681490.0, 6372500.0, null));
		mapSection.setCenter(point);
		mapSection.setScale(150);

		// Step 4: Set imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setHeight(imageSize.width);
		imageInfo.setWidth(imageSize.height);
		imageInfo.setImageParameter("");

		// Step 5: Specify mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT");

		// Step 6: Set value to false
		boolean includeImageInResponse = false;

		// ######################################################################################
		// Copy the map-dependent files such as truck_attributes_<version>.bin into
		// the "truckattributes" folder which should be added into the
		// <root-directory>/re folder before.
		// Otherwise an xMap exception will be thrown!
		// ######################################################################################

		// Step 7: Add layers
		RoadEditorLayer reLayer = new RoadEditorLayer();
		// Demo version: maps.path=data/map is enabled.
		// re/truckattributes folder is already available --> see folder data/map/re
		reLayer.setName("truckattributes");
		reLayer.setVisible(true);
		reLayer.setObjectInfos(ObjectInfoType.NONE);
		Layer[] layers = new Layer[] { reLayer };

		try {
			// Step 8: Call method renderMap
			Map map = client.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse);

			// retrieve the image
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));

			label.setIcon(icon);
			label.setText("");
			this.setMinimumSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public MappingWithTruckAttributes() {
		this.add(label);
		Thread thread = new Thread() {

			public void run() {
				runSample();
			}
		};
		thread.start();
	}

	public static void main(String[] args) {
		// display image in window
		JFrame frame = new JFrame("Mapping with Truck Attributes");
		JPanel sample = new MappingWithTruckAttributes();
		frame.getContentPane().add(sample);
		frame.setVisible(true);
		frame.setSize(imageSize.height, imageSize.width);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

}
