/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All samples are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG, Karlsruhe Germany
 ******************************************************************************/
package samples.xmap.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Rendering with bounding box
 * 
 * (c) 2014 PTV AG
 */

import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

public class RenderMapBoundingBox {

	private static void runSample(JLabel jLabel) {
		JFrame frame = new JFrame("Render Map with Bounding Box");

		frame.getContentPane().add(jLabel);
		frame.setVisible(true);
		frame.pack();
		frame.setSize(340, 280);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public static void main(String[] args) {
		Map map = renderMapBoundingBox();

		if (map == null) {
			System.out.println("Object map returns null!");
			return;
		}

		JLabel jLabel = drawMap(map);

		if (jLabel == null) {
			System.out.println("Object jLabel returns null!");
			return;
		}

		runSample(jLabel);
	}

	private static CallerContext getCallerContext() {
		CallerContext ctx = new CallerContext();
		ctx.setLog1("Basic Mapping");
		// Set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		ctx.setProperties(properties);

		return ctx;
	}

	private static BoundingBox setBoundingBox(double xl, double yl, double xr, double yr) {
		PlainPoint plainLeft = new PlainPoint();
		plainLeft.setX(xl);
		plainLeft.setY(yl);

		Point leftTop = new Point();
		leftTop.setPoint(plainLeft);

		PlainPoint plainRight = new PlainPoint();
		plainRight.setX(xr);
		plainRight.setY(yr);

		Point rightBottom = new Point();
		rightBottom.setPoint(plainRight);

		BoundingBox boundingBox = new BoundingBox();
		boundingBox.setLeftTop(leftTop);
		boundingBox.setRightBottom(rightBottom);

		return boundingBox;
	}

	private static ImageInfo setImageInfo(int width, int height) {
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(height);
		imageInfo.setWidth(width);
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setImageParameter("");

		return imageInfo;
	}

	private static MapParams setMapParams(boolean scale, boolean miles) {
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(scale);
		mapParams.setUseMiles(miles);

		return mapParams;
	}

	private static boolean getImageInResponse() {
		return false;
	}

	private static Map renderMapBoundingBox() {
		// Step 1: Create xMap client.
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapclient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Set CallerContext
		xMapclient.setCallerContext(getCallerContext());

		// Step 3: Set mapSection
		BoundingBox mapSection = setBoundingBox(677712.4, 6378309.4, 693103.0, 6366766.5);

		// Step 4: Set ImageInfo
		ImageInfo imageInfo = setImageInfo(320, 240);

		// Step 5: Set MapParams
		MapParams mapParams = setMapParams(true, false);

		// Step 6: Get the boolean value
		boolean includeImageInResponse = getImageInResponse();

		// Step 7: Call renderMap()
		Map map = null;
		try {
			map = xMapclient.renderMapBoundingBox(mapSection, mapParams, imageInfo, null, includeImageInResponse);
		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}
		return map;
	}

	private static JLabel drawMap(Map map) {
		// Step 8: Get image
		String imageUrl = "http://" + map.getImage().getUrl();
		JPanel jPanel = new JPanel();
		JLabel jLabel = new JLabel();
		try {
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			jLabel.setIcon(icon);
			jLabel.setText("");
			jPanel = new JPanel();
			jPanel.setMinimumSize(new Dimension(icon.getIconWidth() + 20, icon.getIconHeight() + 30));
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return jLabel;
	}

}
