/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xmapmatch;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - Extended Map Matching
 * 
 * (c) 2014 PTV AG
 */

import java.net.URL;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XMapMatchUtils;
import utils.XMapUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.jabba.service.baseservices.ClientFactory;
import com.ptvag.jabba.service.baseservices.RemoteType;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainLineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xmapmatch.MatchResult;
import com.ptvag.xserver.xmapmatch.MatchedLocation;
import com.ptvag.xserver.xmapmatch.MatchedSegment;
import com.ptvag.xserver.xmapmatch.ResultListOptions;
import com.ptvag.xserver.xmapmatch.TrackPosition;
import com.ptvag.xserver.xmapmatch.XMapmatchException;
import com.ptvag.xserver.xmapmatch.XMapmatchRemoteInterface;

public class MatchPositionswithUPM extends JPanel {

	// Note: an error occurs if incorrect configured. Don't add attribute parameter="" into element <profile>.
	public static String codesnippet = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"
			+ "<Profile>"
			+ "<Common coordinateFormat=\"OG_GEODECIMAL\" language=\"en\" majorVersion=\"1\" minorVersion=\"0\">{0,1}"
			+ "</Common>"
			+ "<Mapmatching majorVersion=\"1\" minorVersion=\"0\" probabilityBonusForBestCandidate=\"0.8\" "
			+ "probabilityOutputThreshold=\"0.1\">{0,1}"
			+ "</Mapmatching>" + "</Profile>";

	private static MatchResult runMapMatching(MatchResult matchPositions) throws ParseException {

		// Step 1:
		// Instantiate remote interface for xMapmatch
		final String url = "http://localhost:50040/xmapmatch/ws/XMapmatch";
		XMapmatchRemoteInterface client = (XMapmatchRemoteInterface) ClientFactory.createClient(
				XMapmatchRemoteInterface.class, RemoteType.DOCSTYLE_CXF, "", "", url);

		// Step 2:
		// Load GPS-track
		Vector<String> gps = loadTrack();

		// Step 3:
		// Create object CallerContext and add the XML-snippet to modify specific
		// parameters
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Match Positions");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("OG_GEODECIMAL");
		CallerContextProperty props3 = new CallerContextProperty();
		props3.setKey("ProfileXMLSnippet");
		props3.setValue(codesnippet);
		// add context properties
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2, props3 };
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// Step 4:
		// Create object trackPositions
		int max = gps.size();
		TrackPosition[] trackPositions = new TrackPosition[max];

		Enumeration<String> e = gps.elements();
		int index = 0;
		while (e.hasMoreElements()) {
			String arg0 = e.nextElement();
			TrackPosition trackPosition = new TrackPosition();
			Vector c = XMapMatchUtils.getSubStrings(arg0);
			Enumeration<String> eTrack = c.elements();

			String arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			int id = XMapMatchUtils.stringtoint(arg1);
			trackPosition.setId(id);
			arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			float heading = XMapMatchUtils.stringtofloat(arg1);
			trackPosition.setHeading(heading);
			arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			float speedInMps = XMapMatchUtils.stringtofloat(arg1);
			trackPosition.setSpeedInMps(speedInMps);
			arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			Calendar timestamp = XMapMatchUtils.stringtocalendar(arg1);
			trackPosition.setTimestamp(timestamp);
			arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			double x = XMapMatchUtils.stringtodouble(arg1);
			trackPosition.setLon(x);
			arg1 = eTrack.nextElement();
			// System.out.println(arg1);
			double y = XMapMatchUtils.stringtodouble(arg1);
			trackPosition.setLat(y);

			trackPositions[index] = trackPosition;
			index++;
		}

		// Step 5:
		// Define options for the result list
		ResultListOptions details = new ResultListOptions();
		details.setPolyline(true);
		details.setSpeedClass(true);
		details.setVendorId(true);

		try {
			// Step 6:
			// Execute matchTrackExtended
			matchPositions = client.matchPositions(trackPositions, details);

			// Step 7:
			// Create table and add result
			System.out.println("Total distance: " + (int) matchPositions.getOverallLength() + "m");
			System.out.println("Total time    : " + matchPositions.getOverallTime() + "s");

		} catch (SystemException se) {
			System.err.println("Network or URL problem");
		} catch (XMapmatchException xmm) {
			System.err.println("Problem with GPS track");
			System.out.println("Exception:\n" + xmm);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
			System.out.println("Exception:\n" + xse);
		}
		return matchPositions;
	}

	private static void runMap(MatchResult matchPositions) {
		// Step 1:
		// Initialize xmap client
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xmapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2:
		// Set CallerContext
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Basic Map Matching");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("OG_GEODECIMAL");
		CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		cxt.setProperties(properties);
		xmapClient.setCallerContext(cxt);

		// Step 3:
		// Create imageInfo and set relevant attributes
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(600);
		imageInfo.setWidth(600);
		imageInfo.setImageParameter("");

		// Step 4:
		// Realize object mapParams showing scale without use of miles.
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 5.1:
		// Create a bitmap array and add points from the given GPS track
		Vector<String> gps = loadTrack();
		int max = gps.size();
		Enumeration<String> e = gps.elements();
		Bitmap[] bitmapArray = new Bitmap[max];
		int index = 0;
		int k = 0;
		while (e.hasMoreElements()) {
			String arg0 = e.nextElement();
			Vector c = XMapMatchUtils.getSubStrings(arg0);
			Enumeration<String> eTrack = c.elements();
			eTrack.nextElement();
			eTrack.nextElement();
			eTrack.nextElement();
			eTrack.nextElement();
			String lon = eTrack.nextElement();
			String lat = eTrack.nextElement();
			Point point = new Point();
			point = XMapMatchUtils.stringtopoint(lon, lat);
			if (index == 1 * k) {
				bitmapArray[index] = new Bitmap();
				bitmapArray[index].setPosition(point);
				bitmapArray[index].setName("flaggreen.bmp");
				k++;
			}

			// System.out.println("Point: " + lon + ";" + lat);
			index++;
		}

		// Step 5.2:
		// Create a line from Polyline
		int lmax = matchPositions.getCompletePath().length;
		LineString[] lineStrings = new LineString[lmax];
		int lindex = 0;
		for (MatchedSegment matchedSegment : matchPositions.getCompletePath()) {
			int kmax = matchedSegment.getPolyline().length;
			PlainPoint[] plainPoints = new PlainPoint[kmax];
			int kindex = 0;
			for (Point plainPoint : matchedSegment.getPolyline()) {
				plainPoints[kindex] = plainPoint.getPoint();
				// System.out.println("Point: " + plainPoints[kindex]);
				kindex++;
			}

			PlainLineString lineString = new PlainLineString();
			lineString.setPoints(plainPoints);
			lineStrings[lindex] = new LineString();
			lineStrings[lindex].setLineString(lineString);
			lindex++;
		}

		Lines[] lines = new Lines[1];
		LineOptions options = new LineOptions();
		LinePartOptions partoptions = new LinePartOptions();
		partoptions.setColor(new Color(0, 0, 255));
		partoptions.setVisible(true);
		partoptions.setWidth(-6);
		options.setMainLine(partoptions);

		lines[0] = new Lines(lineStrings, options);

		// Step 6:
		// Set bitmapOptions
		BitmapOptions bitmapOptions = new BitmapOptions();
		// change the reference point for the pixel position of the bitmap
		// by default the bitmap center will be used.
		// here: bitmap displacement of 20 pixels.
		PlainPoint referencePoint = new PlainPoint(20, 20, null);
		bitmapOptions.setReferencePoint(referencePoint);
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 7:
		// Add bitmapArray and bitmapOptions into bitmaps
		Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapArray, bitmapOptions) };

		// Step 8:
		// Initialize customLayer with the relevant settings
		CustomLayer customLayer = new CustomLayer();
		customLayer.setCenterObjects(true);
		customLayer.setDrawPriority(100);
		customLayer.setName("ObjectLayer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setLines(lines);
		customLayer.setBitmaps(bitmaps);

		// Step 9:
		// Put customLayer into layers
		Layer[] layers = new Layer[] { customLayer };

		// Step 10:
		// Define boolean value including image in response
		boolean includeImageInResponse = false;

		try {
			// Step 11:
			// Return map object with its characteristics
			Map map = xmapClient.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse);

			// Step 12:
			// Retrieve icon
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));

			// Step 13:
			// Display image in window
			JFrame frame = new JFrame("Sample Match Positions");
			frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (Exception exc) {
			// proper exception handling: see API for PTV's exception handling!
			exc.printStackTrace();
		}

	}

	private static Vector<String> loadTrack() {
		Vector<String> h = new Vector<String>();
		// Use OG_Geodecimal
		h.addElement("1;251.065;5.00004;2011-07-06T07:57:03.000+0200;6.114670;49.626195");
		h.addElement("2;251.065;5.00004;2011-07-06T07:57:04.000+0200;6.114605;49.626182");
		h.addElement("3;251.065;5.00004;2011-07-06T07:57:05.000+0200;6.114538;49.626167");
		h.addElement("4;251.065;5.00004;2011-07-06T07:57:06.000+0200;6.114473;49.626152");
		h.addElement("5;251.065;5.00004;2011-07-06T07:57:07.000+0200;6.114408;49.626137");
		h.addElement("6;251.065;5.00004;2011-07-06T07:57:08.000+0200;6.114342;49.626123");
		h.addElement("7;251.065;5.00004;2011-07-06T07:57:09.000+0200;6.114277;49.626108");
		h.addElement("8;251.065;5.00004;2011-07-06T07:57:10.000+0200;6.114210;49.626093");
		h.addElement("9;251.065;5.00004;2011-07-06T07:57:11.000+0200;6.114145;49.626078");
		h.addElement("10;251.065;5.00004;2011-07-06T07:57:12.000+0200;6.114080;49.626065");
		h.addElement("11;251.065;5.00004;2011-07-06T07:57:13.000+0200;6.114013;49.626050");
		h.addElement("12;251.065;5.00004;2011-07-06T07:57:14.000+0200;6.113948;49.626065");
		h.addElement("13;224.177;5.00004;2011-07-06T07:57:15.000+0200;6.113885;49.626018");
		h.addElement("14;224.177;5.00004;2011-07-06T07:57:16.000+0200;6.113837;49.625987");
		h.addElement("15;224.177;5.00004;2011-07-06T07:57:17.000+0200;6.113788;49.625953");
		h.addElement("16;162.053;5.00004;2011-07-06T07:57:18.000+0200;6.113772;49.625917");
		h.addElement("17;162.053;5.00004;2011-07-06T07:57:19.000+0200;6.113793;49.625873");
		h.addElement("18;162.053;5.00004;2011-07-06T07:57:20.000+0200;6.113815;49.625832");
		h.addElement("19;162.053;5.00004;2011-07-06T07:57:21.000+0200;6.113837;49.625788");
		h.addElement("20;162.053;5.00004;2011-07-06T07:57:22.000+0200;6.113857;49.625745");
		h.addElement("21;162.053;5.00004;2011-07-06T07:57:23.000+0200;6.113878;49.625703");
		h.addElement("22;162.053;5.00004;2011-07-06T07:57:24.000+0200;6.113900;49.625660");
		h.addElement("23;162.053;5.00004;2011-07-06T07:57:25.000+0200;6.113922;49.625612");
		h.addElement("24;162.053;5.00004;2011-07-06T07:57:26.000+0200;6.113943;49.625575");
		h.addElement("25;162.053;5.00004;2011-07-06T07:57:27.000+0200;6.113965;49.625532");
		h.addElement("26;162.053;5.00004;2011-07-06T07:57:28.000+0200;6.113985;49.625549");
		h.addElement("27;162.053;5.00004;2011-07-06T07:57:29.000+0200;6.114007;49.625447");
		h.addElement("28;147.066;5.00004;2011-07-06T07:57:30.000+0200;6.114033;49.625405");
		h.addElement("29;147.066;5.00004;2011-07-06T07:57:31.000+0200;6.114072;49.625367");
		h.addElement("30;147.066;5.00004;2011-07-06T07:57:32.000+0200;6.114110;49.625330");
		h.addElement("31;147.066;5.00004;2011-07-06T07:57:33.000+0200;6.114147;49.625292");
		h.addElement("32;147.066;5.00004;2011-07-06T07:57:34.000+0200;6.114185;49.625253");
		h.addElement("33;150.959;5.00004;2011-07-06T07:57:35.000+0200;6.114220;49.625215");
		h.addElement("34;150.959;5.00004;2011-07-06T07:57:36.000+0200;6.114253;49.625175");
		h.addElement("35;150.959;5.00004;2011-07-06T07:57:37.000+0200;6.114287;49.625137");
		h.addElement("36;150.959;5.00004;2011-07-06T07:57:38.000+0200;6.114320;49.625097");
		h.addElement("37;150.959;5.00004;2011-07-06T07:57:39.000+0200;6.114355;49.625058");
		h.addElement("38;150.959;5.00004;2011-07-06T07:57:40.000+0200;6.114388;49.625018");
		h.addElement("39;150.959;5.00004;2011-07-06T07:57:41.000+0200;6.114422;49.624978");
		h.addElement("40;148.972;5.00004;2011-07-06T07:57:42.000+0200;6.114455;49.624940");
		h.addElement("41;148.972;5.00004;2011-07-06T07:57:43.000+0200;6.114492;49.624902");
		h.addElement("42;148.972;5.00004;2011-07-06T07:57:44.000+0200;6.114527;49.624863");
		h.addElement("43;148.972;5.00004;2011-07-06T07:57:45.000+0200;6.114563;49.624825");
		h.addElement("44;148.972;5.00004;2011-07-06T07:57:46.000+0200;6.114598;49.624785");
		h.addElement("45;148.972;5.00004;2011-07-06T07:57:47.000+0200;6.114635;49.624747");
		h.addElement("46;148.972;5.00004;2011-07-06T07:57:48.000+0200;6.114671;49.624708");
		h.addElement("47;148.972;5.00004;2011-07-06T07:57:49.000+0200;6.114705;49.624670");
		h.addElement("48;148.972;5.00004;2011-07-06T07:57:50.000+0200;6.114742;49.624632");
		h.addElement("49;148.972;5.00004;2011-07-06T07:57:51.000+0200;6.114777;49.624593");
		h.addElement("50;148.972;5.00004;2011-07-06T07:57:52.000+0200;6.114813;49.624555");
		h.addElement("51;148.972;5.00004;2011-07-06T07:57:53.000+0200;6.114848;49.624517");
		h.addElement("52;148.972;5.00004;2011-07-06T07:57:54.000+0200;6.114885;49.624477");
		h.addElement("53;148.972;5.00004;2011-07-06T07:57:55.000+0200;6.114920;49.624438");
		h.addElement("54;148.972;5.00004;2011-07-06T07:57:56.000+0200;6.114957;49.624400");
		h.addElement("55;148.972;5.00004;2011-07-06T07:57:57.000+0200;6.114992;49.624362");
		h.addElement("56;154.087;5.00004;2011-07-06T07:57:58.000+0200;6.115027;49.624323");
		h.addElement("57;154.087;5.00004;2011-07-06T07:57:59.000+0200;6.115057;49.624282");
		h.addElement("58;154.087;5.00004;2011-07-06T07:58:00.000+0200;6.115087;49.624242");
		h.addElement("59;154.087;5.00004;2011-07-06T07:58:01.000+0200;6.115117;49.624202");
		h.addElement("60;154.087;5.00004;2011-07-06T07:58:02.000+0200;6.115147;49.624162");
		h.addElement("61;154.087;5.00004;2011-07-06T07:58:03.000+0200;6.115178;49.624120");
		h.addElement("62;154.087;5.00004;2011-07-06T07:58:04.000+0200;6.115208;49.624080");
		h.addElement("63;154.087;5.00004;2011-07-06T07:58:05.000+0200;6.115238;49.624040");
		h.addElement("64;154.087;5.00004;2011-07-06T07:58:06.000+0200;6.115268;49.623998");
		h.addElement("65;142.919;5.00004;2011-07-06T07:58:07.000+0200;6.115308;49.623962");
		h.addElement("66;142.919;5.00004;2011-07-06T07:58:08.000+0200;6.115350;49.623927");
		h.addElement("67;142.919;5.00004;2011-07-06T07:58:09.000+0200;6.115392;49.623890");
		h.addElement("68;142.919;5.00004;2011-07-06T07:58:10.000+0200;6.115433;49.623855");
		h.addElement("69;142.919;5.00004;2011-07-06T07:58:11.000+0200;6.115475;49.623818");
		h.addElement("70;142.919;5.00004;2011-07-06T07:58:12.000+0200;6.115517;49.623783");
		h.addElement("71;142.919;5.00004;2011-07-06T07:58:13.000+0200;6.115558;49.623747");
		h.addElement("72;142.919;5.00004;2011-07-06T07:58:14.000+0200;6.115602;49.623712");
		h.addElement("73;142.919;5.00004;2011-07-06T07:58:15.000+0200;6.115643;49.623675");
		h.addElement("74;142.919;5.00004;2011-07-06T07:58:16.000+0200;6.115685;49.623640");
		h.addElement("75;142.919;5.00004;2011-07-06T07:58:17.000+0200;6.115727;49.623603");
		h.addElement("76;142.919;5.00004;2011-07-06T07:58:18.000+0200;6.115768;49.623568");
		h.addElement("77;142.919;5.00004;2011-07-06T07:58:19.000+0200;6.115810;49.623532");
		h.addElement("78;142.919;5.00004;2011-07-06T07:58:20.000+0200;6.115852;49.623495");
		h.addElement("79;144.254;5.00004;2011-07-06T07:58:21.000+0200;6.115893;49.623460");
		h.addElement("80;144.254;5.00004;2011-07-06T07:58:22.000+0200;6.115933;49.623423");
		h.addElement("81;144.254;5.00004;2011-07-06T07:58:23.000+0200;6.115975;49.623387");
		h.addElement("82;144.254;5.00004;2011-07-06T07:58:24.000+0200;6.116015;49.623350");
		h.addElement("83;144.254;5.00004;2011-07-06T07:58:25.000+0200;6.116055;49.623313");
		h.addElement("84;144.254;5.00004;2011-07-06T07:58:26.000+0200;6.116097;49.623277");
		h.addElement("85;144.254;5.00004;2011-07-06T07:58:27.000+0200;6.116137;49.623240");
		h.addElement("86;144.254;5.00004;2011-07-06T07:58:28.000+0200;6.116177;49.623205");
		h.addElement("87;144.254;5.00004;2011-07-06T07:58:29.000+0200;6.116218;49.623168");
		h.addElement("88;144.254;5.00004;2011-07-06T07:58:30.000+0200;6.116258;49.623132");
		h.addElement("89;144.254;5.00004;2011-07-06T07:58:31.000+0200;6.116298;49.623095");
		h.addElement("90;144.526;5.00004;2011-07-06T07:58:32.000+0200;6.116340;49.623058");
		h.addElement("91;144.526;5.00004;2011-07-06T07:58:33.000+0200;6.116380;49.623022");
		h.addElement("92;144.526;5.00004;2011-07-06T07:58:34.000+0200;6.116420;49.622985");
		h.addElement("93;144.526;5.00004;2011-07-06T07:58:35.000+0200;6.116460;49.622948");
		h.addElement("94;144.526;5.00004;2011-07-06T07:58:36.000+0200;6.116500;49.622912");
		h.addElement("95;144.526;5.00004;2011-07-06T07:58:37.000+0200;6.116540;49.622875");
		h.addElement("96;144.526;5.00004;2011-07-06T07:58:38.000+0200;6.116582;49.622838");
		h.addElement("97;144.526;5.00004;2011-07-06T07:58:39.000+0200;6.116662;49.622765");
		h.addElement("98;144.526;5.00004;2011-07-06T07:58:40.000+0200;6.116702;49.622728");
		h.addElement("99;144.526;5.00004;2011-07-06T07:58:41.000+0200;6.116742;49.622692");
		h.addElement("100;144.526;5.00004;2011-07-06T07:58:42.000+0200;6.116782;49.622655");
		h.addElement("101;144.526;5.00004;2011-07-06T07:58:43.000+0200;6.116815;49.622615");
		h.addElement("102;154.086;5.00004;2011-07-06T07:58:44.000+0200;6.116845;49.622575");
		h.addElement("103;154.086;5.00004;2011-07-06T07:58:45.000+0200;6.116875;49.622535");
		h.addElement("104;154.086;5.00004;2011-07-06T07:58:46.000+0200;6.116905;49.622495");
		h.addElement("105;154.086;5.00004;2011-07-06T07:58:47.000+0200;6.117033;49.622423");
		h.addElement("106;154.086;5.00004;2011-07-06T07:58:51.000+0200;6.117127;49.622333");
		h.addElement("107;147.064;5.00004;2011-07-06T07:58:56.000+0200;6.117210;49.622142");
		h.addElement("108;147.064;5.00004;2011-07-06T07:59:02.000+0200;6.117437;49.621915");
		h.addElement("109;147.064;5.00004;2011-07-06T07:59:07.000+0200;6.117625;49.621727");
		h.addElement("110;142.561;5.00004;2011-07-06T07:59:13.000+0200;6.117870;49.621508");
		h.addElement("111;142.561;5.00004;2011-07-06T07:59:20.000+0200;6.118207;49.621223");
		h.addElement("112;147.063;5.00004;2011-07-06T07:59:27.000+0200;6.118440;49.620998");
		h.addElement("113;147.063;5.00004;2011-07-06T07:59:34.000+0200;6.118703;49.620735");
		h.addElement("114;147.063;5.00004;2011-07-06T07:59:39.000+0200;6.118892;49.620547");
		h.addElement("115;130.614;5.00004;2011-07-06T07:59:46.000+0200;6.119220;49.620318");
		h.addElement("116;127.660;5.00004;2011-07-06T07:59:52.000+0200;6.119543;49.620148");
		h.addElement("117;127.660;5.00004;2011-07-06T07:59:57.000+0200;6.119818;49.620010");
		h.addElement("118;127.660;5.00004;2011-07-06T08:00:04.000+0200;6.120201;49.619818");
		h.addElement("119;130.614;5.00004;2011-07-06T08:00:12.000+0200;6.120630;49.619589");
		h.addElement("120;130.614;5.00004;2011-07-06T08:00:17.000+0200;6.120893;49.619442");
		h.addElement("121;130.614;5.00004;2011-07-06T08:00:22.000+0200;6.121208;49.619267");
		h.addElement("122;108.657;5.00004;2011-07-06T08:00:29.000+0200;6.121530;49.619097");
		h.addElement("123;108.657;5.00004;2011-07-06T08:00:34.000+0200;6.122193;49.619010");
		h.addElement("124;108.657;5.00004;2011-07-06T08:00:39.000+0200;6.122225;49.618938");
		h.addElement("125;108.657;5.00004;2011-07-06T08:00:44.000+0200;6.122582;49.618852");
		h.addElement("126;126.755;5.00004;2011-07-06T08:00:49.000+0200;6.122908;49.618790");
		h.addElement("127;126.755;5.00004;2011-07-06T08:00:54.000+0200;6.123185;49.618655");
		h.addElement("128;126.755;5.00004;2011-07-06T08:00:59.000+0200;6.123463;49.618522");
		h.addElement("129;126.755;5.00004;2011-07-06T08:01:04.000+0200;6.123853;49.618333");
		h.addElement("130;126.755;5.00004;2011-07-06T08:01:09.000+0200;6.124132;49.618198");
		h.addElement("131;132.803;5.00004;2011-07-06T08:01:14.000+0200;6.124343;49.618052");
		h.addElement("132;134.777;5.00004;2011-07-06T08:01:19.000+0200;6.124760;49.617870");
		h.addElement("133;135.818;5.00004;2011-07-06T08:01:24.000+0200;6.125307;49.617462");
		h.addElement("134;130.613;5.00004;2011-07-06T08:01:29.000+0200;6.125632;49.617265");
		h.addElement("135;233.566;5.00004;2011-07-06T08:01:34.000+0200;6.125691;49.616873");
		h.addElement("136;233.566;5.00004;2011-07-06T08:01:39.000+0200;6.125246;49.616702");
		h.addElement("137;233.566;5.00004;2011-07-06T08:01:44.000+0200;6.124800;49.616488");
		h.addElement("138;233.566;5.00004;2011-07-06T08:01:49.000+0200;6.124353;49.616275");
		h.addElement("139;233.566;5.00004;2011-07-06T08:01:54.000+0200;6.123907;49.616062");
		h.addElement("140;235.218;5.00004;2011-07-06T08:01:59.000+0200;6.123362;49.615853");
		h.addElement("141;235.218;5.00004;2011-07-06T08:02:04.000+0200;6.122998;49.615648");
		h.addElement("142;242.775;5.00004;2011-07-06T08:02:09.000+0200;6.122513;49.615474");
		h.addElement("143;240.697;5.00004;2011-07-06T08:02:14.000+0200;6.122022;49.615307");
		h.addElement("144;240.697;5.00004;2011-07-06T08:02:19.000+0200;6.121633;49.615105");
		h.addElement("145;211.543;5.00004;2011-07-06T08:02:24.000+0200;6.121343;49.614860");
		h.addElement("146;211.543;5.00004;2011-07-06T08:02:29.000+0200;6.121105;49.614553");
		h.addElement("147;212.941;5.00004;2011-07-06T08:02:34.000+0200;6.120698;49.614188");
		h.addElement("148;212.941;5.00004;2011-07-06T08:02:39.000+0200;6.120467;49.613947");
		h.addElement("149;212.941;5.00004;2011-07-06T08:02:44.000+0200;6.120098;49.613583");
		h.addElement("150;212.941;5.00004;2011-07-06T08:02:49.000+0200;6.119885;49.613330");

		return h;
	}

	private static Vector<Object[][]> runMapMatchingResult(MatchResult matchPositions,
			Vector<Object[][]> jMapMatchingTable) {

		int index = 0;
		int maxMatchedLocations = matchPositions.getMatchedLocations().length;

		Object[][] rows = new Object[maxMatchedLocations][8];

		for (MatchedLocation matchedLocation : matchPositions.getMatchedLocations()) {

			rows[index][0] = matchedLocation.getInputId();

			for (MatchedSegment matchedSegment : matchedLocation.getPath()) {

				rows[index][1] = matchedSegment.getTileId();
				rows[index][2] = matchedSegment.getCountryCode();
				rows[index][3] = matchedSegment.getNetworkClass();
				rows[index][4] = matchedSegment.getSpeedClass();
				rows[index][5] = matchedSegment.getVendorId();

				String startXYN = "[ ";
				for (int num1 : matchedSegment.getStartXYN()) {
					startXYN = startXYN + num1 + " ";
				}
				startXYN = startXYN + "]";
				rows[index][6] = startXYN;

				String endXYN = "[ ";
				for (int num1 : matchedSegment.getEndXYN()) {
					endXYN = endXYN + num1 + " ";
				}
				endXYN = endXYN + "]";
				rows[index][7] = endXYN;
			}

			index++;
		}

		System.out.println("Number of matched locations: " + maxMatchedLocations);
		jMapMatchingTable.addElement(rows);

		return jMapMatchingTable;
	}

	public static void main(String[] args) {
		// Step 1: Call MapMatchingBasic and execute runMap()
		MatchResult matchPositions = null;
		try {
			matchPositions = runMapMatching(matchPositions);
			runMap(matchPositions);
		} catch (ParseException e) {
			e.printStackTrace();
		}

		// Step 2: Create result table jMapSegments
		Vector<Object[][]> jMapSegments = new Vector<Object[][]>();
		jMapSegments = runMapMatchingResult(matchPositions, jMapSegments);

		// Step 3: Visualize jTableSegments
		JTable jTableSegments = new JTable(jMapSegments.firstElement(), new Object[] { "Input ID", "Tile Id", "IU",
				"Network Class", "Speed Class", "VendorID", "StartXYN", "EndXYN" });
		JScrollPane jScrollPaneSegments = new JScrollPane(jTableSegments);

		JFrame jFrameSegments = new JFrame("Result List: Matched Locations");
		jFrameSegments.getContentPane().add(jScrollPaneSegments);
		jFrameSegments.setVisible(true);
		jFrameSegments.setSize(700, 630);
		jFrameSegments.setLocation(620, 0);
		jFrameSegments.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

}
