/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingAvoidCountries {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// set a HTTP proxy if required
		/*
		 * System.getProperties().setProperty("http.proxySet", "true");
		 * System.getProperties().setProperty("http.proxyHost","localhost");
		 * System.getProperties().setProperty("http.proxyPort", "8888");
		 */

		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);
		XRouteRemoteInterface xRouteClient = XRouteUtils
				.makeClient(XROUTE_WS_URL);

		// #################################################################
		// add routing options to whitelist or blacklist countries.
		// Countries are referenceed by their IU code

		RoutingOption[] routingOption = new RoutingOption[] {
		// new RoutingOption( RoutingParameter.ROUTING_COUNTRIES, "49,33") //
		// Germany & France
		new RoutingOption(RoutingParameter.EXCLUDE_COUNTRIES, "52") // avoid
																	// Luxemburg
		};
		// #################################################################

		// -------------------------------------------------
		// Defining waypoints

		// start
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(524730.47, 6397686.86, null));

		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords(new Point[] { startPoint });
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);

		// destination
		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(737927.84, 6397192.86, null));

		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords(new Point[] { destPoint });
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);

		WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

		// -------------------------------------------------

		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		try {
			Route route = xRouteClient.calculateRoute(waypointDescs,
					routingOption, /*
									 * contains ROUTING_COUNTRIES or
									 * EXCLUDE_COUNTRIES
									 */
					null, resultListOptions);

			// -- got the routes, now create a map ...

			// use bounding box of route
			BoundingBox boundingBox = new BoundingBox();
			boundingBox.setLeftTop(route.getTotalRectangle().getLeftBottom());
			boundingBox.setRightBottom(route.getTotalRectangle().getRightTop());

			// get "polygon" from route
			LineString[] lineStrings = new LineString[] { route.getPolygon() };

			// set line options ( color, stroke size )
			LineOptions options = new LineOptions();

			LinePartOptions linePartOpt = new LinePartOptions();
			linePartOpt.setColor(new Color(255, 0, 0));
			linePartOpt.setVisible(true);
			linePartOpt.setWidth(-6);

			options.setShowFlags(true);

			Lines[] linesArray = new Lines[] { new Lines(lineStrings, options) };

			// put lines into a CustomLayer
			CustomLayer customLayer = new CustomLayer();
			customLayer.setName("routeLayer");
			customLayer.setLines(linesArray);
			customLayer.setDrawPriority(90); // this is PRIORROUTE
			customLayer.setCenterObjects(true);
			customLayer.setObjectInfos(ObjectInfoType.FULLGEOMETRYCLIPPED);
			customLayer.setVisible(true);

			ImageInfo imageInfo = new ImageInfo();
			imageInfo.setHeight(600);
			imageInfo.setWidth(800);
			imageInfo.setImageParameter("");
			imageInfo.setFormat(ImageFileFormat.PNG);

			MapParams mapParams = new MapParams();
			mapParams.setShowScale(false);
			mapParams.setUseMiles(false);
			mapParams.setLanguage("x-ptv-DFT");

			Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams,
					imageInfo, new Layer[] { customLayer }, false);

			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			System.err.println(imageUrl);

			// display image in window
			JFrame frame = new JFrame("Sample Map");
			frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);

			// exit when window is closed
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}
	}
}
