/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

/**
 * <p>
 * This example shows you how to assign a malus or bonus to certain streets. An
 * assigned bonus will favor a street in the subsequent route calculation and a
 * malus will penalize the usage of a street in the route. The figure on the
 * left illustrates the effect on the route calculation if you assign a bonus to
 * nearby street.
 * </p>
 * 
 * <p>
 * For this example we are going to use the calculateRoute call introduced in
 * xRoute Sample02 and use the ExceptionPath parameter of the call. In xRoute
 * Sample04 you have already learned how to use an ExceptionPath to calculate an
 * alternative route. In this example we use the ExceptionPath in a similar
 * fashion, though the path is not specified by a calculated route but given
 * through a street name.
 * </p>
 * 
 * @see RoutingWithItinerary
 * @see RoutingWithAlternative
 * 
 */
public class RoutingAvoidStreets {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// set a HTTP proxy if required
		/*
		 * System.getProperties().setProperty("http.proxySet", "true");
		 * System.getProperties().setProperty("http.proxyHost","localhost");
		 * System.getProperties().setProperty("http.proxyPort", "8888");
		 */

		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);
		XRouteRemoteInterface xRouteClient = XRouteUtils
				.makeClient(XROUTE_WS_URL);

		// -------------------------------------------------
		// 1.) Defining waypoints
		// Note: German map is required!

		// start
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(842716, 4900805, null));

		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords(new Point[] { startPoint });
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);

		// destination
		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(843793, 4900919, null));

		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords(new Point[] { destPoint });
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);

		WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };
		// -------------------------------------------------

		// -------------------------------------------------
		// 2. setting result list options
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		// #################################################
		// 3. Defining ExceptionPaths

		com.ptvag.xserver.xroute.ExceptionPath[] exceptionPaths = new com.ptvag.xserver.xroute.ExceptionPath[] { new com.ptvag.xserver.xroute.ExceptionPath() };

		// block street
		exceptionPaths[0].setStreet("Gerwigstrae");
		exceptionPaths[0].setRelMalus(2500);
		exceptionPaths[0].setAbsTimeMalus(-1);

		// you can list multiple streets like this
		//exceptionPaths[0].setStreet("Gerwigstrae/Tullastrasse/Esseweinstrasse"
		// );

		// OR

		// strongly prefer street
		/*
		 * exceptionPaths[0].setStreet("Striederstrae");
		 * exceptionPaths[0].setRelMalus(-100);
		 * exceptionPaths[0].setAbsTimeMalus(-1);
		 */

		// #################################################
		// set coordinate format in caller context to GeoDecimal
		CallerContextProperty coordFormat = new CallerContextProperty(
				"CoordFormat", "PTV_GEODECIMAL");
		CallerContext callerContext = new CallerContext();
		callerContext
				.setProperties(new CallerContextProperty[] { coordFormat });

		// overwrite default callercontext
		xRouteClient.setCallerContext(callerContext);
		xMapClient.setCallerContext(callerContext);

		try {
			// Calculate route with exceptions ( streets with malus or bonus )
			Route route_withException = xRouteClient.calculateRoute(
					waypointDescs, null, exceptionPaths, resultListOptions);

			// calculate default route (without exceptions) for comparison
			Route defaultRoute = xRouteClient.calculateRoute(waypointDescs,
					null, null, resultListOptions);

			// -- got the routes, now create a map ...

			// use bounding box of route
			BoundingBox boundingBox = new BoundingBox();
			boundingBox.setLeftTop(route_withException.getTotalRectangle()
					.getLeftBottom());
			boundingBox.setRightBottom(route_withException.getTotalRectangle()
					.getRightTop());

			// get "polygon" from route
			LineString[] lineStrings1 = new LineString[] { route_withException
					.getPolygon() };

			// set line options ( color, stroke size )
			LineOptions options1 = new LineOptions();
			LinePartOptions linePartOpt1 = new LinePartOptions();
			linePartOpt1.setColor(new Color(0, 255, 0));
			linePartOpt1.setVisible(true);
			linePartOpt1.setWidth(-6);
			options1.setMainLine(linePartOpt1);

			// same with default route ... just different color
			LineString[] lineStrings2 = new LineString[] { defaultRoute
					.getPolygon() };
			LineOptions options2 = new LineOptions();
			LinePartOptions linePartOpt2 = new LinePartOptions();
			linePartOpt2.setColor(new Color(255, 255, 0));
			linePartOpt2.setVisible(true);
			linePartOpt2.setWidth(-6);
			options2.setMainLine(linePartOpt2);

			Lines[] linesArray = new Lines[] {
					new Lines(lineStrings1, options1),
					new Lines(lineStrings2, options2) };

			// put lines into a CustomLayer
			CustomLayer customLayer = new CustomLayer();
			customLayer.setName("routeLayer");
			customLayer.setLines(linesArray);
			customLayer.setDrawPriority(100); // this is PRIORROUTE
			customLayer.setCenterObjects(true);
			customLayer.setObjectInfos(ObjectInfoType.FULLGEOMETRYCLIPPED);
			customLayer.setVisible(true);

			ImageInfo imageInfo = new ImageInfo();
			imageInfo.setHeight(600);
			imageInfo.setWidth(800);
			imageInfo.setImageParameter("");
			imageInfo.setFormat(ImageFileFormat.PNG);

			MapParams mapParams = new MapParams();
			mapParams.setShowScale(false);
			mapParams.setUseMiles(false);
			mapParams.setLanguage("x-ptv-DFT");

			// call xMap web service
			Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams,
					imageInfo, new Layer[] { customLayer }, false);

			// create ImageIcon from image URL
			String imageUrl = "http://" + map.getImage().getUrl();
			ImageIcon icon = new ImageIcon(new URL(imageUrl));
			System.err.println(imageUrl);

			// display image in window
			JFrame frame = new JFrame("Sample Map");
			frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 30);
			frame.getContentPane().add(new JLabel(icon));
			frame.setVisible(true);

			// exit when window is closed
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
		}
	}
}
