/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import utils.XRouteUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.ExceptionPath;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithAlternative {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// set a HTTP proxy if required
		/*System.getProperties().setProperty("http.proxySet", "true");
		System.getProperties().setProperty("http.proxyHost","localhost");
		System.getProperties().setProperty("http.proxyPort", "8888");*/

		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		// start
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(681593.765751686, 6371793.192390807, null));
		
		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords( new Point[]{startPoint} );
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);
		
		// destination
		Point destPoint = new Point();
		destPoint.setPoint( new PlainPoint(678909.5202224851, 6395906.712571196, null) );
		
		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords( new Point[]{destPoint} );
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);
		
		WaypointDesc[] waypointDescs = new WaypointDesc[] {wpdStart, wpdDest};
		
		// we want a binary path description in the result,
		// because we need it later
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setBinaryPathDesc(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);
		
		try {
			// calculate a first route
			Route firstRoute = xRouteClient.calculateRoute(waypointDescs, null,
					null, resultListOptions);
			
			// take the calculated route and put it into an exception path
			// and set a malus to the supplied route
			ExceptionPath exceptionPath = new ExceptionPath();
			exceptionPath.setBinaryPathDesc( firstRoute.getBinaryPathDesc() );
			exceptionPath.setRelMalus(500);

			ExceptionPath[] arrayOfExceptionPaths = new ExceptionPath[]{ exceptionPath };
			
			// calculate a alternative route
			Route alternativeRoute = xRouteClient.calculateRoute(waypointDescs, null,
					arrayOfExceptionPaths, resultListOptions);
			
			// ... do something with the calculated routes ...
			System.out.println("Binary Path Description:\n" + alternativeRoute.getBinaryPathDesc());
		}catch(Exception e){
			// proper exception handling
			e.printStackTrace();
		}
	}

}
