/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - use bcr converter for guided navigation
 * 
 * (c) 2012 PTV AG
 */

import java.awt.Container;
import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.utils.bcr.BCRConverter;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithBCR {
	// Define the relevant arguments for BCRConverter.
	// Define maximum angle, maximum distance and minimum distance
	public static double maxangle = 3.0;
	public static int maxdistance = 50000;
	public static int mindistance = 10;

	public static void main(String[] args) {
		// call xRoute
		Route route = null;
		route = runRouting(route);

		// call xMap
		ImageIcon icon = null;
		icon = runMappingSample(route, icon);

		// call BCRConverter
		String bcr_export = runBCR(route);

		// draw map extract
		JFrame frameMap = new JFrame("Sample BCR export for guided navigation");
		frameMap.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 50);
		frameMap.getContentPane().add(new JLabel(icon));
		frameMap.setVisible(true);
		frameMap.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// draw BCR export
		JFrame frameBCR = new JFrame("Sample Basic BCR Export");
		frameBCR.setSize(600,650);
		Container cp = frameBCR.getContentPane();
		JTextArea jBCRExport = new JTextArea();
		String bcrExport = "<html><body>" + "Complete result list of a selected route." + "</body></html>";
		jBCRExport.setToolTipText(bcrExport);
		jBCRExport.setEditable(false);
		jBCRExport.setText(bcr_export);
		JScrollPane jBCRExportScrollPane = new JScrollPane(jBCRExport);
		jBCRExportScrollPane.setPreferredSize(new Dimension(600, 650));
		cp.add(jBCRExportScrollPane);
		frameBCR.setVisible(true);
		frameBCR.setLocation(820, 0);
		//frameBCR.setLocationRelativeTo(frameMap);
		frameBCR.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Route runRouting(Route route) {
		// Step 1: instantiate xRoute client. Use the packages utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2.1: Create callerContextProperty and set profile truckfast.
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");

		// Step 2.2: Set properties into callerContextProperties (type is defined as
		// an array)
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		// Step 2.3: Define callerContext, set properties and overwrite default
		// class in xRoute client
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(callerContextProperties);
		xRouteClient.setCallerContext(callerContext);

		// Step 3.1: define way-points <start> and <destination>
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(681593.76, 6371793.19, null));
		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords(new Point[] { startPoint });
		wpdStart.setLinkType(LinkType.AUTO_LINKING);

		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(678909.52, 6395906.716, null));
		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords(new Point[] { destPoint });
		wpdDest.setLinkType(LinkType.AUTO_LINKING);

		// Step 3.2: Set both way-points into the specific array WaypointDesc[]
		WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

		// Step 4: Set result list options
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		try {
			// Step 4: Call method calculateRoute and return result from object route
			route = xRouteClient.calculateRoute(waypointDescs, null, null, resultListOptions);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route" + xre);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return route;
	}

	private static ImageIcon runMappingSample(Route route, ImageIcon icon) {
		// Step 1: Initialize xMap client
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Create callerContextProperty and set profile default.
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("default");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(callerContextProperties);
		xMapClient.setCallerContext(callerContext);

		// Step 3: Define boundingBox as input for the mapping function
		BoundingBox boundingBox = new BoundingBox();
		boundingBox.setLeftTop(route.getTotalRectangle().getLeftBottom());
		boundingBox.setRightBottom(route.getTotalRectangle().getRightTop());

		// Step 4.1: Get "polygon" from route
		LineString[] lineStrings = new LineString[] { route.getPolygon() };

		// Step 4.2: Set line options ( color, stroke size )
		LineOptions options = new LineOptions();
		LinePartOptions linePartOpt = new LinePartOptions();
		linePartOpt.setColor(new Color(255, 0, 0));
		linePartOpt.setVisible(true);
		linePartOpt.setWidth(-6);
		options.setShowFlags(true);

		// Step 4.3: Put all together into array of lines
		Lines[] linesArray = new Lines[] { new Lines(lineStrings, options) };

		// Step 5.1: Define customLayer and put lines into it. Besides this, set
		// priority, center object, set information and visibility.
		CustomLayer customLayer = new CustomLayer();
		customLayer.setName("routeLayer");
		customLayer.setLines(linesArray);
		customLayer.setDrawPriority(90); // this is PRIORROUTE
		customLayer.setCenterObjects(true);
		customLayer.setObjectInfos(ObjectInfoType.FULLGEOMETRYCLIPPED);
		customLayer.setVisible(true);

		// Step 5.2: Create new object layers and put customLayer into it.
		Layer[] layers = new Layer[] { customLayer };

		// Step 6: Define imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(600);
		imageInfo.setWidth(800);
		imageInfo.setImageParameter("");
		imageInfo.setFormat(ImageFileFormat.PNG);

		// Step 7: Use mapParams for scale and distance unit
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		try {
			// Step 8: Call method renderMapBoundingBox
			Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, false);

			// Step 9: Define icon for mapping
			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				icon = new ImageIcon(new URL(imageUrl));
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap): " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server" + xse);
		}

		return icon;
	}

	private static String runBCR(Route route) {
		String bcr_export = "";
		try {
			/**
			 * Converts a xRoute Route object to a BCR string.
			 * The route is represented as polygon of Magnetic View Points (MVP) that are set in the middle of each segment and
			 * 	also contain the heading, length, and station index of the current segment
			 * @param route Route object
			 * @param maxangle Quality parameter for route tapering:
			 * 					polygon legs differing less than maxangle from the previous leg
			 * 					will be regarded as redundant (0 = no tapering, max 180 (degrees))
			 * @param maxdistance Quality parameter for route tapering: after maxdistance (meters) of redundant legs, an MVP will
			 * 					be added to ensure enough legs per route
			 * @param mindistance Minimal length of a polygon segment (to avoid many small segments e.g. in roundabouts)
			 * @return	BCR String that can be loaded into navigation tool
			 * @throws Exception On empty Route object or invalid parameter ranges
			 */
			bcr_export = BCRConverter.route2BCR(route, maxangle, maxdistance, mindistance);
		} catch (Exception ex) {
			bcr_export = "Converting failed";
		}
		return bcr_export;
	}
}
