/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.CountryInfo;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.ExtendedRoute;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.RouteInfo;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;


@SuppressWarnings("serial")
public class RoutingWithCountryInfos extends JPanel{

	final String[] networkClassNames = new String[] {
			 "MOTORWAY", "HIGHWAY", "TRUNK_ROAD",
			 "COUNTRY_ROAD", "CITY_ROAD", "RESIDENTIAL_ROAD",
			 "SPECIAL_ROAD", "CYCLE_AND_WALKWAY" };
	 
	public RoutingWithCountryInfos() {
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// start
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(681593.765751686, 6371793.192390807, null));

		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords( new Point[]{startPoint} );
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);

		// destination
		Point destPoint = new Point();
		destPoint.setPoint( new PlainPoint(678909.5202224851, 6395906.712571196, null) );

		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords( new Point[]{destPoint} );
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);

		WaypointDesc[] waypointDescs = new WaypointDesc[] {wpdStart, wpdDest};

		// Specify the ResultListOptions. They provide the information, e.g. how detailed the itinerary  
		// should be. 
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		try {
			ExtendedRoute route = xRouteClient
				.calculateExtendedRoute(waypointDescs, null, null, resultListOptions, null);

			Object[][] rows = 
				new Object[ route.getCountryInfos().length*8 ][3];
			
			int row_idx=0;
			
			for (CountryInfo info : route.getCountryInfos() ){
				final RouteInfo[] routeInfo = info.getPerNCRouteInfo();
				
				// get distance for each network class (street type)
				for (int j=0; j < networkClassNames.length; j++) {
					rows[row_idx++] = new Object[]{
							info.getIuCode(), 			// 1st column (countries IU code)
							networkClassNames[j], 		// 2nd column ( NC name ) 
							routeInfo[j].getDistance()  // 3rd column ( meters per NC )
							};
				}
			}
			
			JTable table = new JTable(rows, new Object[]{"Country (IU code)", "NetworkClass", "distance in m"});
			JScrollPane scrollPane = new JScrollPane(table);
			this.add(scrollPane);
	
		}catch(Exception e){
			// proper exception handling
			e.printStackTrace();
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		JPanel demoPanel = new RoutingWithCountryInfos();
		
		// display table in window
		JFrame frame = new JFrame("Distances per Network class (NC)");
		frame.setSize(500,200);
		frame.add(demoPanel);
		frame.setVisible(true);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
	}

}
