/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.CountryInfoVehicleOptions;
import com.ptvag.xserver.xroute.EmissionType;
import com.ptvag.xserver.xroute.EmissionLevel;
import com.ptvag.xserver.xroute.ExtendedRoute;
import com.ptvag.xserver.xroute.HBEFAType;
import com.ptvag.xserver.xroute.HBEFAVersion;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.VehicleOption;
import com.ptvag.xserver.xroute.VehicleParameter;
import com.ptvag.xserver.xroute.WayPoint;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithEmissions {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// set a HTTP proxy if required
		/*System.getProperties().setProperty("http.proxySet", "true");
		System.getProperties().setProperty("http.proxyHost","localhost");
		System.getProperties().setProperty("http.proxyPort", "8888");*/

		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		// We should use a truck profile to get some valuable toll cost information  
		// therefore we make use of the CallerContext  
		CallerContextProperty[] props = new CallerContextProperty[3];
		props[0] = new CallerContextProperty("Profile", "truckfast"); 
		props[1] = new CallerContextProperty("CoordFormat", "PTV_MERCATOR"); 
		props[2] = new CallerContextProperty("ResponseGeometry", "PLAIN"); 
		
		// create a CallerContext and add the property
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties( props );
		
		// overwrite default callercontext
		xRouteClient.setCallerContext(callerContext);
		
		
		double[][] points = new double[][]{
		//Luxembourg coordinates used for sample route
				{670182, 6464052},
				{682299, 6372572},
				{665180, 6354110},
				{670182, 6464052},
				{665180, 6354110}
				
		// If Europe City Map Premium is installed you can use the following coordinates:		
			/*  {  262175.0, 6242590.0}, // Paris
				{  933441.0, 6268969.0}, // Karlsruhe
				{ 1820467.0, 6134685.0}, //Wien
				{ 1267389.0, 5978788.0}, //Innsbruck
				{ 1262552.0, 5853474.0} //Bolzano*/
		};
		
		WaypointDesc[] waypointDescs  = new WaypointDesc[points.length];
		for (int i=0; i < points.length; i++) {
			Point pt = new Point();
			pt.setPoint(new PlainPoint(points[i][0], points[i][1], null));
			
			waypointDescs[i] = new WaypointDesc();
			waypointDescs[i].setCoords( new Point[]{pt} );
			waypointDescs[i].setLinkType(LinkType.AUTO_LINKING);
		}
		
		// Specify the HBEFA version
		// Enable the parameter for HBEFA 2.1 (use version 2.1; version 3.1 needs additional data and licence)
		// Enable the path for HBEFA 2.1
		// Check license key: xroute.emissions=true
		// Restart xRoute server
		HBEFAType hbefaType = new HBEFAType();
		hbefaType.setVersion(HBEFAVersion.HBEFA_3_1);
		
		// Specify the ResultListOptions. They provide the information, e.g. how detailed the itinerary  
		// should be. 
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setHbefaType(hbefaType);
		EmissionType emissions = new EmissionType();
		emissions.setEmissionLevel(EmissionLevel.STATIONS);
		resultListOptions.setEmissions(emissions);
				 
		//Defining the Parameters of the vehicle used for the emission calculation (if not defined profile parameters are used)
		//For detailed information refer to: 
		//http://localhost:50030/xroute/com/ptvag/xserver/xroute/doc.html#com.ptvag.xserver.xroute.VehicleParameter
		CountryInfoVehicleOptions countryInfoVehcileOptions = new CountryInfoVehicleOptions();  
		VehicleOption vehType = new VehicleOption();	
		vehType.setParameter(VehicleParameter.EMISSION_TECHNOLOGY);
		vehType.setValue("EGR");
		
		VehicleOption vehType0 = new VehicleOption();	
		vehType0.setParameter(VehicleParameter.EMISSION_CLASS);
		vehType0.setValue("EURO_5");
		
		// Note: this is the official permitted weight of the vehicle 
		// (see vehicle registration certificate)
		// All three parameters are necessary in order to calculate correct emissions.
		VehicleOption vehType1 = new VehicleOption();	
		vehType1.setParameter(VehicleParameter.TOTAL_WEIGHT);
		vehType1.setValue("40000");
		
		// Note: empty weight of chosen vehicle type 
		VehicleOption vehType2 = new VehicleOption();	
		vehType2.setParameter(VehicleParameter.EMPTY_WEIGHT);
		vehType2.setValue("12000");
		
		// Note: load weight of chosen vehicle type
		VehicleOption vehType3 = new VehicleOption();	
		vehType3.setParameter(VehicleParameter.LOAD_WEIGHT);
		vehType3.setValue("5000");
		
		VehicleOption vehType4 = new VehicleOption();	
		vehType4.setParameter(VehicleParameter.TYPE);
		vehType4.setValue("TRU");
		
		VehicleOption vehType5 = new VehicleOption();	
		vehType5.setParameter(VehicleParameter.FUEL_TYPE);
		vehType5.setValue("DIESEL");
		
		VehicleOption[] options = { vehType, vehType0, vehType1, vehType2, vehType3, vehType4, vehType5 };
		countryInfoVehcileOptions.setOptions(options);
		
		try {
			
			ExtendedRoute route = xRouteClient.calculateExtendedRoute(
						waypointDescs, null, null, resultListOptions, countryInfoVehcileOptions);
								
			Object rows[][] = new Object[route.getRoute().getStations().length][4];
			
			int rowIdx =0;
			double x = 0;
			double y = 0;
			double carbondioxide = 0;
			double carbonmonoxide = 0;
			double nitrogenoxide = 0;
			double distance = 0;
			
			//get the emission information per station
			for (WayPoint station : route.getRoute().getStations()){
			   
			    distance = station.getAccDist();			    
				x = station.getLocationCoord().getPoint().getX();
				y = station.getLocationCoord().getPoint().getY();							
				carbondioxide = station.getEmissions().getCarbonDioxide();
				carbonmonoxide = station.getEmissions().getCarbonMonoxide();
				nitrogenoxide = station.getEmissions().getNitrogenOxides();
				
			// create a new row in the table with the desired information				
			rows[rowIdx++] = new Object[]{
					String.format("%.2f", (double)x),
					String.format("%.2f", (double)y),
					String.format("%.2f km", (double)distance/1000.0),
					String.format("%.2f kg", (double)carbondioxide),
					String.format("%.2f g", (double)carbonmonoxide),
					String.format("%.2f g", (double)nitrogenoxide),
					
			};
			}
			
			// display table in window
			JFrame frame = new JFrame("Sample Emsission Calculation");
			frame.setSize(800,150);
			JTable table = new JTable(rows, new Object[]{"x", "y", "Distance", "CO2", "CO", "NO"});
			JScrollPane scrollPane1 = new JScrollPane(table);
			frame.getContentPane().add(scrollPane1);
			frame.setVisible(true);
			// exit when window is closed
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			
		}
		catch(Exception e){
			// TODO proper exception handling
			e.printStackTrace();
		}
	}
}
