/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;


public class RoutingWithFuzzy extends JPanel {
	private static final long serialVersionUID = 1L;
	
	JLabel label = new JLabel("Loading ...");

	private void runSample() {
		final String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		// Defining waypoints
		// Note: german map is required
		
		// start
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(980172, 6286080, null)); 

		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords( new Point[]{startPoint} );
		wpdStart.setLinkType(LinkType.AUTO_LINKING);

		// destination
		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(989947, 6285085, null)); 
		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords( new Point[]{destPoint} );
		wpdDest.setLinkType(LinkType.AUTO_LINKING);
			
		// soft via-point
		Point viaPoint = new Point();
		viaPoint.setPoint(new PlainPoint(990923, 6291255, null));
		
		WaypointDesc wpdVia = new WaypointDesc();
		wpdVia.setCoords( new Point[]{viaPoint} );
		
		// ###################################################
		wpdVia.setLinkType(LinkType.FUZZY_LINKING); 
		wpdVia.setFuzzyRadius(5000);
		// ###################################################
		WaypointDesc[] waypointDescs = new WaypointDesc[] {wpdStart, wpdVia, wpdDest};
		//-------------------------------------------------
		RoutingOption routingOption1 = new RoutingOption();
		routingOption1.setParameter(RoutingParameter.EXCLUDE_COUNTRIES);
		routingOption1.setValue("33");
		
		RoutingOption[] routingOptions = { routingOption1 };
		
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);  
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		try {
			Route route = xRouteClient.calculateRoute(
					waypointDescs, 
					routingOptions, /* contains ROUTING_COUNTRIES or EXCLUDE_COUNTRIES*/
					null,
					resultListOptions);

			// show a map of the route
			mapRoute(route,viaPoint);
			
		}catch(Exception e){
			// TODO proper exception handling
			e.printStackTrace();
		}
	}

	private void mapRoute(Route route, Point viaPoint) {
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// use bounding box of route
		BoundingBox boundingBox = new BoundingBox();  
		boundingBox.setLeftTop( route.getTotalRectangle().getLeftBottom() );  
		boundingBox.setRightBottom( route.getTotalRectangle().getRightTop() );  

		// get "polygon" from route
		LineString[] lineStrings = new LineString[]{ route.getPolygon()};

		// set line options ( color, stroke size )			 
		LineOptions options =  new LineOptions();  
		LinePartOptions linePartOptions = new LinePartOptions();
		linePartOptions.setVisible(true);
		linePartOptions.setColor(new Color(255 ,0 ,0));
		linePartOptions.setWidth(-6);
		options.setMainLine( linePartOptions );
		options.setShowFlags(true);
		options.setTransparent(true);

		Lines[] linesArray = new Lines[]{ new Lines(lineStrings,options) };

		Bitmap bitmap = new Bitmap();
		bitmap.setName("location_rough.bmp");
		bitmap.setDescr("SoftVia");
		bitmap.setPosition(viaPoint);
		
		Bitmap[] bitmapArray = new Bitmap[]{ bitmap };
		
		BitmapOptions bitmapOptions = new BitmapOptions();
		bitmapOptions.setTransparencyColor(new Color(255,255,255) );
		Bitmaps[] bitmaps = new Bitmaps[]{ new Bitmaps(bitmapArray, bitmapOptions ) };
		
		// put lines into a CustomLayer 
		CustomLayer customLayer = new CustomLayer();
		customLayer.setName("routeLayer");
		customLayer.setLines( linesArray );
		customLayer.setDrawPriority(500);    
		customLayer.setCenterObjects(true);  
		customLayer.setObjectInfos( ObjectInfoType.NONE);
		customLayer.setVisible(true);
		customLayer.setBitmaps( bitmaps );

		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setHeight(this.getHeight());
		imageInfo.setWidth(this.getWidth());
		
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(false);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT");

		try {
			Map map = xMapClient.renderMapBoundingBox(
					boundingBox, mapParams, imageInfo, new Layer[]{customLayer}, false);

			String imageUrl = "http://"+map.getImage().getUrl();
			label.setIcon( new ImageIcon(new URL(imageUrl)) );
			label.setText("");
			
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		// set a HTTP proxy if required
		/*System.getProperties().setProperty("http.proxySet", "true");
		System.getProperties().setProperty("http.proxyHost","localhost");
		System.getProperties().setProperty("http.proxyPort", "8888");*/

		JPanel example = new RoutingWithFuzzy();
		// display image in window
		JFrame frame = new JFrame("Sample Map");
		frame.setSize(800,600);
		frame.getContentPane().add(example);
		frame.setVisible(true);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public RoutingWithFuzzy() {
		this.add(label);
		this.setSize(640, 480);
		Thread thread = new Thread(){
			public void run() { runSample();}
		};
		thread.start();
	}
}
