/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate isochrones 
 * 
 * (c) 2012 PTV AG
 */
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainLinearRing;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.PlainPolygon;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.Polygon;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Geometries;
import com.ptvag.xserver.xmap.Geometry;
import com.ptvag.xserver.xmap.GeometryLayer;
import com.ptvag.xserver.xmap.GeometryOption;
import com.ptvag.xserver.xmap.GeometryOptions;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.ExpansionDescription;
import com.ptvag.xserver.xroute.ExpansionType;
import com.ptvag.xserver.xroute.Isochrone;
import com.ptvag.xserver.xroute.IsochroneDetail;
import com.ptvag.xserver.xroute.IsochroneOptions;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithIsochrones extends JPanel {
	final static double x = 611798.0;
	final static double y = 4962511.0;

	public static void main(String[] args) {
		// call xRoute
		Isochrone isochrone = null;
		isochrone = runRoutingSample(isochrone);

		// call xMap
		ImageIcon icon = null;
		icon = runMappingSample(isochrone, icon);

		// draw route onto map
		JFrame frame = new JFrame("Calculate isochrones");
		frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 50);
		frame.getContentPane().add(new JLabel(icon));
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Isochrone runRoutingSample(Isochrone isochrone) {
		// Step 1: Instantiate xRoute client
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Create callerContextProperty and set profile truckfast.
		CallerContextProperty profile1 = new CallerContextProperty();
		profile1.setKey("profile");
		profile1.setValue("truckfast");
		CallerContextProperty profile2 = new CallerContextProperty();
		profile2.setKey("CoordFormat");
		profile2.setValue("PTV_GEODECIMAL");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile1, profile2 };
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(callerContextProperties);
		xRouteClient.setCallerContext(callerContext);

		// Step 3: Define center location and add to class WaypointDesc.
		Point centerPoint = new Point();
		centerPoint.setPoint(new PlainPoint(x, y, null));

		WaypointDesc wpdCenter = new WaypointDesc();
		wpdCenter.setCoords(new Point[] { centerPoint });
		wpdCenter.setLinkType(LinkType.AUTO_LINKING);

		// Step 4.1: Create expansionDesc and set distance for expansion and horizon
		ExpansionDescription expansionDesc = new ExpansionDescription();
		expansionDesc.setExpansionType(ExpansionType.EXP_DIST);
		expansionDesc.setHorizons(new int[] { 20000 });

		// Step 4.2: Define isochroneOptions and add expansion's description plus
		// detail
		IsochroneOptions isochroneOptions = new IsochroneOptions();
		isochroneOptions.setExpansionDesc(expansionDesc);
		isochroneOptions.setIsoDetail(IsochroneDetail.POLY_AND_SEGMENTS);

		try {
			// Step 5: Call method calculateIsochrones and return object isochrone
			isochrone = xRouteClient.calculateIsochrones(wpdCenter, null, isochroneOptions);
		} catch (SystemException se) {
			System.err.println("Network or URL problem (xroute): " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route" + xre);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xRoute server" + xse);
		}
		return isochrone;
	}

	private static ImageIcon runMappingSample(Isochrone isochrone, ImageIcon icon) {
		// Step 1: Instantiate xMap client
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Create callerContextProperty and set profile default.
		CallerContextProperty profile1 = new CallerContextProperty();
		profile1.setKey("profile");
		profile1.setValue("default");
		CallerContextProperty profile2 = new CallerContextProperty();
		profile2.setKey("CoordFormat");
		profile2.setValue("PTV_GEODECIMAL");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile1, profile2 };
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(callerContextProperties);
		xMapClient.setCallerContext(callerContext);

		// Step 3: Prepare polygon to draw the area which is assigned by the
		// isochrone.
		PlainPoint[] points = isochrone.getIsochrones()[0].getPolys().getLineString().getPoints();

		PlainLinearRing ring = new PlainLinearRing();
		ring.setPoints(points);

		PlainPolygon plainPoly = new PlainPolygon();
		plainPoly.setLinearRings(new PlainLinearRing[] { ring });

		Polygon polygon = new Polygon();
		polygon.setPolygon(plainPoly);

		// Step 4: Create referencePoint
		Point referencePoint = new Point();
		referencePoint.setPoint(new PlainPoint(x, y, null));

		// Step 5.1: Create several objects as described below to realize geometry
		// layer.
		Geometry geometry = new Geometry();
		geometry.setDescription("Isochrones");
		// Use this method to return LineString. But take into account that only the
		// border of the isochrones will be drawn.
		// geometry.setGeometry(isochrone.getIsochrones()[0].getPolys());
		geometry.setGeometry(polygon);
		geometry.setId(1);
		geometry.setReferencePoint(referencePoint);
		Geometry[] geometries = new Geometry[] { geometry };

		GeometryOption geometryCenterObjects = new GeometryOption();
		geometryCenterObjects.setOption(GeometryOptions.AUTOCENTEROBJECTS);
		geometryCenterObjects.setValue("1");

		String colorIsochrone = XMapUtils.rgbToColorString(50, 50, 255);
		GeometryOption geometryFillColor = new GeometryOption();
		geometryFillColor.setOption(GeometryOptions.FILLCOLOR);
		geometryFillColor.setValue(colorIsochrone);

		String colorLine = XMapUtils.rgbToColorString(255, 0, 0);
		GeometryOption geometryLineColor = new GeometryOption();
		geometryLineColor.setOption(GeometryOptions.LINECOLOR);
		geometryLineColor.setValue(colorLine);

		GeometryOption geometryLineWidth = new GeometryOption();
		geometryLineColor.setOption(GeometryOptions.LINEWIDTH);
		geometryLineColor.setValue("6");

		GeometryOption[] options = new GeometryOption[] { geometryCenterObjects, geometryFillColor, geometryLineColor,
				geometryLineWidth };

		Geometries isochrones = new Geometries(geometries, options);

		GeometryLayer geometryLayer = new GeometryLayer();
		geometryLayer.setName("Isochrones");
		geometryLayer.setDrawPriority(100);
		geometryLayer.setVisible(true);
		geometryLayer.setObjectInfos(ObjectInfoType.NONE);
		geometryLayer.setGeometries(new Geometries[] { isochrones });

		// Step 5.2: Create new object layers and put customLayer into it.
		Layer[] layers = new Layer[] { geometryLayer };

		// Step 6: Define mapSection and set center.
		MapSection mapSection = new MapSection();
		mapSection.setCenter(referencePoint);

		// Step 7: Define imageInfo and define height and width of the image.
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(600);
		imageInfo.setWidth(800);
		imageInfo.setImageParameter("");
		imageInfo.setFormat(ImageFileFormat.PNG);

		// Step 8: Use mapParams for scale and distance unit
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		try {
			// Step 9: call renderMap and return object map.
			Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, false);

			// Step 10: Define icon for mapping
			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				icon = new ImageIcon(new URL(imageUrl));
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap): " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server" + xse);
		}

		return icon;
	}
}
