/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - routing with output itinerary
 * 
 * (c) 2012 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RouteListSegment;
import com.ptvag.xserver.xroute.RouteManoeuvre;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithItinerary {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// Step 1: Instantiate xRoute client. Use the packages utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2.1: Define way-points start and destination.
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(681593.765751686, 6371793.192390807, null));
		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords(new Point[] { startPoint });
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);

		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(678909.5202224851, 6395906.712571196, null));
		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords(new Point[] { destPoint });
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);

		// Step 2.2: Set start and destination into way-point description
		WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

		// Step 3: Create resultListOptions. Set properties manouevres, texts,
		// segments and detail level.
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setManoeuvres(true);
		resultListOptions.setTexts(true);
		resultListOptions.setSegments(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		// Step 4: Set the routing option for language
		RoutingOption[] routingOption = new RoutingOption[] { new RoutingOption(RoutingParameter.ROUTE_LANGUAGE, "EN") };

		try {
			// Step 5: Return object route
			Route route = xRouteClient.calculateRoute(waypointDescs, routingOption, null, resultListOptions);
			final RouteManoeuvre[] manoeuvers = route.getManoeuvres();
			final RouteListSegment[] _routeSegments = route.getSegments();

			// Step 6.1: Create a two-dimensional array rows
			Object rows[][] = new Object[manoeuvers.length][4];

			for (int i = 0; i < manoeuvers.length; i++) {
				// Step 6.2: Get street name, manoeuvre description, distance and time
				int segmentId = manoeuvers[i].getRouteListSegmentIdx();
				final RouteListSegment segment = _routeSegments[segmentId];
				int streetId = segment.getStreetNameIdx();

				rows[i] = new Object[] { String.format("%.2f", (float) segment.getAccDist() / 1000.0), // distance
																																															 // in
																																															 // km
						String.format("%.2f", (float) segment.getAccTime() / 60.0), // time
																																				// in
																																				// minutes
						manoeuvers[i].getManoeuvreDesc(), // description
						(streetId >= 0 ? route.getTexts()[streetId] : "") // street name
				};
			}

			// Step 7.1: Display all in JTable
			JTable table = new JTable(rows,
					new Object[] { "dist in km", "time in min", "Manoeuvre description", "StreetName" });
			JScrollPane scrollPane = new JScrollPane(table);

			// Step 7.2: Use JFrame to enable visibility, size, ...
			JFrame frame = new JFrame("Sample Route");
			frame.setSize(800, 150);
			frame.getContentPane().add(scrollPane);
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
	}

}
