/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import utils.XRouteUtils;

import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.ExpansionDescription;
import com.ptvag.xserver.xroute.ExpansionType;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.Reach;
import com.ptvag.xserver.xroute.ReachInfo;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithReachableObjects {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// set a HTTP proxy if required
		/*System.getProperties().setProperty("http.proxySet", "true");
		System.getProperties().setProperty("http.proxyHost","localhost");
		System.getProperties().setProperty("http.proxyPort", "8888");*/

		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);
		
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption[] options = new RoutingOption[] { hasViolations };
		
		//-------------------------------------------------
		// 1.) Defining waypoints
		//-------------------------------------------------
		
		// a) the location from which we want to "reach" other waypoints 
		Point homePoint = new Point();
		homePoint.setPoint(new PlainPoint(681593.765751686, 6371793.192390807, null));
		
		WaypointDesc home = new WaypointDesc();
		home.setCoords( new Point[]{homePoint} );
		home.setLinkType(LinkType.NEXT_SEGMENT);
		
		// b) waypoints that we want to test for "reachability"
		Point point1 = new Point();
		point1.setPoint( new PlainPoint(678909.0, 6395906.0, null) );
		
		WaypointDesc wpd1 = new WaypointDesc();
		wpd1.setCoords( new Point[]{point1} );
		wpd1.setLinkType(LinkType.NEXT_SEGMENT);
		
		//- - - - - - - - - - - - - - - -
		Point point2 = new Point();
		point2.setPoint( new PlainPoint(678009.0, 6398906.0, null) );
		
		WaypointDesc wpd2 = new WaypointDesc();
		wpd2.setCoords( new Point[]{point2} );
		wpd2.setLinkType(LinkType.NEXT_SEGMENT);
		
		WaypointDesc[] waypointDescs = new WaypointDesc[] {wpd1, wpd2};
		//-------------------------------------------------
		
		//-------------------------------------------------
		// 2.) Defining the time/distance constraint
		//-------------------------------------------------
		ExpansionDescription expansionDescription =	new ExpansionDescription();
		expansionDescription.setExpansionType(ExpansionType.EXP_DIST);
		expansionDescription.setHorizons(new int[] {22000});
				
		// => reachable are waypoints which can be reached from 'home' 
		//    with a route shorter than 20km
		//-------------------------------------------------
		
		try {
			 Reach reach = xRouteClient.calculateReachableObjects(
					home, null, waypointDescs, options, expansionDescription);
			 
			 // for each location output if it is reachable
			 for (ReachInfo ri : reach.getReachInfo()){
				 System.out.println("isReachable: "+ ri.isReachable());
				 if (ri.isReachable()) {
					 // if reachable, we can get extra info ...
					 System.out.println(String.format("\t distance: %.2f km   time: %.2f minutes",
					 (double)ri.getRouteInfo().getDistance()/1000.0,
					 (double)ri.getRouteInfo().getTime()/60.0));
				 }
			 }
		}catch(Exception e){
			// TODO proper exception handling
			e.printStackTrace();
		}
		
		
	}

}
