/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.Interval;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RouteListSegment;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.TimeEvent;
import com.ptvag.xserver.xroute.Tour;
import com.ptvag.xserver.xroute.TourPointDesc;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;


@SuppressWarnings("serial")
public class RoutingWithTimeCalc1  extends JPanel{

	public RoutingWithTimeCalc1() {
		this.setSize(780,250);
		runSample();
	}
	
	public static void main(String[] args) {
		// set a HTTP proxy if required
//		System.getProperties().setProperty("http.proxySet", "true");
//		System.getProperties().setProperty("http.proxyHost","localhost");
//		System.getProperties().setProperty("http.proxyPort", "8888");
		
		JPanel sample = new RoutingWithTimeCalc1();
		
		// display table in window
		JFrame frame = new JFrame("Sample Route");
		frame.setSize(800,270);
		
		frame.getContentPane().add(sample);
		frame.setVisible(true);
		
		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	
	private void runSample() {
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);
		
		Point point;

		//---------------------------------------------------------------------
		// 1. Tour point: Hamburg
		
		TourPointDesc hamburg = new TourPointDesc();
		hamburg.setLinkType(LinkType.AUTO_LINKING);
		point = new Point();
		point.setPoint(new PlainPoint(1108401, 7078031, null));
		hamburg.setCoords(new Point[]{point });
		
		//---------------------------------------------------------------------
		// 2. Tour point: Leipzig
		
		TourPointDesc leipzip = new TourPointDesc();
		leipzip.setLinkType(LinkType.AUTO_LINKING);
		point = new Point();
		point.setPoint(new PlainPoint(1378012, 6681209, null));
		leipzip.setCoords(new Point[]{point });
		
		// it takes 1 hour to unload the truck in Leipzig
		leipzip.setServicePeriod(3600);
		
		// the driver does not have to help and can use the service period as break
		leipzip.setUseServicePeriodForRecreation(true);
		
		// the driver has to arrive within the opening hours
		leipzip.setOpeningIntervals(new Interval[]{
				// from 2008-11-16 8:00h 
				new Interval(
						new GregorianCalendar(2008, 10, 16,  8, 0, 0),
						new GregorianCalendar(2008, 10, 16, 18, 0, 0)
				)
		});
		
		 // the unloading does not need to be finished within the opening hours
		leipzip.setCompleteServiceInIntervals(false);
		//---------------------------------------------------------------------
		
		//---------------------------------------------------------------------
		// 3. Tour point: Frankfurt
	
		TourPointDesc frankfurt = new TourPointDesc();
		frankfurt.setLinkType(LinkType.AUTO_LINKING);
		point = new Point();
		point.setPoint(new PlainPoint(962659, 6461857, null));
		frankfurt.setCoords(new Point[]{point });
		frankfurt.setServicePeriod(1800); // 30 minutes
		
		frankfurt.setOpeningIntervals(new Interval[]{
				new Interval(
						new GregorianCalendar(2008, 10, 16,  8, 0, 0), // from 2008-11-16  8:00h 
						new GregorianCalendar(2008, 10, 16, 13, 0, 0)  // till 2008-11-16 13:00h 
				),
				new Interval(
						new GregorianCalendar(2008, 10, 16, 15, 0, 0), // from 2008-11-16 15:00h 
						new GregorianCalendar(2008, 10, 16, 22, 0, 0)  // till 2008-11-16 22:00h 
				)
		});
		//---------------------------------------------------------------------
		
		
		//---------------------------------------------------------------------
		// 4. Tour point: Munich
	
		TourPointDesc munich = new TourPointDesc();
		munich.setLinkType(LinkType.AUTO_LINKING);
		point = new Point();
		point.setPoint(new PlainPoint(1292573, 6125849, null));
		munich.setCoords(new Point[]{point });
		munich.setCoords(new Point[]{point });
		munich.setServicePeriod(1800); // 30 minutes
		munich.setOpeningIntervals(new Interval[]{
				new Interval(
						new GregorianCalendar(2008, 10, 17,  8, 0, 0), // from 2008-11-17  8:00h 
						new GregorianCalendar(2008, 10, 17, 13, 0, 0)  // till 2008-11-17 13:00h 
				),
				new Interval(
						new GregorianCalendar(2008, 10, 17, 15, 0, 0), // from 2008-11-17 15:00h 
						new GregorianCalendar(2008, 10, 17, 20, 0, 0)  // till 2008-11-17 20:00h 
				)
		});
		//---------------------------------------------------------------------
		// tourPoints is an array of stations defined by WaypointDesc or TourPointDesc objects
		// first and last station MUST be of type TourPointDesc
		WaypointDesc[] tourPoints = new WaypointDesc[]{ hamburg, leipzip, frankfurt, munich};
		
		ResultListOptions resultListOptions = new ResultListOptions();
		// set manoveurs to true, if itinerary with manoveurs is wanted
//		resultListOptions.setManoeuvres(true);
		resultListOptions.setTexts(true);
		resultListOptions.setSegments(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);


		// ------ Setting START_TIME --------
		// the point of time set in START_TIME will be used as reference for
		// relative time specifications.
		// If you do not set the START_TIME parameter, it will be set to 'now'
		final Calendar earliestStartTime = new GregorianCalendar(2008, 10, 16,  8, 0, 0);
		final SimpleDateFormat rfc3339_format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
		// disable RoadEditor
		RoutingOption routingOptionRE = new RoutingOption();
		routingOptionRE.setParameter(RoutingParameter.ENABLE_ROADEDITOR);
		routingOptionRE.setValue("false");
		
		RoutingOption[] params = new RoutingOption[]{
				new RoutingOption(RoutingParameter.START_TIME,rfc3339_format.format(earliestStartTime.getTime())),
				routingOptionRE
		};
		
		try {
			Tour tour 
				= xRouteClient.calculateTour(tourPoints, params, null, resultListOptions, null);
			
			Route route = tour.getRoute();
			
			final RouteListSegment[] routeSegments = route.getSegments();
		
			// the Texts array contains strings which are referenced
			// by other data of the route (e.g. street names)
			final String[] texts = route.getTexts();
		

			// the TimeEvents array contains events, which (should) occur during the
			// tour. Events include driving periods, rest periods, breaks, etc.
			// The observable event types are documented in the API reference. 
			final TimeEvent[] events =  tour.getTimeEvents();
			
			Object rows[][] = new Object[events.length][];
			Object cols[] = new Object[]{"Event", "at tour point", "street", "start time", "end time",};
			
			final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss Z");
			//final Calendar tourStart = Calendar.getInstance(); 
				//tour.getTourSummary().getLastestTourStart();
			
			for (int i=0; i<events.length; i++ )  {
				
				String streetName = "";
				// in order to get the street name we 
				// check if the events references a valid segment ID
				if (events[i].getSegmentIdx() < routeSegments.length) {
					// get the reference segment
					final RouteListSegment segment = routeSegments[events[i].getSegmentIdx()];
					// get the street name ID from the segment
					final int streetId = segment.getStreetNameIdx();
					
					// if street idx is valid, get the street name
					// from the array of texts
					if (streetId >= 0)
						streetName = texts[streetId];
				}
				
				Calendar c = (Calendar)earliestStartTime.clone();
				// times are relative to START_TIME
				c.add(Calendar.SECOND,events[i].getStartTime() );
				String startTime = df.format(c.getTime());
				
				c.add(Calendar.SECOND,events[i].getPeriod() );
				String endTime = df.format(c.getTime());
				
				
				rows[i] = new Object[]{
						events[i].getType(), // type of the event (eg.: TOUR_START, DRIVING ...)
						( events[i].getTourPointIdx() < 0 ? "" : events[i].getTourPointIdx()),
						streetName, // name of street if available
						startTime, // start time of the event
						endTime    // end time of the event
				};
			}
			// create a JTable, add the data and put it into a JScrollPane
			JTable table = new JTable(rows,cols);
			JScrollPane scrollPane = new JScrollPane(table);
			scrollPane.setPreferredSize( this.getSize());
			this.add(scrollPane);
		
		}catch(Exception e){
			// TODO proper exception handling
			e.printStackTrace();
		}
	}

}
