/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2008 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.Interval;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.RouteManoeuvre;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.TimeEvent;
import com.ptvag.xserver.xroute.Tour;
import com.ptvag.xserver.xroute.TourPointDesc;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

@SuppressWarnings("serial")
public class RoutingWithTimeEvents extends JPanel {

	public RoutingWithTimeEvents() {
		this.setSize(780, 380);
		runSample();
	}

	public static void main(String[] args) {
		// set a HTTP proxy if required
		// System.getProperties().setProperty("http.proxySet", "true");
		// System.getProperties().setProperty("http.proxyHost","localhost");
		// System.getProperties().setProperty("http.proxyPort", "8888");

		JPanel sample = new RoutingWithTimeEvents();

		// display table in window
		JFrame frame = new JFrame("Sample Route");
		frame.setSize(800, 400);

		frame.getContentPane().add(sample);
		frame.setVisible(true);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void runSample() {
		final Calendar earliestStartTime = new GregorianCalendar(2008, 10, 16, 6, 0, 0);
		// calculate a tour
		// (the CalculateTour.java showed how this is done)
		final Tour tour = calculateSampleTour(earliestStartTime);

		if (tour == null) {
			System.out.println("Tour is not calculated");
			return;
		}

		// We use a HashMap to map a list of TimeEvents to their route segment ID
		// later, we use this to get the associated TimeEvents when we have a
		// segment ID
		HashMap<Integer, List<TimeEvent>> eventsPerSegmentIdx = new HashMap<Integer, List<TimeEvent>>();

		// go thru all TimeEvents and add them to the HashMap ...
		for (TimeEvent event : tour.getTimeEvents()) {
			final Integer key = Integer.valueOf(event.getSegmentIdx());
			List<TimeEvent> eventList;

			if (eventsPerSegmentIdx.containsKey(key)) {
				eventList = eventsPerSegmentIdx.get(key);
			} else {
				eventList = new LinkedList<TimeEvent>();
				eventsPerSegmentIdx.put(key, eventList);
			}
			eventList.add(event);
		}

		// we want to display the data in a JTable,
		// so we store the data in a Vector of rows
		Vector<Vector<String>> tableRows = new Vector<Vector<String>>();

		// loop thru all manoeuvres ...
		for (RouteManoeuvre manoeuvre : tour.getRoute().getManoeuvres()) {
			Vector<String> row = new Vector<String>();
			final Integer key = Integer.valueOf(manoeuvre.getRouteListSegmentIdx());

			// skip manoeuvres that have no associated TimeEvent
			// AND have a different DetailLevel than VERY_IMPORTANT
			if (!eventsPerSegmentIdx.containsKey(key) && manoeuvre.getDetailLevel() != DetailLevel.VERY_IMPORTANT)
				continue;

			row.add(manoeuvre.getManoeuvreDesc()); // 1st column: manoeuvre desc
			row.add(manoeuvre.getManoeuvreType().toString());// 2nd column: manoeuvre
																											 // type

			StringBuilder sb = new StringBuilder();
			// if the segment of the current manoeuvre has an
			// associated list of time events we create a comma
			// seperated list
			if (eventsPerSegmentIdx.containsKey(key)) {
				String pre = "";
				for (TimeEvent e : eventsPerSegmentIdx.get(key)) {
					sb.append(pre + e.getType().toString());
					pre = ", ";
				}
			}
			row.add(sb.toString()); // 3rd column: tour's TimeEvents
			tableRows.add(row);
		}

		Vector<String> columnNames = new Vector<String>();
		columnNames.add("Manoeuvre");
		columnNames.add("");
		columnNames.add("associated Event(s)");

		JTable table = new JTable(tableRows, columnNames);
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(this.getSize());
		this.add(scrollPane);
	}

	/**
	 * Calculates a sample tour as described in CalculateTour.java
	 * 
	 * @param startTime
	 *          the start time
	 * 
	 * @return the tour
	 */
	private Tour calculateSampleTour(Calendar startTime) {
		final String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// ---------------------------------------------------------------------
		// 1. Tour point: Hamburg

		TourPointDesc hamburg = new TourPointDesc();
		hamburg.setLinkType(LinkType.AUTO_LINKING);
		Point point1 = new Point();
		point1.setPoint(new PlainPoint(1108579, 7076729, null));
		hamburg.setCoords(new Point[] { point1 });

		// ---------------------------------------------------------------------
		// 2. Tour point: Leipzig

		TourPointDesc leipzig = new TourPointDesc();
		leipzig.setLinkType(LinkType.AUTO_LINKING);
		Point point2 = new Point();
		point2.setPoint(new PlainPoint(1377181, 6680649, null));
		leipzig.setCoords(new Point[] { point2 });

		// it takes 1 hour to unload the truck in Leipzig
		leipzig.setServicePeriod(3600);

		// the driver does not have to help and can use the service period as break
		leipzig.setUseServicePeriodForRecreation(true);

		// the driver has to arrive within the opening hours
		leipzig.setOpeningIntervals(new Interval[] {
		// from 2008-11-16 8:00h
		new Interval(new GregorianCalendar(2008, 10, 16, 8, 0, 0), new GregorianCalendar(2008, 10, 16, 18, 0, 0)) });

		// the unloading does not need to be finished within the opening hours
		leipzig.setCompleteServiceInIntervals(false);
		// ---------------------------------------------------------------------

		// ---------------------------------------------------------------------
		// 3. Tour point: Frankfurt

		TourPointDesc frankfurt = new TourPointDesc();
		frankfurt.setLinkType(LinkType.AUTO_LINKING);
		Point point3 = new Point();
		point3.setPoint(new PlainPoint(962659, 6461857, null));
		frankfurt.setCoords(new Point[] { point3 });
		frankfurt.setServicePeriod(1800); // 30 minutes

		frankfurt.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2008, 10, 16, 8, 0, 0), // from
																																																							// 2008-11-16
																																																							// 8:00h
				new GregorianCalendar(2008, 10, 16, 13, 0, 0) // till 2008-11-16 13:00h
				), new Interval(new GregorianCalendar(2008, 10, 16, 15, 0, 0), // from
																																			 // 2008-11-16
																																			 // 15:00h
						new GregorianCalendar(2008, 10, 16, 22, 0, 0) // till 2008-11-16
																													// 22:00h
				) });
		// ---------------------------------------------------------------------

		// ---------------------------------------------------------------------
		// 4. Tour point: Munich

		TourPointDesc munich = new TourPointDesc();
		munich.setLinkType(LinkType.AUTO_LINKING);
		Point point4 = new Point();
		point4.setPoint(new PlainPoint(1292573, 6125849, null));
		munich.setCoords(new Point[] { point4 });
		munich.setServicePeriod(1800); // 30 minutes
		munich.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2008, 10, 17, 8, 0, 0), // from
																																																					 // 2008-11-17
																																																					 // 8:00h
				new GregorianCalendar(2008, 10, 17, 13, 0, 0) // till 2008-11-17 13:00h
				), new Interval(new GregorianCalendar(2008, 10, 17, 15, 0, 0), // from
																																			 // 2008-11-17
																																			 // 15:00h
						new GregorianCalendar(2008, 10, 17, 20, 0, 0) // till 2008-11-17
																													// 20:00h
				) });
		// ---------------------------------------------------------------------
		// tourPoints is an array of stations defined by WaypointDesc or
		// TourPointDesc objects
		// first and last station MUST be of type TourPointDesc
		WaypointDesc[] tourPoints = new WaypointDesc[] { hamburg, munich, leipzig, frankfurt };

		ResultListOptions resultListOptions = new ResultListOptions();
		// set manoveurs to true, if itinerary with manoveurs is wanted
		resultListOptions.setManoeuvres(true);
		resultListOptions.setTexts(true);
		resultListOptions.setSegments(true);
		resultListOptions.setDetailLevel(DetailLevel.ALL);

		// ------ Setting START_TIME --------
		// the point of time set in START_TIME will be used as reference for
		// relative time specifications.
		// If you do not set the START_TIME parameter, it will be set to 'now'
		final SimpleDateFormat rfc3339_format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");

		// disable RoadEditor
		RoutingOption routingOptionRE = new RoutingOption();
		routingOptionRE.setParameter(RoutingParameter.ENABLE_ROADEDITOR);
		routingOptionRE.setValue("false");
		System.out.println("Start_Time: " + rfc3339_format.format(startTime.getTime()));
		RoutingOption[] params = new RoutingOption[] {
				new RoutingOption(RoutingParameter.START_TIME, rfc3339_format.format(startTime.getTime())), routingOptionRE };

		try {
			Tour tour = xRouteClient.calculateTour(tourPoints, params, null, resultListOptions, null);
			return tour;
		} catch (Exception e) {
			// TODO proper exception handling
			e.printStackTrace();
			return null;
		}
	}

}
