/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xroute.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - routing inclusive toll calculation
 * 
 * (c) 2012 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.CountryInfo;
import com.ptvag.xserver.xroute.CountryInfoVehicleOptions;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.ExtendedRoute;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.TollCostInfo;
import com.ptvag.xserver.xroute.VehicleOption;
import com.ptvag.xserver.xroute.VehicleParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithToll {

	public static void main(String[] args) {
		Object rows[][] = new Object[1][1];
		rows = runSample(rows);
		
		JTable table = new JTable(rows, new Object[] { "IU Code", "Distance", "TollDistance", "TollPrice" });
		JScrollPane scrollPane = new JScrollPane(table);
		
		JFrame frame = new JFrame("Sample Route");
		frame.setSize(800, 165);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] runSample(Object rows[][]) {
		// Step 1: Instantiate xRoute client
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2.1: Create callerContextProperty and set profile truckfast.
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");

		// Step 2.2: Set properties into callerContextProperties (type is defined as
		// an array)
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		// Step 2.3: Define callerContext, set properties and overwrite default
		// class in xRoute client
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(callerContextProperties);
		xRouteClient.setCallerContext(callerContext);

		// Step 3: Add way-points for routing
		// Note: PTV Europe City Map Premium is required and additional toll data
		// has to be licensed.
		double[][] points = new double[][] { { 262175.0, 6242590.0 }, // Paris
				{ 933441.0, 6268969.0 }, // Karlsruhe
				{ 1820467.0, 6134685.0 }, // Wien
				{ 1267389.0, 5978788.0 }, // Innsbruck
				{ 1262552.0, 5853474.0 } // Bolzano
		};

		WaypointDesc[] waypointDescs = new WaypointDesc[points.length];
		for (int i = 0; i < points.length; i++) {
			Point pt = new Point();
			pt.setPoint(new PlainPoint(points[i][0], points[i][1], null));

			waypointDescs[i] = new WaypointDesc();
			waypointDescs[i].setCoords(new Point[] { pt });
			waypointDescs[i].setLinkType(LinkType.AUTO_LINKING);
		}

		// Step 4: Specify the ResultListOptions. They provide the information, e.g.
		// how
		// detailed the itinerary should be listed.
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setManoeuvres(true);
		resultListOptions.setTexts(true);
		resultListOptions.setSegments(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		// Step 5.1: Define countryInfoVehicleOptions and vehicleOption. Set the
		// relevant attributes.
		CountryInfoVehicleOptions countryInfoVehicleOptions = new CountryInfoVehicleOptions();
		countryInfoVehicleOptions.setAllEuro(true);
		countryInfoVehicleOptions.setDetailedTollCosts(true);

		VehicleOption vehicleOption = new VehicleOption();
		vehicleOption.setParameter(VehicleParameter.TYPE);
		vehicleOption.setValue("TRU");

		// Step 5.2: Put the vehicleOption into the array VehicleOption[]
		VehicleOption[] vehicleOptions = { vehicleOption };

		// Step 5.3: Set option "vehicleOptions" into countryInfoVehicleOptions
		countryInfoVehicleOptions.setOptions(vehicleOptions);

		try {
			// Step 6: Return route including the toll information
			ExtendedRoute route = xRouteClient.calculateExtendedRoute(waypointDescs, null, null, resultListOptions,
					countryInfoVehicleOptions);

			// Step 7: Plot overview of toll prices and distances
			rows = new Object[route.getCountryInfos().length][4];
			int rowIdx = 0;
			for (CountryInfo countryInfo : route.getCountryInfos()) {
				int totalTollPrice = 0;
				int totalTollDistance = 0;

				for (TollCostInfo tollCostInfo : countryInfo.getTollCostInfos()) {
					totalTollPrice += tollCostInfo.getTollPrice();
					totalTollDistance += tollCostInfo.getTollDistance();
				}

				rows[rowIdx++] = new Object[] { countryInfo.getIuCode(),
						String.format("%.2f km", (float) countryInfo.getPartRouteInfo().getDistance() / 1000.0),
						String.format("%.2f km", (float) totalTollDistance / 1000.0),
						String.format("%.2f ", (float) totalTollPrice / 100.0) };
			}
			
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return rows;
	}
}
