/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non-exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. Therefore, PTV Group cannot
 * guarantee or imply reliability, serviceability or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
/*
 * PTV xServer Code Samples
 * 
 * PTV Route Server - Routing with Truck Attributes
 * 
 * (c) 2014 PTV AG
 */
package samples.xroute.advanced;

import java.awt.HeadlessException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.core.exception.UnexpectedException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.RoadEditorLayer;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RouteListSegment;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class RoutingWithTruckAttributes {

	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// Step 1: Initialize clients
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2.1: Add attribute "Profile" into class CallerContext
		CallerContextProperty profile = new CallerContextProperty("Profile", "truckfast");
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(new CallerContextProperty[] { profile });
		
		// Step 2.2: Overwrite default value
		xRouteClient.setCallerContext(callerContext);
		xMapClient.setCallerContext(callerContext);

		// Step 3.1: Enable RoadEditor
		RoutingOption enableRE = new RoutingOption(RoutingParameter.ENABLE_ROADEDITOR, "true");

		// Step 3.2: Define reLayer 
		// Demo version: maps.path=data/map is enabled.
		// re/truckattributes folder is already available --> see folder data/map/re
		RoutingOption reLayer = new RoutingOption(RoutingParameter.ROADEDITOR_LAYERNAME, "truckattributes");

		// Step 3.3: Add additional options
		RoutingOption reOptions = new RoutingOption(RoutingParameter.ROADEDITOR_ADDITIONAL_OPTIONS, "MAX_HEIGHT=250");
		RoutingOption[] routingOptions = new RoutingOption[] { enableRE, reLayer, reOptions };
		
		// Step 4: Specify the ResultListOptions
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);
		resultListOptions.setSegments(true);
		resultListOptions.setSegmentAttributes(true);

		// Step 5: Get array waypointsDesc[]
		final WaypointDesc[] waypointDescs = getWaypoints();

		try {
			// Step 6: Call method calculateRoute
			Route route = xRouteClient.calculateRoute(waypointDescs, routingOptions, null, resultListOptions);
			
			// Step 7: Return result list
			for (RouteListSegment routeListSegment : route.getSegments()) {
				String list = "" + routeListSegment.getSegmentAttr().getAdditionalRE();
				String id = "" + routeListSegment.getNC();
				String node = "" + routeListSegment.getFirstNodeIdx();
				
				if (!list.equals("null")) {
					System.out.println("Node ID: " + node + "   Network Class: " + id + "   Truck Restriction: " + list);
				}
				
			}
			
			// Step 8: Display truck attributes and polygon
			displayMapOfRoute(xMapClient, route);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static void displayMapOfRoute(XMapRemoteInterface xMapClient, Route route) throws RemoteException,
			SystemException, XMapException, UnexpectedException, XServiceException, MalformedURLException, HeadlessException {

		// Step 1: Define imageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(600);
		imageInfo.setWidth(800);
		imageInfo.setImageParameter("");
		imageInfo.setFormat(ImageFileFormat.GIF);

		// Step 2: Add mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(false);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT");
		
		boolean includeImageInResponse = false;

		// Step 3: Create a line from the given route
		LineString[] lineStrings = new LineString[] { route.getPolygon() };
		Lines[] lines = new Lines[1];
		LineOptions options = new LineOptions();
		LinePartOptions linePartOpt = new LinePartOptions();
		linePartOpt.setColor(new Color(0, 255, 0));
		linePartOpt.setVisible(true);
		linePartOpt.setWidth(-12);
		options.setMainLine(linePartOpt);

		lines[0] = new Lines(lineStrings, options);

		// Step 4: Define a custom layer that contains the line(s)
		CustomLayer customLayer = new CustomLayer();
		customLayer.setVisible(true);
		customLayer.setDrawPriority(100); // this is PRIORROUTE
		customLayer.setLines(lines);
		customLayer.setObjectInfos(ObjectInfoType.NONE);
		customLayer.setCenterObjects(true);

		// Step 5: Create reLayer
		RoadEditorLayer reLayer = new RoadEditorLayer();
		// Demo version: maps.path=data/map is enabled.
		// re/truckattributes folder is already available --> see folder data/map/re
		reLayer.setName("truckattributes");
		reLayer.setVisible(true);
		reLayer.setObjectInfos(ObjectInfoType.NONE);
		// Layer[] layers = new Layer[]{ reLayer};

		Layer[] layers = new Layer[] { customLayer, reLayer };

		// Step 6: Set the bounding box
		BoundingBox boundingBox = new BoundingBox();
		boundingBox.setLeftTop(route.getTotalRectangle().getLeftBottom());
		boundingBox.setRightBottom(route.getTotalRectangle().getRightTop());

		// Step 7: Call method renderMapBoundingBox
		Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse);

		// Step 8: Retrieve the image
		String imageUrl = "http://" + map.getImage().getUrl();
		ImageIcon icon = new ImageIcon(new URL(imageUrl));

		// Step 9: Display image in window
		JFrame frame = new JFrame("Sample Routing with Truck Attributes");
		frame.setSize(icon.getIconWidth() + 10, icon.getIconHeight() + 30);
		frame.getContentPane().add(new JLabel(icon));
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static WaypointDesc[] getWaypoints() {
		// points = list of pairs of (x,y) coordinates (in order of route)
		double[][] points = new double[][] { { 681949.0, 6370040.0 }, { 681144.0, 6365010.0 } };

		WaypointDesc[] waypointDescs = new WaypointDesc[points.length];
		for (int i = 0; i < points.length; i++) {
			Point pt = new Point();
			pt.setPoint(new PlainPoint(points[i][0], points[i][1], null));

			waypointDescs[i] = new WaypointDesc();
			waypointDescs[i].setCoords(new Point[] { pt });
			waypointDescs[i].setLinkType(LinkType.NEXT_SEGMENT);
		}
		return waypointDescs;
	}

}
