/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate advanced tour
 * 
 * (c) 2014 PTV AG
 */
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.AdvancedTour;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.DriverSettings;
import com.ptvag.xserver.xroute.Interval;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.TourEvent;
import com.ptvag.xserver.xroute.TourOptions;
import com.ptvag.xserver.xroute.TourPointDesc;
import com.ptvag.xserver.xroute.TourPointResult;
import com.ptvag.xserver.xroute.TourRestrictions;
import com.ptvag.xserver.xroute.TransportPeriodRestriction;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcAdvancedTour {

	public static void main(String[] args) {
		AdvancedTour advancedTour = tourInformation();

		if (advancedTour == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultListTourPoints(advancedTour);
		Object[] cols = new Object[] { "Station Id", "Arrival", "Departure", "Service Start", "Service Stop" };
		createTableTourpoints(rows, cols);
		
		rows = resultListTourEvents(advancedTour);
		cols = new Object[] { "TourPoint", "Start Time", "Period", "Segment ID", "Descriptions", "Violations" };
		createTableTourevents(rows, cols);
		
		rows = resultListTourOverview(advancedTour);
		cols = new Object[] { "Name", "Value" };
		createTableTourOverview(rows, cols);
	}

	private static void createTableTourpoints(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("List Tour Points");
		frame.setSize(700, 170);
		frame.setLocation(0, 251);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	private static void createTableTourevents(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("List Tour Events");
		frame.setSize(700, 250);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	private static void createTableTourOverview(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("List Tour Overview");
		frame.setSize(700, 200);
		frame.setLocation(0, 421);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultListTourOverview(AdvancedTour advancedTour) {
		Object rows[][] = new Object[7][2];

		rows[0][0] = "Total Distance [m]";
		rows[0][1] = advancedTour.getRoute().getInfo().getDistance();
		rows[1][0] = "Total Time [s]";
		rows[1][1] = advancedTour.getRoute().getInfo().getTime();
		rows[2][0] = "Start Time";
		rows[2][1] = advancedTour.getStartTime().getTime();
		rows[3][0] = "Tour Period [s]";
		rows[3][1] = advancedTour.getTourPeriod();
		rows[4][0] = "Waiting Period [s]";
		rows[4][1] = advancedTour.getGenuineWaitingPeriod();
		rows[5][0] = "Deferment Potential [s]";
		rows[5][1] = advancedTour.getDefermentPotential();
		rows[6][0] = "Violated";
		rows[6][1] = advancedTour.getIsViolated();
		
		return rows;
	}
	
	private static Object[][] resultListTourPoints(AdvancedTour advancedTour) {
		int max = advancedTour.getTourPointResults().length;
		Object rows[][] = new Object[max][5];

		int index = 0;
		for (TourPointResult tourPoint : advancedTour.getTourPointResults()) {
			rows[index][0] = tourPoint.getStationIndex();
			rows[index][1] = tourPoint.getArrivalTime().getTime();
			rows[index][2] = tourPoint.getDepartureTime().getTime();
			rows[index][3] = tourPoint.getStartServiceTime().getTime();
			rows[index][4] = tourPoint.getEndServiceTime().getTime();
					
			index++;
		}
		
		return rows;
	}
	
	private static Object[][] resultListTourEvents(AdvancedTour advancedTour) {
		int max = advancedTour.getTourEvents().length;
		Object rows[][] = new Object[max][6];

		int index = 0;
		for (TourEvent tourEvent : advancedTour.getTourEvents()) {
			int tpid = tourEvent.getTourPointIndex();
			if (tpid < 0) {
				rows[index][0] = "";
				
				String violations = "";
				for (String tourViol : tourEvent.getViolations()) {
					violations = violations + tourViol + "; ";
				}
				if (violations.isEmpty()) {
					rows[index][5] = "no violations";
				} else {
					rows[index][5] = violations;	
				}
				
			} else {
				rows[index][0] = tpid;	
			}
			
			rows[index][1] = tourEvent.getStartTime().getTime();
			rows[index][2] = tourEvent.getPeriod();
			rows[index][3] = tourEvent.getSegmentIndex();
			String descriptions = "";
			for (String tourDesc : tourEvent.getDescriptions()) {
				descriptions = descriptions + tourDesc + "; ";
			}
			rows[index][4] = descriptions;
			
			index++;
		}
		
		return rows;
	}

	private static WaypointDesc[] getTourpoints() {
		
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(682077, 6374851, null));
		TourPointDesc start = new TourPointDesc();
		start.setCoords(new Point[] { startPoint });
		start.setSiteID(1);
		start.setLinkType(LinkType.AUTO_LINKING);
		//start.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 7, 0, 0),
		//		new GregorianCalendar(2014, 9, 13, 19, 0, 0)) });

		Point viaPoint1 = new Point();
		viaPoint1.setPoint(new PlainPoint(657480, 6385390, null));
		TourPointDesc via1 = new TourPointDesc();
		via1.setCoords(new Point[] { viaPoint1 });
		via1.setSiteID(101);
		via1.setServicePeriod(3900);
		via1.setUseServicePeriodForRecreation(false);
		via1.setCompleteServiceInIntervals(true);
		via1.setLinkType(LinkType.AUTO_LINKING);
		via1.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 8, 0, 0),
				new GregorianCalendar(2014, 9, 13, 18, 0, 0)) });
		
		Point viaPoint2 = new Point();
		viaPoint2.setPoint(new PlainPoint(679110, 6396180, null));
		TourPointDesc via2 = new TourPointDesc();
		via2.setCoords(new Point[] { viaPoint2 });
		via2.setSiteID(102);
		via2.setServicePeriod(3800);
		via2.setUseServicePeriodForRecreation(false);
		via2.setCompleteServiceInIntervals(true);
		via2.setLinkType(LinkType.AUTO_LINKING);
		via2.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 8, 0, 0),
				new GregorianCalendar(2014, 9, 13, 18, 0, 0)) });
		
		Point viaPoint3 = new Point();
		viaPoint3.setPoint(new PlainPoint(684480, 6405430, null));
		TourPointDesc via3 = new TourPointDesc();
		via3.setCoords(new Point[] { viaPoint3 });
		via3.setSiteID(103);
		via3.setServicePeriod(3700);
		via3.setUseServicePeriodForRecreation(false);
		via3.setCompleteServiceInIntervals(true);
		via3.setLinkType(LinkType.AUTO_LINKING);
		via3.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 8, 0, 0),
				new GregorianCalendar(2014, 9, 13, 18, 0, 0)) });
		
		Point viaPoint4 = new Point();
		viaPoint4.setPoint(new PlainPoint(695938, 6396610, null));
		TourPointDesc via4 = new TourPointDesc();
		via4.setCoords(new Point[] { viaPoint4 });
		via4.setSiteID(104);
		via4.setServicePeriod(3600);
		via4.setUseServicePeriodForRecreation(false);
		via4.setCompleteServiceInIntervals(true);
		via4.setLinkType(LinkType.AUTO_LINKING);
		via4.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 8, 0, 0),
				new GregorianCalendar(2014, 9, 13, 18, 0, 0)) });

		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(681870, 6380358, null));
		TourPointDesc dest = new TourPointDesc();
		dest.setCoords(new Point[] { destPoint });
		dest.setSiteID(105);
		dest.setServicePeriod(4200);
		dest.setUseServicePeriodForRecreation(false);
		dest.setCompleteServiceInIntervals(true);
		dest.setLinkType(LinkType.AUTO_LINKING);
		dest.setOpeningIntervals(new Interval[] { new Interval(new GregorianCalendar(2014, 9, 13, 8, 0, 0),
				new GregorianCalendar(2014, 9, 13, 18, 0, 0)) });

		WaypointDesc[] tourPoints = new WaypointDesc[] { start, via1, via2, via3, via4, dest };

		return tourPoints;
	}

	private static CallerContext getCallerContext() {
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("carfast");
		CallerContextProperty coordFormat = new CallerContextProperty();
		coordFormat.setKey("CoordFormat");
		coordFormat.setValue("PTV_MERCATOR");
		CallerContextProperty response = new CallerContextProperty();
		response.setKey("ResponseGeometry");
		response.setValue("PLAIN");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile, coordFormat, response };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		return ctx;
	}

	private static TourOptions getTourOptions() {
		TourOptions tourOptions = new TourOptions();
		DriverSettings driverSettings = new DriverSettings();
		driverSettings.setBreakRuleDisabled(false);
		driverSettings.setDailyRestRuleDisabled(false);
		driverSettings.setWeeklyRestRuleDisabled(false);
		driverSettings.setWorkingRuleDisabled(false);
		tourOptions.setDriverSettings(driverSettings);

		TourRestrictions tourRestrictions = new TourRestrictions();
		tourRestrictions.setMaxTourPeriod(32400);
		TransportPeriodRestriction tpr1 = new TransportPeriodRestriction();
		tpr1.setStartTourPointIndex(0);
		tpr1.setDestinationTourPointIndex(1);
		tpr1.setMaxPeriodBetweenServices(7200);
		TransportPeriodRestriction[] transportPeriodRestrictions = new TransportPeriodRestriction[] { tpr1 };
		tourRestrictions.setTransportPeriodRestrictions(transportPeriodRestrictions);
		tourOptions.setTourRestrictions(tourRestrictions);

		return tourOptions;
	}
	
	private static String getReferenceTime() {
		final SimpleDateFormat rfc3339_format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
		final Calendar startTime = new GregorianCalendar(2014, 9, 13, 7, 0, 0);

		return rfc3339_format.format(startTime.getTime());
	}

	private static RoutingOption[] getRoutingOptions() {
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption referenceTime = new RoutingOption();
		referenceTime.setParameter(RoutingParameter.START_TIME);
		referenceTime.setValue(getReferenceTime());
		RoutingOption[] routingOptions = new RoutingOption[] { hasViolations, referenceTime };

		return routingOptions;
	}

	private static ResultListOptions getResultListOptions() {
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setManoeuvres(false);
		resultListOptions.setTexts(false);
		resultListOptions.setSegments(false);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		return resultListOptions;
	}

	private static AdvancedTour tourInformation() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define tour points for start, via and destination
		WaypointDesc[] tourPoints = getTourpoints();

		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContext ctx = getCallerContext();
		xRouteClient.setCallerContext(ctx);

		// Step 4: Configure TourOptions to be respected when calculating a tour.
		TourOptions tourOptions = getTourOptions();

		// Step 5: Define RoutingOption to allow segment violations and set
		// reference time.
		RoutingOption[] routingOptions = getRoutingOptions();

		// Step 6: Add ResultListOptions and set manoeuvres, texts, segments and
		// detail level.
		ResultListOptions details = getResultListOptions();

		// Step 7: Call calculateRouteInfo(). ExceptionPath and CountryInfoOptions
		// are hereby not considered.
		AdvancedTour advancedTour = null;
		try {
			advancedTour = xRouteClient.calculateAdvancedTour(tourPoints, tourOptions, routingOptions, null, details, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return advancedTour;
	}
}
