/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate bulk routing information
 * 
 * (c) 2014 PTV AG
 */
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.BulkRouteInfo;
import com.ptvag.xserver.xroute.BulkRouteInfoResult;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcBulkRouteInfo {

	public static void main(String[] args) {
		BulkRouteInfo bulkRouteInfo = bulkRouteInformation();

		if (bulkRouteInfo == null) {
		System.out.println("Validate error message!");
		return;
		}
		
		Object[][] rows = resultList(bulkRouteInfo);
		Object[] cols = new Object[] { "Index", "Distance [m]", "Travel time [s]", "Internal costs", "Has violation" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Calculate Bulk Route Information");
		frame.setSize(500, 120);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(BulkRouteInfo bulkRouteInfo) {
		Object rows[][] = new Object[bulkRouteInfo.getBulkRouteInfoResult().length][5];

		int index = 0;
		for (BulkRouteInfoResult result : bulkRouteInfo.getBulkRouteInfoResult()) {
			rows[index][0] = index;
			rows[index][1] = result.getRouteInfo().getDistance();
			rows[index][2] = result.getRouteInfo().getTime();
			rows[index][3] = result.getRouteInfo().getCost();
			rows[index][4] = result.getRouteInfo().getHasViolations();
			index++;
		}
		

		return rows;
	}

	private static BulkRouteInfo bulkRouteInformation() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define locations for starts and destinations
		Vector<Double> locations1 = new Vector<Double>();
		locations1.add(681593.0);
		locations1.add(6371793.0);
		locations1.add(678909.0);
		locations1.add(6395906.0);
		
		Iterator i1 = locations1.iterator();
		int max = locations1.size()/2;
	
		WaypointDesc[] waypointDesc1 = new WaypointDesc[max];
		int index = 0;
		while (i1.hasNext()) {
			PlainPoint plain = new PlainPoint();
			double x = (Double) i1.next();
			double y = (Double) i1.next();
			plain.setX(x);
			plain.setY(y);
			plain.setZ(null); // optional
			Point point = new Point();
			point.setPoint(plain);
			WaypointDesc waypoint = new WaypointDesc();
			Point[] coords = new Point[] { point };
			waypoint.setCoords(coords);
			waypoint.setLinkType(LinkType.AUTO_LINKING);
			waypointDesc1[index] = waypoint;
			index++;
		};
		
		Vector<Double> locations2 = new Vector<Double>();
		locations2.add(681013.0);
		locations2.add(6371680.0);
		locations2.add(685903.0);
		locations2.add(6372958.0);
	
		Iterator i2 = locations2.iterator();
		WaypointDesc[] waypointDesc2 = new WaypointDesc[locations2.size()/2];
		index = 0;
		while (i2.hasNext()) {
			PlainPoint plain = new PlainPoint();
			double x = (Double) i2.next();
			double y = (Double) i2.next();
			plain.setX(x);
			plain.setY(y);
			plain.setZ(null); // optional
			Point point = new Point();
			point.setPoint(plain);
			WaypointDesc waypoint = new WaypointDesc();
			Point[] coords = new Point[] { point };
			waypoint.setCoords(coords);
			waypoint.setLinkType(LinkType.AUTO_LINKING);
			waypointDesc2[index] = waypoint;
			index++;
		};
		
		Vector<Double> locations3 = new Vector<Double>();
		locations3.add(676437.0);
		locations3.add(6448604.0);
		locations3.add(685099.0);
		locations3.add(6416093.0);
		locations3.add(660301.0);
		locations3.add(6427670.0);

		Iterator i3 = locations3.iterator();
		WaypointDesc[] waypointDesc3 = new WaypointDesc[locations3.size()/2];
		index = 0;
		while (i3.hasNext()) {
			PlainPoint plain = new PlainPoint();
			double x = (Double) i3.next();
			double y = (Double) i3.next();
			plain.setX(x);
			plain.setY(y);
			plain.setZ(null); // optional
			Point point = new Point();
			point.setPoint(plain);
			WaypointDesc waypoint = new WaypointDesc();
			Point[] coords = new Point[] { point };
			waypoint.setCoords(coords);
			waypoint.setLinkType(LinkType.AUTO_LINKING);
			waypointDesc3[index] = waypoint;
			index++;
		};
		
		WaypointDesc[][] waypoints = new WaypointDesc[][]{waypointDesc1, waypointDesc2, waypointDesc3};
		
		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 4: Define RoutingOption to allow segment violations.
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption[] options = new RoutingOption[] { hasViolations };

		BulkRouteInfo bulkRouteInfo = null;
		try {
			// Step 5: Call calculateBulkRouteInfo().
			bulkRouteInfo = xRouteClient.calculateBulkRouteInfo(waypoints, options, null);

		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return bulkRouteInfo;
	}

}
