/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate routing with country info vehicle options
 * 
 * (c) 2014 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.CountryInfo;
import com.ptvag.xserver.xroute.CountryInfoVehicleOptions;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.ExtendedRoute;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.TollCostInfo;
import com.ptvag.xserver.xroute.VehicleOption;
import com.ptvag.xserver.xroute.VehicleParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcExtendedRoute {

	public static void main(String[] args) {
		ExtendedRoute extendedRoute = extendedRoute();

		if (extendedRoute == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(extendedRoute);
		Object[] cols = new Object[] { "IU Code", "Distance", "TollDistance", "TollPrice" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Calculate Route With Country Vehicle Info Options");
		frame.setSize(600, 165);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(ExtendedRoute extendedRoute) {
		Object rows[][] = new Object[1][1];
		rows = new Object[extendedRoute.getCountryInfos().length][4];
		int rowIdx = 0;
		for (CountryInfo countryInfo : extendedRoute.getCountryInfos()) {
			int totalTollPrice = 0;
			int totalTollDistance = 0;

			for (TollCostInfo tollCostInfo : countryInfo.getTollCostInfos()) {
				totalTollPrice += tollCostInfo.getTollPrice();
				totalTollDistance += tollCostInfo.getTollDistance();
			}

			rows[rowIdx++] = new Object[] { countryInfo.getIuCode(),
					String.format("%.2f km", (float) countryInfo.getPartRouteInfo().getDistance() / 1000.0),
					String.format("%.2f km", (float) totalTollDistance / 1000.0),
					String.format("%.2f ", (float) totalTollPrice / 100.0) };
		}
		return rows;
	}

	private static ExtendedRoute extendedRoute() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Add locations for the given route.
		// Please notice, that the PTV Europe City Map Premium is required and
		// additional toll data
		// have to be licensed. Otherwise an error occurs after running the code
		// sample.
		double[][] points = new double[][] { { 262175.0, 6242590.0 }, // Paris
				{ 933441.0, 6268969.0 }, // Karlsruhe
				{ 1820467.0, 6134685.0 }, // Wien
				{ 1267389.0, 5978788.0 }, // Innsbruck
				{ 1262552.0, 5853474.0 } // Bolzano
		};

		WaypointDesc[] waypointDescs = new WaypointDesc[points.length];
		for (int i = 0; i < points.length; i++) {
			Point pt = new Point();
			pt.setPoint(new PlainPoint(points[i][0], points[i][1], null));

			waypointDescs[i] = new WaypointDesc();
			waypointDescs[i].setCoords(new Point[] { pt });
			waypointDescs[i].setLinkType(LinkType.AUTO_LINKING);
		}

		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 4: Define RoutingOption to allow segment violations.
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption[] options = new RoutingOption[] { hasViolations };
		
		// Step 5: Define details for output
		ResultListOptions details = new ResultListOptions();
		details.setDetailLevel(DetailLevel.STANDARD);

		// Step 6: Realize CountryInfoOptions plotting country info vehicle options
		CountryInfoVehicleOptions countryInfoVehicleOptions = new CountryInfoVehicleOptions();
		countryInfoVehicleOptions.setAllEuro(true);
		countryInfoVehicleOptions.setDetailedTollCosts(true);

		VehicleOption vehicleOption = new VehicleOption();
		vehicleOption.setParameter(VehicleParameter.TYPE);
		vehicleOption.setValue("TRU");
		VehicleOption[] vehicleOptions = { vehicleOption };
		countryInfoVehicleOptions.setOptions(vehicleOptions);

		ExtendedRoute extendedRoute = null;
		try {
			// Step 7: Call calculateExtendedRoute().
			extendedRoute = xRouteClient.calculateExtendedRoute(waypointDescs, options, null, details,
					countryInfoVehicleOptions);

		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route\n\n" + xre);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server\n\n" +xse);
		}
		return extendedRoute;
	}

}
