/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate isochrones
 * 
 * (c) 2014 PTV AG
 */
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainLinearRing;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.PlainPolygon;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.Polygon;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Geometries;
import com.ptvag.xserver.xmap.Geometry;
import com.ptvag.xserver.xmap.GeometryLayer;
import com.ptvag.xserver.xmap.GeometryOption;
import com.ptvag.xserver.xmap.GeometryOptions;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.ExpansionDescription;
import com.ptvag.xserver.xroute.ExpansionType;
import com.ptvag.xserver.xroute.Isochrone;
import com.ptvag.xserver.xroute.IsochroneDetail;
import com.ptvag.xserver.xroute.IsochroneOptions;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcIsochrones {

	final static int width = 600;
	final static int height = 600;
	static Point center;

	public static void main(String[] args) {
		Isochrone isochrone = getIsochrone();

		if (isochrone == null) {
			System.out.println("Validate error message in getIsochrone()!");
			return;
		}

		JLabel label = drawIsochrone(isochrone);

		if (label == null) {
			System.out.println("Validate error message in drawIsochrone()!");
			return;
		}

		runSample(label);
	}

	private static JLabel drawIsochrone(Isochrone isochrone) {
		String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		PlainPoint[] points = isochrone.getIsochrones()[0].getPolys().getLineString().getPoints();
		PlainLinearRing ring = new PlainLinearRing();
		ring.setPoints(points);
		PlainPolygon plainPoly = new PlainPolygon();
		plainPoly.setLinearRings(new PlainLinearRing[] { ring });
		Polygon polygon = new Polygon();
		polygon.setPolygon(plainPoly);

		Point referencePoint = center;

		Geometry geometry = new Geometry();
		geometry.setDescription("Isochrones");
		geometry.setGeometry(polygon);
		geometry.setId(1);
		geometry.setReferencePoint(referencePoint);
		Geometry[] geometries = new Geometry[] { geometry };

		GeometryOption geometryCenterObjects = new GeometryOption();
		geometryCenterObjects.setOption(GeometryOptions.AUTOCENTEROBJECTS);
		geometryCenterObjects.setValue("1");

		String colorIsochrone = XMapUtils.rgbToColorString(50, 50, 255);
		GeometryOption geometryFillColor = new GeometryOption();
		geometryFillColor.setOption(GeometryOptions.FILLCOLOR);
		geometryFillColor.setValue(colorIsochrone);

		String colorLine = XMapUtils.rgbToColorString(255, 0, 0);
		GeometryOption geometryLineColor = new GeometryOption();
		geometryLineColor.setOption(GeometryOptions.LINECOLOR);
		geometryLineColor.setValue(colorLine);

		GeometryOption geometryLineWidth = new GeometryOption();
		geometryLineColor.setOption(GeometryOptions.LINEWIDTH);
		geometryLineColor.setValue("6");

		GeometryOption[] options = new GeometryOption[] { geometryCenterObjects, geometryFillColor, geometryLineColor,
				geometryLineWidth };

		Geometries isochrones = new Geometries(geometries, options);

		GeometryLayer geometryLayer = new GeometryLayer();
		geometryLayer.setName("Isochrones");
		geometryLayer.setDrawPriority(100);
		geometryLayer.setVisible(true);
		geometryLayer.setObjectInfos(ObjectInfoType.NONE);
		geometryLayer.setGeometries(new Geometries[] { isochrones });

		Layer[] layers = new Layer[] { geometryLayer };

		MapSection mapSection = new MapSection();
		mapSection.setCenter(referencePoint);

		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(600);
		imageInfo.setWidth(600);
		imageInfo.setImageParameter("");
		imageInfo.setFormat(ImageFileFormat.PNG);

		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT"); // optional

		JLabel label = new JLabel();
		try {
			Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, false);

			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				label.setIcon(new ImageIcon(new URL(imageUrl)));
				label.setText("");
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap): " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map\n\n" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server\n\n" + xse);
		}

		return label;
	}

	private static void runSample(JLabel label) {
		JPanel panel = new JPanel();
		panel.add(label);
		panel.setSize(width, height);
		// display image in window
		JFrame frame = new JFrame("Draw Isochrone");
		frame.setSize(width + 20, height + 55);
		frame.getContentPane().add(panel);
		frame.setVisible(true);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Isochrone getIsochrone() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define center
		center = new Point();
		PlainPoint loc = new PlainPoint();
		loc.setX(681593);
		loc.setY(6371793);
		loc.setZ(null); // optional
		center.setPoint(loc);

		WaypointDesc sink = new WaypointDesc();
		sink.setCoords(new Point[] { center });
		sink.setLinkType(LinkType.AUTO_LINKING); // optional

		// Step 4: Create expansionDesc and set distance for maximum horizon
		ExpansionDescription expansionDesc = new ExpansionDescription();
		expansionDesc.setExpansionType(ExpansionType.EXP_DIST);
		expansionDesc.setHorizons(new int[] { 20000 });

		// Step 5: Set options for isochrones.
		IsochroneOptions isochroneOptions = new IsochroneOptions();
		isochroneOptions.setExpansionDesc(expansionDesc);
		isochroneOptions.setIsoDetail(IsochroneDetail.POLY_AND_SEGMENTS);

		// Step 6: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		Isochrone isochrone = null;
		try {
			// Step 6: Call calculateIsochrones().
			isochrone = xRouteClient.calculateIsochrones(sink, null, isochroneOptions);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route\n\n" + xre);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server\n\n" + xse);
		}
		return isochrone;
	}
}
