/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate matrix information
 * 
 * (c) 2014 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.MatrixInfo;
import com.ptvag.xserver.xroute.MatrixOptions;
import com.ptvag.xserver.xroute.RouteInfo;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcMatrixInfo {

	public static void main(String[] args) {
		MatrixInfo matrixInfo = matrixInformation();

		if (matrixInfo == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(matrixInfo);
		Object[] cols = new Object[] { "Index", "Distance [m]", "Travel time [s]", "Has violation" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Calculate Matrix Information");
		frame.setSize(600, 165);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(MatrixInfo matrixInfo) {
		Object rows[][] = new Object[matrixInfo.getMatrixInfo().length][4];

		int index = 0;
		for (RouteInfo routeInfo : matrixInfo.getMatrixInfo()) {
			rows[index][0] = "Relation " + (index + 1);
			rows[index][1] = routeInfo.getDistance();
			rows[index][2] = routeInfo.getTime();
			rows[index][3] = routeInfo.getHasViolations();
			index++;
		}
		return rows;
	}

	private static MatrixInfo matrixInformation() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define locations for start
		WaypointDesc[] startWPs = new WaypointDesc[2];

		// Start 1: Rue Principale 27, L-5240 Sandweiler
		Point start_1 = new Point();
		PlainPoint plainstart_1 = new PlainPoint();
		plainstart_1.setX(691495);
		plainstart_1.setY(6372987);
		plainstart_1.setZ(null); // optional
		start_1.setPoint(plainstart_1);
		startWPs[0] = new WaypointDesc();
		startWPs[0].setCoords(new Point[] { start_1 });

		// Start 2: Steil 2, L-9651 Eschweiler
		Point start_2 = new Point();
		PlainPoint plainstart_2 = new PlainPoint();
		plainstart_2.setX(661614);
		plainstart_2.setY(6438775);
		plainstart_2.setZ(null); // optional
		start_2.setPoint(plainstart_2);
		startWPs[1] = new WaypointDesc();
		startWPs[1].setCoords(new Point[] { start_2 });

		// Step 3: Define locations for destination
		WaypointDesc[] destWPs = new WaypointDesc[3];

		// Destination 1: Rue du Nord 16, L-8806 Rambrouch
		Point dest_1 = new Point();
		PlainPoint plaindest_1 = new PlainPoint();
		plaindest_1.setX(650023);
		plaindest_1.setY(6410109);
		plaindest_1.setZ(null); // optional
		dest_1.setPoint(plaindest_1);
		destWPs[0] = new WaypointDesc();
		destWPs[0].setCoords(new Point[] { dest_1 });

		// Destination 2: Burrewee 3, L-9760 Wilwerwiltz
		Point dest_2 = new Point();
		PlainPoint plaindest_2 = new PlainPoint();
		plaindest_2.setX(668520);
		plaindest_2.setY(6436053);
		plaindest_2.setZ(null); // optional
		dest_2.setPoint(plaindest_2);
		destWPs[1] = new WaypointDesc();
		destWPs[1].setCoords(new Point[] { dest_2 });

		// Destination 3: Rue des Violettes 10, L-4940 Bascharage
		Point dest_3 = new Point();
		PlainPoint plaindest_3 = new PlainPoint();
		plaindest_3.setX(657647);
		plaindest_3.setY(6365485);
		plaindest_3.setZ(null); // optional
		dest_3.setPoint(plaindest_3);
		destWPs[2] = new WaypointDesc();
		destWPs[2].setCoords(new Point[] { dest_3 });

		// Step 4: Create callerContextProperty and set profile "truckfast" and
		// select coordinate format.
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty coordFormat = new CallerContextProperty();
		coordFormat.setKey("CoordFormat");
		coordFormat.setValue("PTV_MERCATOR");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile, coordFormat };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 5: Define RoutingOption to allow segment violations.
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption[] options = new RoutingOption[] { hasViolations };

		// Step 6: Realize matrixOptions
		MatrixOptions matrixOptions = new MatrixOptions();
		int ncFilter[] = new int[] { 1, 1, 1, 1, 1, 0, 0 };
		matrixOptions.setNcFilter(ncFilter);

		MatrixInfo matrixInfo = null;
		try {
			// Step 7: Call calculateMatrixInfo().
			matrixInfo = xRouteClient.calculateMatrixInfo(startWPs, destWPs, options, matrixOptions);

		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return matrixInfo;
	}

}
