/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate reachable objects
 * 
 * (c) 2014 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.ExpansionDescription;
import com.ptvag.xserver.xroute.ExpansionType;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.Reach;
import com.ptvag.xserver.xroute.ReachInfo;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcReachableObjects {

	public static void main(String[] args) {
		Reach reach = reachObjects();

		if (reach == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(reach);
		Object[] cols = new Object[] { "Is reachable", "Distance [m]", "Travel time [s]" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Calculate Reachable Objects");
		frame.setSize(600, 180);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(Reach reach) {
		Object rows[][] = new Object[reach.getReachInfo().length][3];

		int index = 0;
		for (ReachInfo ri : reach.getReachInfo()) {
			if (ri.isReachable()) {
				rows[index][0] = "true";
				rows[index][1] = ri.getRouteInfo().getDistance();
				rows[index][2] = ri.getRouteInfo().getTime();
			} else {
				rows[index][0] = "false";
				rows[index][1] = "null";
				rows[index][2] = "null";
			}
			index++;
		}

		return rows;
	}

	private static Reach reachObjects() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		
		// Step 2: Define home location.
		Point home = new Point();
		PlainPoint plainHome = new PlainPoint();
		plainHome.setX(681593);
		plainHome.setY(6371793);
		plainHome.setZ(null); // optional
		home.setPoint(plainHome);

		WaypointDesc sink = new WaypointDesc();
		sink.setCoords(new Point[] { home });
		sink.setLinkType(LinkType.NEXT_SEGMENT);
		
		// Step 3: Define locations to detect the reachability according to the
		// given maximum distance (or time period) from the pre-defined home
		// location.
		Point loc1 = new Point();
		PlainPoint plainLoc1 = new PlainPoint();
		plainLoc1.setX(678909);
		plainLoc1.setY(6395906);
		plainLoc1.setZ(null); // optional
		loc1.setPoint(plainLoc1);

		WaypointDesc location1 = new WaypointDesc();
		location1.setCoords(new Point[] { loc1 });

		Point loc2 = new Point();
		PlainPoint plainLoc2 = new PlainPoint();
		plainLoc2.setX(678009);
		plainLoc2.setY(6398906);
		plainLoc2.setZ(null); // optional
		loc2.setPoint(plainLoc2);
		
		WaypointDesc location2 = new WaypointDesc();
		location2.setCoords(new Point[] { loc2 });

		Point loc3 = new Point();
		PlainPoint plainLoc3 = new PlainPoint();
		plainLoc3.setX(669829);
		plainLoc3.setY(6415420);
		plainLoc3.setZ(null); // optional
		loc3.setPoint(plainLoc3);

		WaypointDesc location3 = new WaypointDesc();
		location3.setCoords(new Point[] { loc3 });
		location3.setLinkType(LinkType.NEXT_SEGMENT);

		Point loc4 = new Point();
		PlainPoint plainLoc4 = new PlainPoint();
		plainLoc4.setX(666922);
		plainLoc4.setY(6404844);
		plainLoc4.setZ(null); // optional
		loc4.setPoint(plainLoc4);
		
		WaypointDesc location4 = new WaypointDesc();
		location4.setCoords(new Point[] { loc4 });

		Point loc5 = new Point();
		PlainPoint plainLoc5 = new PlainPoint();
		plainLoc5.setX(691754);
		plainLoc5.setY(6406247);
		plainLoc5.setZ(null); // optional
		loc5.setPoint(plainLoc5);

		WaypointDesc location5 = new WaypointDesc();
		location5.setCoords(new Point[] { loc5 });

		Point loc6 = new Point();
		PlainPoint plainLoc6 = new PlainPoint();
		plainLoc6.setX(703879);
		plainLoc6.setY(6362696);
		plainLoc6.setZ(null); // optional
		loc6.setPoint(plainLoc6);

		WaypointDesc location6 = new WaypointDesc();
		location6.setCoords(new Point[] { loc6 });

		Point loc7 = new Point();
		PlainPoint plainLoc7 = new PlainPoint();
		plainLoc7.setX(653259);
		plainLoc7.setY(6363047);
		plainLoc7.setZ(null); // optional
		loc7.setPoint(plainLoc7);

		WaypointDesc location7 = new WaypointDesc();
		location7.setCoords(new Point[] { loc7 });

		WaypointDesc[] locations = new WaypointDesc[] { location1, location2, location3, location4, location5, location6,
				location7 };
				
		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 4: Enter the maximum distance (m) or period (s)
		ExpansionDescription expansionDescription = new ExpansionDescription();
		expansionDescription.setExpansionType(ExpansionType.EXP_DIST);
		// expansionDescription.setExpansionType(ExpansionType.EXP_TIME);
		expansionDescription.setHorizons(new int[] { 22000 });
		// expansionDescription.setHorizons(new int[] {1200});

		Reach reach = null;
		try {
			// Step 5: Call calculateReachableObjects().
			reach = xRouteClient.calculateReachableObjects(sink, null, locations, null, expansionDescription);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return reach;
	}

}
