/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate routing
 * 
 * (c) 2014 PTV AG
 */
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcRoute {

	final static int width = 640;
	final static int height = 480;
	
	public static void main(String[] args) {
		Route route = route();

		if (route == null) {
		System.out.println("Validate error message in route()!");
		return;
		}
		
		JLabel label = drawPolygon(route); 
		
		if (label == null) {
			System.out.println("Validate error message in drawPolygon()!");
			return;
			}
		
		runSample(label);
	}
	
	private static JLabel drawPolygon(Route route) {
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// use bounding box of route
		BoundingBox boundingBox = new BoundingBox();  
		boundingBox.setLeftTop( route.getTotalRectangle().getLeftBottom() );  
		boundingBox.setRightBottom( route.getTotalRectangle().getRightTop() );  

		// get "polygon" from route
		LineString[] lineStrings = new LineString[]{ route.getPolygon()};

		// set line options ( color, stroke size )			 
		LineOptions options =  new LineOptions();  
		LinePartOptions linePartOptions = new LinePartOptions();
		linePartOptions.setVisible(true);
		linePartOptions.setColor(new Color(255 ,0 ,0));
		linePartOptions.setWidth(-6);
		options.setMainLine( linePartOptions );
		options.setShowFlags(true);
		options.setTransparent(true);

		Lines[] linesArray = new Lines[]{ new Lines(lineStrings,options) };
		
		// put lines into a CustomLayer 
		CustomLayer customLayer = new CustomLayer();
		customLayer.setName("routeLayer");
		customLayer.setLines( linesArray );
		customLayer.setDrawPriority(500);    
		customLayer.setCenterObjects(true);  
		customLayer.setObjectInfos( ObjectInfoType.NONE);
		customLayer.setVisible(true);
		
		Layer[] layers = new Layer[] {customLayer};
	
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setHeight(height);
		imageInfo.setWidth(width);
		
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(false);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT");
		
		JLabel label = new JLabel();
		try {
			Map map = xMapClient.renderMapBoundingBox(
					boundingBox, mapParams, imageInfo, layers, false);

			String imageUrl = "http://"+map.getImage().getUrl();
			try {
				label.setIcon( new ImageIcon(new URL(imageUrl)) );
				label.setText("");
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with rendering" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server" + xse);
		}
		
		return label;
	}

	private static void runSample(JLabel label) {
		JPanel panel = new JPanel();
		panel.add(label);
		panel.setSize(width, height);
		// display image in window
		JFrame frame = new JFrame("Draw Routing");
		frame.setSize(width+20,height+52);
		frame.getContentPane().add(panel);
		frame.setVisible(true);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Route route() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define locations for start and destination
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(681593, 6371793, null));
		WaypointDesc wpdStart = new WaypointDesc();
		wpdStart.setCoords(new Point[] { startPoint });
		wpdStart.setLinkType(LinkType.NEXT_SEGMENT);

		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(678909, 6395906, null));
		WaypointDesc wpdDest = new WaypointDesc();
		wpdDest.setCoords(new Point[] { destPoint });
		wpdDest.setLinkType(LinkType.NEXT_SEGMENT);

		WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 4: Define RoutingOption to allow segment violations.
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption[] options = new RoutingOption[] { hasViolations };
		
		// Step 5: Define result list option setting polygon and total rectangle true.
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);

		Route route = null;
		try {
			// Step 6: Call calculateRoute().
			route = xRouteClient.calculateRoute(waypointDescs, options, null, resultListOptions);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route\n\n" + xre);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server\n\n" + xse);
		}
		return route;
	}
}
