/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - calculate tour
 * 
 * (c) 2014 PTV AG
 */
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.Interval;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.RoutingOption;
import com.ptvag.xserver.xroute.RoutingParameter;
import com.ptvag.xserver.xroute.Tour;
import com.ptvag.xserver.xroute.TourPointDesc;
import com.ptvag.xserver.xroute.TourSummary;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class CalcTour {

	public static void main(String[] args) {
		final Calendar earliestStartTime = new GregorianCalendar(2014, 9, 13,  7, 0, 0);
		Tour tourInfo = tourInformation(earliestStartTime);

		if (tourInfo == null) {
		System.out.println("Validate error message!");
		return;
		}
		
		Object[][] rows = resultList(tourInfo);
		Object[] cols = new Object[] { "Name", "Value" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Calculate Tour Summary");
		frame.setSize(400, 156);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(Tour tourInfo) {
		Object rows[][] = new Object[5][2];

		TourSummary tour = tourInfo.getTourSummary();
		rows[0][0] = "Driving Period (s)";
		rows[0][1] = tour.getDrivingPeriod();
		rows[1][0] = "Service Period (s)";
		rows[1][1] = tour.getServicePeriod();
		rows[2][0] = "Total Period (s)";
		rows[2][1] = tour.getTotalPeriod();
		rows[3][0] = "Latest Tour Start";
		rows[3][1] = tour.getLatestTourStart().getTime(); 
		rows[4][0] = "Waiting Period (s)";
		rows[4][1] = tour.getWaitingPeriod();
		
		return rows;
	}

	private static Tour tourInformation(Calendar startTime ) {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		// Step 2: Define tour points for start, via and destination
		Point startPoint = new Point();
		startPoint.setPoint(new PlainPoint(676437, 6448604, null));
		TourPointDesc start = new TourPointDesc();
		start.setCoords(new Point[] { startPoint });
		start.setServicePeriod(9000);
		start.setUseServicePeriodForRecreation(false);
		start.setCompleteServiceInIntervals(true);
		start.setLinkType(LinkType.AUTO_LINKING);
		start.setOpeningIntervals(new Interval[]{
				new Interval(
						new GregorianCalendar(2014, 9, 13,  8, 0, 0),
						new GregorianCalendar(2014, 9, 13, 18, 0, 0)
				)
		});

		Point viaPoint = new Point();
		viaPoint.setPoint(new PlainPoint(685099, 6416093, null));
		TourPointDesc via = new TourPointDesc();
		via.setCoords(new Point[] { viaPoint });
		via.setServicePeriod(8000);
		via.setUseServicePeriodForRecreation(false);
		via.setCompleteServiceInIntervals(true);
		via.setLinkType(LinkType.AUTO_LINKING);
		via.setOpeningIntervals(new Interval[]{
				new Interval(
						new GregorianCalendar(2014, 9, 13,  8, 0, 0),
						new GregorianCalendar(2014, 9, 13, 18, 0, 0)
				)
		});
		
		Point destPoint = new Point();
		destPoint.setPoint(new PlainPoint(660301, 6427670, null));
		TourPointDesc dest = new TourPointDesc();
		dest.setCoords(new Point[] { destPoint });
		dest.setServicePeriod(7000);
		dest.setUseServicePeriodForRecreation(false);
		dest.setCompleteServiceInIntervals(true);
		dest.setLinkType(LinkType.AUTO_LINKING);
		dest.setOpeningIntervals(new Interval[]{
				new Interval(
						new GregorianCalendar(2014, 9, 13,  8, 0, 0),
						new GregorianCalendar(2014, 9, 13, 18, 0, 0)
				)
		});

		WaypointDesc[] tourPoints = new WaypointDesc[] { start, via, dest };

		// Step 3: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty coordFormat = new CallerContextProperty();
		coordFormat.setKey("CoordFormat");
		coordFormat.setValue("PTV_MERCATOR");
		CallerContextProperty response = new CallerContextProperty();
		response.setKey("ResponseGeometry");
		response.setValue("PLAIN");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile, coordFormat, response };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 4: Define RoutingOption to allow segment violations and set reference time.
		RoutingOption hasViolations = new RoutingOption();
		hasViolations.setParameter(RoutingParameter.ALLOW_SEGMENT_VIOLATIONS);
		hasViolations.setValue("true");
		RoutingOption referenceTime = new RoutingOption();
		referenceTime.setParameter(RoutingParameter.START_TIME);
		final SimpleDateFormat rfc3339_format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
		referenceTime.setValue(rfc3339_format.format(startTime.getTime()));
		System.out.println("Start_Time: " + rfc3339_format.format(startTime.getTime()));
		RoutingOption[] options = new RoutingOption[] { hasViolations, referenceTime };
		
		// Step 5: Add ResultListOptions and set manoeuvres, texts, segments and detail level.
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setManoeuvres(false);
		resultListOptions.setTexts(false);
		resultListOptions.setSegments(false);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);
		
		
		Tour tourInfo = null;
		try {
			// Step 6: Call calculateRouteInfo(). ExceptionPath and CountryInfoOptions are not yet used.
			tourInfo = xRouteClient.calculateTour(tourPoints, options, null, resultListOptions, null);

		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return tourInfo;
	}

}
