/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * The entire sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xroute.basic;

/*
 * PTV xServer Code Samples
 * 
 * PTV xRoute Server - search for reachable objects
 * 
 * (c) 2014 PTV AG
 */
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xroute.ExpansionDescription;
import com.ptvag.xserver.xroute.ExpansionType;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ReachableObject;
import com.ptvag.xserver.xroute.ReachableObjects;
import com.ptvag.xserver.xroute.ReachableObjectsOptions;
import com.ptvag.xserver.xroute.RoutingDirectionType;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class SearchForReachableObjects {

	public static void main(String[] args) {
		ReachableObjects reachableObjects = reachObjects();

		if (reachableObjects == null) {
			System.out.println("Validate error message!");
			return;
		}

		Object[][] rows = resultList(reachableObjects);
		Object[] cols = new Object[] { "Object", "Distance [m]", "Travel time [s]" };

		runSample(rows, cols);
	}

	private static void runSample(Object[][] rows, Object[] cols) {
		JTable table = new JTable(rows, cols);

		JScrollPane scrollPane = new JScrollPane(table);

		JFrame frame = new JFrame("Search For Reachable Objects");
		frame.setSize(800, 500);
		frame.getContentPane().add(scrollPane);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static Object[][] resultList(ReachableObjects reachableObjects) {
		Object rows[][] = new Object[reachableObjects.getReachableObject().length][3];
		
		int index = 0;
		
		for (ReachableObject reach : reachableObjects.getReachableObject()){
		double x = reach.getObject().getCoord().getPoint().getX();
		double y = reach.getObject().getCoord().getPoint().getY();
		String point = String.format("Point = %.2f ;%.2f", x, y);
		String id = String.format("ID = %s;", reach.getObject().getId());
		rows[index][0] =	id + point;
		rows[index][1] =	reach.getRouteInfo().getDistance();
		rows[index][2] =	reach.getRouteInfo().getTime();
		
		index++;
		}

		return rows;
	}

	private static ReachableObjects reachObjects() {
		// Step 1: Instantiate xRoute client. Use the provided package utils.
		String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils.makeClient(XROUTE_WS_URL);

		
		// Step 2: Define home location.
		Point home = new Point();
		PlainPoint plainHome = new PlainPoint();
		plainHome.setX(680493);
		plainHome.setY(6371893);
		plainHome.setZ(null); // optional
		home.setPoint(plainHome);

		WaypointDesc sink = new WaypointDesc();
		sink.setCoords(new Point[] { home });
		sink.setLinkType(LinkType.NEXT_SEGMENT);
				
		// Step 2: Create callerContextProperty and set profile "truckfast".
		CallerContextProperty profile = new CallerContextProperty();
		profile.setKey("Profile");
		profile.setValue("truckfast");
		CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		CallerContext ctx = new CallerContext();
		ctx.setProperties(callerContextProperties);

		xRouteClient.setCallerContext(ctx);

		// Step 3: Get reachable objects from derby database using POI-Layer.
		ReachableObjectsOptions reachableObjectOptions = new ReachableObjectsOptions();
		reachableObjectOptions.setGeodatasourceLayer("default.points-of-interest");
		ExpansionDescription expansionDescription = new ExpansionDescription();
		expansionDescription.setExpansionType(ExpansionType.EXP_DIST);
		// expansionDescription.setExpansionType(ExpansionType.EXP_TIME);
		expansionDescription.setHorizons(new int[] { 1000 });
		// expansionDescription.setHorizons(new int[] {120});
		reachableObjectOptions.setExpansionDesc(expansionDescription);
		reachableObjectOptions.setRoutingDirection(RoutingDirectionType.FORWARD);
		reachableObjectOptions.setBoundingBoxCount(5);
		
		ReachableObjects reachableObjects = null;
		try {
			// Step 5: Call calculateReachableObjects().
			reachableObjects = xRouteClient.searchForReachableObjects(sink, null, null, reachableObjectOptions, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem: " + se);
		} catch (XRouteException xre) {
			System.err.println("Problem with calculating a route");
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server");
		}
		return reachableObjects;
	}

}
