/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample codes are provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2014 PTV AG
 ******************************************************************************/
package samples.xsaas;
/*
 * PTV xServer Code Samples
 * 
 * PTV xServer internet - Basic Mapping
 * 
 * (c) 2014 PTV AG
 */

import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.XMapUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;


public class BasicAccessWithxMap extends JPanel {
	Dimension imageSize = new Dimension(240, 320);
	JLabel label = new JLabel("Loading ...");

	private void runSample() {

		// step 1:
		// instantiate remote interface for xMap
		String country = "eu";  // use eu, na or au
		String url = "";
		
		if (country.equals("eu")) {
			url = "https://xmap-eu-n-test.cloud.ptvgroup.com/xmap/ws/XMap";
		} else if (country.equals("na")) {
			url = "https://xmap-na-n-test.cloud.ptvgroup.com/xmap/ws/XMap";
		} else {
			url = "https://xmap-au-n-test.cloud.ptvgroup.com/xmap/ws/XMap";
		}
		
		XMapRemoteInterface client = XMapUtils.makeClient(url);
		client.setUsername("xtok");
		client.setPassword("add_your_token");  

		// step 2:
		// create object MapSection
		MapSection mapSection = new MapSection();
		Point point = new Point();
		PlainPoint plainPoint = new PlainPoint();
		double x = 0.0;
		double y = 0.0;
		
		if (country.equals("eu")) {
			x = 682007.75; // eu-n-test
			y = 6372537.96; // eu-n-test
		} else if (country.equals("na")) {
			x = -8229219.0; // na-n-test
			y = 4964765.0; // na-n-test
		} else {
			x = 16582009.0; // au-n-test
		  y = -4200906.0; // au-n-test
		}
			  
		plainPoint.setX(x);
		plainPoint.setY(y);
		point.setPoint(plainPoint);
		
		mapSection.setCenter(point);
		mapSection.setScale(1000);
		
		// step 3:
		// create object CallerContext
		CallerContext cxt = new CallerContext();
		cxt.setLog1("Basic Mapping");
		// set CallerContext properties to define profile and CoordFormat
		CallerContextProperty props1 = new CallerContextProperty();
		props1.setKey("profile");
		props1.setValue("default");
		CallerContextProperty props2 = new CallerContextProperty();
		props2.setKey("CoordFormat");
		props2.setValue("PTV_MERCATOR");
		CallerContextProperty[] properties = new CallerContextProperty[]{props1, props2};
		cxt.setProperties(properties);
		client.setCallerContext(cxt);

		// step 4:
		// create object ImageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setHeight(240);
		imageInfo.setWidth(320);
		imageInfo.setImageParameter("");

		// step 5:
		// create object MapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// step 6:
		// set attribute includeImageInResponse
		boolean includeImageInResponse = false;
		
		try {
			// step 7:
			// render map
			Map map = client.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse);

			// step 8:
			// retrieve further map values
			BoundingBox bbox = map.getVisibleSection().getBoundingBox(); 
			double leftTopX = bbox.getLeftTop().getPoint().getX(); 
			double leftTopY =	bbox.getLeftTop().getPoint().getY(); 
			double rightBottomX =	bbox.getLeftTop().getPoint().getX(); 
			double rightBottomY =	bbox.getLeftTop().getPoint().getY();
			int scale = map.getVisibleSection().getScale();
			
			System.out.println(String.format("Left top corner:(%.2f/%.2f)\nRight bottom corner:(%.2f/%.2f)\nMap scale: %s", 
					leftTopX, leftTopY, rightBottomX, rightBottomY, scale));
			
			// step 9:
			// get the image
			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				ImageIcon icon = new ImageIcon(new URL(imageUrl));
				label.setIcon(icon);
				label.setText("");
				this.setMinimumSize(new Dimension(icon.getIconWidth() + 20, icon.getIconHeight() + 30));
			} catch (MalformedURLException e) {
				System.err.println(e.getMessage());
			}	

		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap):\n" + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map\n" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server\n" + xse);
		} 

	}

	public BasicAccessWithxMap() {
		this.add(label);
		Thread thread = new Thread() {

			public void run() {
				runSample();
			}
		};
		thread.start();
	}

	public static void main(String[] args) {

		// display image in window
		JFrame frame = new JFrame("PTV xServer internet - Basic Mapping");
		JPanel sample = new BasicAccessWithxMap();
		frame.getContentPane().add(sample);
		frame.setVisible(true);
		frame.pack();
		frame.setSize(340, 280);

		// exit when window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}

}
