/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xtour;

/*
 * PTV xServer Code Samples
 * 
 * PTV xTour Server - Sequencing with unscheduled order analysis
 * 
 * (c) 2013 PTV AG
 */
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XMapUtils;
import utils.XTourUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainLineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.Font;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.Text;
import com.ptvag.xserver.xmap.TextAlignment;
import com.ptvag.xserver.xmap.TextOptions;
import com.ptvag.xserver.xmap.Texts;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xtour.Depot;
import com.ptvag.xserver.xtour.DistanceMatrixByRoad;
import com.ptvag.xserver.xtour.Interval;
import com.ptvag.xserver.xtour.OpeningIntervalConstraint;
import com.ptvag.xserver.xtour.Quantities;
import com.ptvag.xserver.xtour.Restrictions;
import com.ptvag.xserver.xtour.SequencingParams;
import com.ptvag.xserver.xtour.SequencingPlan;
import com.ptvag.xserver.xtour.SequencingTour;
import com.ptvag.xserver.xtour.SequencingTourPoint;
import com.ptvag.xserver.xtour.SequencingTransportAB;
import com.ptvag.xserver.xtour.SequencingTransportDepot;
import com.ptvag.xserver.xtour.SequencingTransportOrder;
import com.ptvag.xserver.xtour.SequencingVehicle;
import com.ptvag.xserver.xtour.TourEvent;
import com.ptvag.xserver.xtour.TourRestrictions;
import com.ptvag.xserver.xtour.TourViolations;
import com.ptvag.xserver.xtour.TransportPoint;
import com.ptvag.xserver.xtour.UnscheduledOrderAnalysisParams;
import com.ptvag.xserver.xtour.UnscheduledOrderAnalysisResult;
import com.ptvag.xserver.xtour.VehicleViolations;
import com.ptvag.xserver.xtour.XTourException;
import com.ptvag.xserver.xtour.XTourRemoteInterface;

public class SequencingWithUnscheduledOrderAnalysis extends JPanel {

	public static Hashtable<Integer, double[]> numbers;

	public static void main(String[] args) {
		// add xTour
		SequencingPlan plan = null;
		plan = runSequencingSample(plan);

		// add tour events
		Vector<Object[][]> jSequencingTourEvents = new Vector<Object[][]>();
		jSequencingTourEvents = runSequencingResultTourEvents(plan, jSequencingTourEvents);

		// add scheduled order id
		Vector<Object[][]> jSequencingScheduledOrderID = new Vector<Object[][]>();
		jSequencingScheduledOrderID = runSequencingResultOrderID(plan, jSequencingScheduledOrderID);

		// add tour points
		Vector<Object[][]> jSequencingTourPoints = new Vector<Object[][]>();
		jSequencingTourPoints = runSequencingResultTourPoints(plan, jSequencingTourPoints);

		// add general results
		Vector<Object[][]> jSequencingResults = new Vector<Object[][]>();
		jSequencingResults = runSequencingResult(plan, jSequencingResults);

		// call xMap to draw routing
		ImageIcon icon = null;
		icon = runMappingSample(plan, icon);

		// visualize jSequencingTourPoints
		JTable tableSequencingTourPoints = new JTable(jSequencingTourPoints.firstElement(), new Object[] { "Description",
				"Driving Distance [km]", "Arrival Time [h:m:s]", "Driving Period [h:m:s]" });
		JScrollPane scrollPaneSequencingTourPoints = new JScrollPane(tableSequencingTourPoints);

		JFrame frameSequencingTourPoints = new JFrame("Tour points");
		frameSequencingTourPoints.getContentPane().add(scrollPaneSequencingTourPoints);
		frameSequencingTourPoints.setVisible(true);
		frameSequencingTourPoints.setSize(800, 180);
		frameSequencingTourPoints.setLocation(icon.getIconWidth() + 20, 0);
		frameSequencingTourPoints.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// visualize jSequencingTourEvents
		JTable tableSequencingTourEvents = new JTable(jSequencingTourEvents.firstElement(), new Object[] { "ID",
				"Start [h:m:s]", "Period [h:m:s]", "Type" });
		JScrollPane scrollPaneSequencingTourEvents = new JScrollPane(tableSequencingTourEvents);

		JFrame frameSequencingTourEvents = new JFrame("Tour events");
		frameSequencingTourEvents.getContentPane().add(scrollPaneSequencingTourEvents);
		frameSequencingTourEvents.setVisible(true);
		frameSequencingTourEvents.setSize(600, 280);
		frameSequencingTourEvents.setLocation(icon.getIconWidth() + 20, 180);
		frameSequencingTourEvents.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// visualize jSequencingResults
		JTable tableSequencingResults = new JTable(jSequencingResults.firstElement(), new Object[] { "Type", "Result" });
		JScrollPane scrollPaneSequencingResults = new JScrollPane(tableSequencingResults);
		JFrame frameSequencingResults = new JFrame("Tour results");
		frameSequencingResults.getContentPane().add(scrollPaneSequencingResults);
		frameSequencingResults.setVisible(true);
		frameSequencingResults.setSize(500, 180);
		frameSequencingResults.setLocation(icon.getIconWidth() + 20, 460);
		frameSequencingResults.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// visualize jSequencingOrderIDs
		JTable tableSequencingOrderId = new JTable(jSequencingScheduledOrderID.firstElement(), new Object[] { "Type",
				"Order ID", "Analysis", "Tour Violations", "Vehicle Violations" });
		JScrollPane scrollPaneSequencingOrderID = new JScrollPane(tableSequencingOrderId);
		JFrame frameSequencingOrderId = new JFrame("Shipment Information");
		frameSequencingOrderId.getContentPane().add(scrollPaneSequencingOrderID);
		frameSequencingOrderId.setVisible(true);
		frameSequencingOrderId.setSize(800, 135);
		frameSequencingOrderId.setLocation(icon.getIconWidth() + 20, 640);
		frameSequencingOrderId.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// draw route onto map
		JFrame frame = new JFrame("Basic Sequencing");
		frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 50);
		frame.getContentPane().add(new JLabel(icon));
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static ImageIcon runMappingSample(SequencingPlan plan, ImageIcon icon) {
		// Step 1: Instantiate xMap client. Use package utils to create java
		// client automatically.
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Create object MapSection
		MapSection mapSection = new MapSection();
		Point point = new Point();
		double xy[] = numbers.get(0);
		point.setPoint(new PlainPoint(xy[0], xy[1], null));
		mapSection.setCenter(point);
		mapSection.setScale(500);

		// Step 3: Define an array of lines
		int numberOfDestinationPoints = 0;
		for (TourEvent tourEvents : plan.getResult().getTourEvents()) {
			int tourPointID = tourEvents.getLogisticPointId();
			if (tourPointID != -1) {
				numberOfDestinationPoints++;
			}
		}
		// System.out.println("Number of stops: " + numberOfDestinationPoints);
		PlainPoint[] destinationPlainPoints = new PlainPoint[numberOfDestinationPoints];
		int[] orderID = new int[numberOfDestinationPoints];

		int index = 0;
		for (TourEvent tourEvents : plan.getResult().getTourEvents()) {
			int tourPointID = tourEvents.getLogisticPointId();

			if (tourPointID != -1) {
				double loc[] = numbers.get(tourPointID);
				orderID[index] = tourPointID;
				// System.out.println("Locations: " + tourPointID + "  " + loc[0] + "  "
				// + loc[1]);
				PlainPoint plainPoint = new PlainPoint();
				plainPoint.setX(loc[0]);
				plainPoint.setY(loc[1]);
				destinationPlainPoints[index] = plainPoint;
				index++;
			}
		}

		Lines[] lines = new Lines[numberOfDestinationPoints];
		int[][] colorValues = new int[][] {
				// {red, green, blue}
				{ 0, 0, 255 }, { 0, 0, 255 }, { 0, 255, 0 }, { 238, 134, 28 }, { 139, 123, 108 }, { 79, 79, 47 },
				{ 131, 139, 131 }, { 0, 290, 139 } };

		for (int i = 0; i < numberOfDestinationPoints - 1; i++) {
			// Step 3.1: Create plainLineString
			PlainLineString plainLineString = new PlainLineString();
			plainLineString.setPoints(new PlainPoint[] { destinationPlainPoints[i], destinationPlainPoints[i + 1] });
			// Step 3.2: Set object into lineStrings[0]. Note: object lineStrings has
			// to be reinitialized in every loop.
			LineString[] lineStrings = new LineString[1];
			lineStrings[0] = new LineString();
			lineStrings[0].setLineString(plainLineString);
			// Step 3.3: Create new lines[i] and fill array
			lines[i] = new Lines();
			lines[i].setLines(lineStrings);
			// Step 3.4: Set lineOptions transparency
			LineOptions lineOptions = new LineOptions();
			lineOptions.setTransparent(true);
			// Step 3.5: Set linePartOptions visible, define width of line and
			// specific color
			LinePartOptions linePartOptions = new LinePartOptions();
			linePartOptions.setVisible(true);
			linePartOptions.setWidth(5);
			linePartOptions.setColor(new Color(colorValues[0][0], colorValues[0][1], colorValues[0][2]));
			// Step 3.6: Set main line
			lineOptions.setMainLine(linePartOptions);
			// Step 3.7: Set all options for specific line
			lines[i].setOptions(lineOptions);
		}

		// Step 4: Generate a new object named bitmap as type of Bitmap[]
		Bitmap[] bitmap = new Bitmap[numberOfDestinationPoints];
		for (int i = 0; i < numberOfDestinationPoints - 1; i++) {
			Point bitmapPointDestination = new Point();

			if (i == 0) {
				bitmapPointDestination.setPoint(destinationPlainPoints[i]);
				bitmap[i] = new Bitmap();
				bitmap[i].setName("flaggreen.bmp");
				bitmap[i].setPosition(bitmapPointDestination);
				bitmap[i].setDescr("Start and Destination");
			} else {
				bitmapPointDestination.setPoint(destinationPlainPoints[i]);
				bitmap[i] = new Bitmap();
				bitmap[i].setName("flagred.bmp");
				bitmap[i].setPosition(bitmapPointDestination);
				bitmap[i].setDescr("Tourstop");
			}
		}

		// Step 5: Create bitmapOptions and set color for transparency
		BitmapOptions bitmapOptions = new BitmapOptions();
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 6: Create object bitmaps and put bitmap and bitmapOptions into its
		// array
		Bitmaps[] bitmaps = new Bitmaps[1];
		bitmaps[0] = new Bitmaps();
		bitmaps[0].setBitmaps(bitmap);
		bitmaps[0].setOptions(bitmapOptions);

		Text[] textTourpoints = new Text[numberOfDestinationPoints - 1];
		for (int i = 0; i < numberOfDestinationPoints - 1; i++) {
			Point pointText = new Point();
			pointText.setPoint(destinationPlainPoints[i]);

			Text text = new Text();
			text.setPosition(pointText);
			text.setText("" + orderID[i]);

			textTourpoints[i] = text;
		}

		// Step 7: Set text options
		Font font = new Font();
		font.setName("Arial");
		font.setSize(14);
		font.setBold(true);

		TextOptions textOptions = new TextOptions();
		textOptions.setFont(font);
		textOptions.setTextColor(new Color(0, 0, 0));
		textOptions.setFillBg(true);
		textOptions.setBgColor(new Color(255, 255, 255));
		textOptions.setShowFrame(true);
		textOptions.setFrameColor(new Color(0, 0, 255));
		textOptions.setAlignment(TextAlignment.CENTER);
		textOptions.setPixelX(40);
		textOptions.setPixelY(-5);
		textOptions.setRelY(0);
		textOptions.setRelY(0);

		// Step 8: If you want to style each text object differently
		// you should create an object "Texts" for each text.
		// This allows you to use different text options.
		Texts texts = new Texts();
		texts.setOptions(textOptions);
		texts.setTexts(textTourpoints);

		// Step 9: Define customLayer and set the relevant attributes
		CustomLayer customLayer = new CustomLayer();
		customLayer.setDrawPriority(25000);
		customLayer.setCenterObjects(true);
		customLayer.setName("Text layer");
		customLayer.setObjectInfos(ObjectInfoType.REFERENCEPOINT);
		customLayer.setVisible(true);
		customLayer.setBitmaps(bitmaps);
		customLayer.setLines(lines);
		customLayer.setTexts(new Texts[] { texts });

		// Step 10: Add customLayer into layers
		Layer[] layers = new Layer[] { customLayer };

		// Step 11: Create object ImageInfo
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setHeight(725);
		imageInfo.setWidth(800);

		// Step 12: Create object MapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		// Step 13: Set attribute includeImageInResponse
		boolean includeImageInResponse = false;
		try {

			// Step 14: Call renderMapBoundingBox and return object map.
			Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse);

			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				icon = new ImageIcon(new URL(imageUrl));
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}

		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap): " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server");
		}

		return icon;
	}

	private static Vector<Object[][]> runSequencingResult(SequencingPlan plan, Vector<Object[][]> jTourTables) {
		Object[][] rows = new Object[4][2];

		SequencingTour sequencingTour = plan.getTour();

		rows[0][0] = "Distance";
		rows[0][1] = sequencingTour.getResult().getDistance() / 1000 + " km";

		rows[1][0] = "Period";
		rows[1][1] = second2hours(sequencingTour.getResult().getPeriod()) + " [h]";

		rows[2][0] = "Driving Period";
		rows[2][1] = second2hours(sequencingTour.getResult().getDrivingPeriod()) + " [h]";

		rows[3][0] = "Service Period";
		rows[3][1] = second2hours(sequencingTour.getResult().getServicePeriod()) + " [h]";

		jTourTables.addElement(rows);

		return jTourTables;
	}

	private static Vector<Object[][]> runSequencingResultTourEvents(SequencingPlan plan, Vector<Object[][]> jTourTables) {
		int index = 0;

		Object[][] rows = new Object[plan.getResult().getTourEvents().length][5];

		// Get tour events
		for (TourEvent tourEvents : plan.getResult().getTourEvents()) {

			int id = tourEvents.getLogisticPointId();
			if (id == -1) {
				rows[index][0] = "";
			} else {
				rows[index][0] = id;
			}

			int start = tourEvents.getStartTime();
			rows[index][1] = second2hours(start);

			int period = tourEvents.getPeriod();
			rows[index][2] = second2hours(period);

			rows[index][3] = tourEvents.getDescriptions()[0];

			// System.out.println("Point: " + rows[index][0] + "   Start:" +
			// rows[index][1] + "   Period: " + rows[index][2] +
			// "   Index: " + rows[index][3] + "   Description:" + rows[index][4]);

			index++;
		}

		jTourTables.addElement(rows);

		return jTourTables;
	}

	private static Vector<Object[][]> runSequencingResultOrderID(SequencingPlan plan, Vector<Object[][]> jTourTables) {
		int index = 0;
		int scheduledOrderId = plan.getResult().getScheduledOrderIds().length;
		int unscheduledOrderId = 0;
		boolean available = false;

		if (plan.getResult().getUnscheduledOrderIds() != null) {
			unscheduledOrderId = plan.getResult().getUnscheduledOrderIds().length;
			available = true;
		}

		Object[][] rows = new Object[scheduledOrderId + unscheduledOrderId][5];

		// Get scheduled order Id
		for (int orderID : plan.getResult().getScheduledOrderIds()) {
			rows[index][0] = "scheduled";
			rows[index][1] = orderID;
			rows[index][2] = "okay";
			index++;
		}

		if (available) {
			for (int orderID : plan.getResult().getUnscheduledOrderIds()) {
				rows[index][0] = "unscheduled";
				rows[index][1] = orderID;
			}

			String result = "incomplete";
			String listofTourViolations = "";
			String listofVehicleViolations = "";
			if (plan.getResult().getUnscheduledOrderAnalysisResults() != null) {
				for (UnscheduledOrderAnalysisResult analysis : plan.getResult().getUnscheduledOrderAnalysisResults()) {
					result = analysis.getStatus().getValue();

					// int num1 = 0;
					for (TourViolations tourViolation : analysis.getTourViolations()) {
						// listofTourViolations = "Tourviolation-" + ++num1 + ": ";
						for (String violation : tourViolation.getViolations()) {
							listofTourViolations = listofTourViolations + violation + "; ";
						}
					}

					// int num2 = 0;
					for (VehicleViolations vehicleViolation : analysis.getVehicleViolations()) {
						// listofVehicleViolations = "Vehicleviolation-" + ++num2 + ": ";
						for (String violation : vehicleViolation.getViolations()) {
							listofVehicleViolations = listofVehicleViolations + violation + "; ";
						}
					}
				}

			}

			rows[index][2] = result;
			if (listofTourViolations.equals("")) {
				listofTourViolations = "no tour violations";
			}
			rows[index][3] = listofTourViolations;
			if (listofVehicleViolations.equals("")) {
				listofVehicleViolations = "no vehicle violations";
			}
			rows[index][4] = listofVehicleViolations;

			index++;
		}

		jTourTables.addElement(rows);

		return jTourTables;
	}

	private static Vector<Object[][]> runSequencingResultTourPoints(SequencingPlan plan, Vector<Object[][]> jTourTables) {
		int index = 0;

		Object[][] rows = new Object[plan.getTour().getTourPoints().length][5];

		// Get tour points
		for (SequencingTourPoint tourPoint : plan.getTour().getTourPoints()) {
			rows[index][0] = tourPoint.getDescription();
			rows[index][1] = tourPoint.getResult().getDrivingDistance() / 1000.0;
			rows[index][2] = second2hours(tourPoint.getResult().getArrivalTime());
			rows[index][3] = second2hours(tourPoint.getResult().getDrivingPeriod());
			index++;
		}

		jTourTables.addElement(rows);

		return jTourTables;
	}

	private static String second2hours(int second) {
		String time = "";
		String hours = "";
		String minutes = "";
		String seconds = "";

		int h = (second / 3600);
		if (h < 10) {
			hours = "0" + h;
		} else {
			hours = "" + h;
		}

		int rest1 = (second % 3600);
		int min = (rest1 / 60);
		if (min < 10) {
			minutes = "0" + min;
		} else {
			minutes = "" + min;
		}

		int sec = (rest1 % 60);
		if (sec < 10) {
			seconds = "0" + sec;
		} else {
			seconds = "" + sec;
		}

		time = (hours + ":" + minutes + ":" + seconds);
		return time;
	}

	private static SequencingPlan runSequencingSample(SequencingPlan plan) {
		numbers = new Hashtable<Integer, double[]>();
		// Step 1.0: Define locations
		double[] xy1010 = new double[2];
		xy1010[0] = 675000.0;
		xy1010[1] = 6380000.0;
		numbers.put(1010, xy1010);
		double[] xy1020 = new double[2];
		xy1020[0] = 671006.0;
		xy1020[1] = 6382294.0;
		numbers.put(1020, xy1020);
		double[] xy1030 = new double[2];
		xy1030[0] = 682676.0;
		xy1030[1] = 6388154.0;
		numbers.put(1030, xy1030);
		double[] xy1041 = new double[2];
		xy1041[0] = 681000.0;
		xy1041[1] = 6391000.0;
		numbers.put(1041, xy1041);
		double[] xy1042 = new double[2];
		xy1042[0] = 682000.0;
		xy1042[1] = 6382000.0;
		numbers.put(1042, xy1042);
		double[] xy201 = new double[2];
		xy201[0] = 681812.0;
		xy201[1] = 6375071.0;
		numbers.put(201, xy201);
		// Note: Center of map tile
		double[] xyCenter = new double[2];
		xyCenter[0] = 685407.0;
		xyCenter[1] = 6372537.0;
		numbers.put(0, xyCenter);
		// Step 1.1: Instantiate xTour client for sequencing
		String URL = "http://127.0.0.1:50090/xtour/ws/XTour";
		XTourRemoteInterface xSequencingClient = XTourUtils.makeClient(URL);
		// Step 1.2: Set a new CallerContext
		CallerContext cxt = new CallerContext();
		CallerContextProperty ccp = new CallerContextProperty();
		ccp.setKey("CoordFormat");
		ccp.setValue("PTV_MERCATOR");
		ccp.setKey("ResponseGeometry");
		ccp.setValue("WKT");
		ccp.setKey("Profile");
		ccp.setValue("car");
		CallerContextProperty[] properties = { ccp };
		cxt.setProperties(properties);
		// Step 1.3: Add CallerContext to xTour client.
		xSequencingClient.setCallerContext(cxt);

		// Step 2.1: Define sequencing transport depot 101
		TransportPoint transportPoint1010 = new TransportPoint();
		transportPoint1010.setId(1010);
		transportPoint1010.setDescription("Tourstop 1010");
		Point point1010 = new Point();
		point1010.setPoint(new PlainPoint(xy1010[0], xy1010[1], null));
		transportPoint1010.setLocation(point1010);
		transportPoint1010.setOpeningIntervalConstraint(OpeningIntervalConstraint.START_OF_SERVICE);
		transportPoint1010.setAllowServiceSplit(true);
		transportPoint1010.setCoDriverRequired(false);
		transportPoint1010.setServicePeriod(3600);
		transportPoint1010.setTourSection(0);
		transportPoint1010.setUseServicePeriodForRecreation(false);

		SequencingTransportDepot sequencingTransportDepot101 = new SequencingTransportDepot();
		sequencingTransportDepot101.setId(101);
		sequencingTransportDepot101.setTransportPoint(transportPoint1010);
		sequencingTransportDepot101.setPriority(1);
		Quantities quantities101 = new Quantities();
		quantities101.setQuantities(new int[] { 100 });
		sequencingTransportDepot101.setDeliveryQuantities(quantities101);

		// Step 2.2: Define sequencing transport depot 102
		TransportPoint transportPoint1020 = new TransportPoint();
		transportPoint1020.setId(1020);
		transportPoint1020.setDescription("Tourstop 1020");
		Point point1020 = new Point();
		point1020.setPoint(new PlainPoint(xy1020[0], xy1020[1], null));
		transportPoint1020.setLocation(point1020);
		transportPoint1020.setOpeningIntervalConstraint(OpeningIntervalConstraint.START_OF_SERVICE);
		transportPoint1020.setAllowServiceSplit(true);
		transportPoint1020.setCoDriverRequired(false);
		transportPoint1020.setServicePeriod(3500);
		transportPoint1020.setTourSection(0);
		transportPoint1020.setUseServicePeriodForRecreation(false);

		SequencingTransportDepot sequencingTransportDepot102 = new SequencingTransportDepot();
		sequencingTransportDepot102.setId(102);
		sequencingTransportDepot102.setTransportPoint(transportPoint1020);
		sequencingTransportDepot102.setPriority(1);
		Quantities quantities102 = new Quantities();
		quantities102.setQuantities(new int[] { 100 });
		sequencingTransportDepot102.setDeliveryQuantities(quantities102);

		// Step 2.3: Define sequencing transport depot 103
		TransportPoint transportPoint1030 = new TransportPoint();
		transportPoint1030.setId(1030);
		transportPoint1030.setDescription("Tourstop 1030");
		Point point1030 = new Point();
		point1030.setPoint(new PlainPoint(xy1030[0], xy1030[1], null));
		transportPoint1030.setLocation(point1030);
		transportPoint1030.setOpeningIntervalConstraint(OpeningIntervalConstraint.START_OF_SERVICE);
		transportPoint1030.setAllowServiceSplit(true);
		transportPoint1030.setCoDriverRequired(false);
		transportPoint1030.setServicePeriod(3300);
		transportPoint1030.setTourSection(0);
		transportPoint1030.setUseServicePeriodForRecreation(false);

		SequencingTransportDepot sequencingTransportDepot103 = new SequencingTransportDepot();
		sequencingTransportDepot103.setId(103);
		sequencingTransportDepot103.setTransportPoint(transportPoint1030);
		sequencingTransportDepot103.setPriority(1);
		Quantities quantities103 = new Quantities();
		quantities103.setQuantities(new int[] { 100 });
		sequencingTransportDepot103.setDeliveryQuantities(quantities103);

		// Step 2.4: Define sequencing transport AB 104
		TransportPoint transportPoint1041 = new TransportPoint();
		transportPoint1041.setId(1041);
		transportPoint1041.setDescription("Tourstop 1041");
		Point point1041 = new Point();
		point1041.setPoint(new PlainPoint(xy1041[0], xy1041[1], null));
		transportPoint1041.setLocation(point1041);
		transportPoint1041.setOpeningIntervalConstraint(OpeningIntervalConstraint.START_OF_SERVICE);
		transportPoint1041.setAllowServiceSplit(true);
		transportPoint1041.setCoDriverRequired(false);
		transportPoint1041.setServicePeriod(3200);
		transportPoint1041.setTourSection(0);
		transportPoint1041.setUseServicePeriodForRecreation(false);

		TransportPoint transportPoint1042 = new TransportPoint();
		transportPoint1042.setId(1042);
		transportPoint1042.setDescription("Tourstop 1042");
		Point point1042 = new Point();
		point1042.setPoint(new PlainPoint(xy1042[0], xy1042[1], null));
		transportPoint1042.setLocation(point1042);
		transportPoint1042.setOpeningIntervalConstraint(OpeningIntervalConstraint.START_OF_SERVICE);
		transportPoint1042.setAllowServiceSplit(true);
		transportPoint1042.setCoDriverRequired(false);
		transportPoint1042.setServicePeriod(5400);
		transportPoint1042.setTourSection(0);
		transportPoint1042.setUseServicePeriodForRecreation(false);

		SequencingTransportAB sequencingTransportAB104 = new SequencingTransportAB();
		sequencingTransportAB104.setId(104);
		sequencingTransportAB104.setTransportPointA(transportPoint1041);
		sequencingTransportAB104.setTransportPointB(transportPoint1042);
		Quantities quantities104 = new Quantities();
		quantities104.setQuantities(new int[] { 100 });
		sequencingTransportAB104.setQuantities(quantities104);
		sequencingTransportAB104.setPriority(1);

		// Step 2.5: Define array of transport orders
		SequencingTransportOrder[] sequencingTransportOrders = new SequencingTransportOrder[] {
				sequencingTransportDepot101, sequencingTransportDepot102, sequencingTransportDepot103, sequencingTransportAB104 };

		// Step 3.1: Define depot 201
		Depot depotLux = new Depot();
		depotLux.setId(201);
		depotLux.setDescription("Depot 201");
		Point pointDepot = new Point();
		pointDepot.setPoint(new PlainPoint(xy201[0], xy201[1], null));
		depotLux.setLocation(pointDepot);
		Interval openingInterval = new Interval();
		openingInterval.setFrom(28800);
		openingInterval.setTill(64800);
		Interval[] openingIntervals = new Interval[] { openingInterval };
		depotLux.setOpeningIntervals(openingIntervals);

		// Step 3.2: Define array of depots
		Depot[] depots = new Depot[] { depotLux };

		// Step 4: Define vehicle at depot location
		SequencingVehicle vehicle = new SequencingVehicle();
		vehicle.setDepotIdStart(201);
		vehicle.setDepotIdEnd(201);
		vehicle.setDimaId(1);

		// Step 5.1: Define planning parameters
		SequencingParams sequencingParams = new SequencingParams();
		sequencingParams.setCoDriverTransportPointsInSeparateTours(false);

		// Step 5.2: Set properties for distance matrix.
		DistanceMatrixByRoad distanceMatrixByRoad = new DistanceMatrixByRoad();
		distanceMatrixByRoad.setDimaId(1);
		distanceMatrixByRoad.setDeleteBeforeUsage(true);
		distanceMatrixByRoad.setDeleteAfterUsage(true);
		DistanceMatrixByRoad[] dimas = new DistanceMatrixByRoad[] { distanceMatrixByRoad };
		sequencingParams.setDistanceMatrixCalculation(dimas);

		// Step 5.3: Define tour restrictions
		TourRestrictions tourRestrictions = new TourRestrictions();
		tourRestrictions.setMaxDistanceBetweenTransportPoints(10000); // 10 km
		tourRestrictions.setMaxDrivingPeriod(3600); // 1h
		tourRestrictions.setMaxPeriod(21600); // 6h
		Restrictions restrictions = new Restrictions();
		restrictions.setTourRestrictions(tourRestrictions);
		sequencingParams.setRestrictions(restrictions);

		// Step 5.4: Set values for unscheduled order ids (here: all order ids)
		sequencingParams.setMaximumNumberOfUnscheduledOrdersToBeAnalyzed(4);

		// Step 5.5: Define unscheduled order ids to be analyzed (here: all order
		// ids)
		UnscheduledOrderAnalysisParams unscheduledOrderAnalysisParams = new UnscheduledOrderAnalysisParams();
		unscheduledOrderAnalysisParams.setUnscheduledOrderIdsToBeAnalyzed(new int[] { 101, 102, 103, 104 });

		// Step 6: There is no input plan used!

		try {
			// Step 7: Call method planSequence and add the arguments as
			// defined above.
			plan = xSequencingClient.planSequence(sequencingTransportOrders, depots, vehicle, sequencingParams, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem\n" + se);
		} catch (XTourException xte) {
			System.err.println("Problem tour planning\n\n" + xte);
			System.out.println(xte.getMessage());
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server\n" + xse);
		}

		return plan;
	}
}
