/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a non exclusive copyright license
 * to use all programming code examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG
 ******************************************************************************/
package samples.xtour;

/*
 * PTV xServer Code Samples
 * 
 * PTV xTour Server - Basic Tour Planning
 * 
 * (c) 2012 PTV AG
 */
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import utils.XMapUtils;
import utils.XTourUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainLineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xtour.Capacities;
import com.ptvag.xserver.xtour.Chain;
import com.ptvag.xserver.xtour.Depot;
import com.ptvag.xserver.xtour.DistanceMatrixByRoad;
import com.ptvag.xserver.xtour.Fleet;
import com.ptvag.xserver.xtour.Interval;
import com.ptvag.xserver.xtour.Plan;
import com.ptvag.xserver.xtour.Quantities;
import com.ptvag.xserver.xtour.Restrictions;
import com.ptvag.xserver.xtour.StandardParams;
import com.ptvag.xserver.xtour.Tour;
import com.ptvag.xserver.xtour.TourPoint;
import com.ptvag.xserver.xtour.TransportAB;
import com.ptvag.xserver.xtour.TransportDepot;
import com.ptvag.xserver.xtour.TransportOrder;
import com.ptvag.xserver.xtour.TransportPoint;
import com.ptvag.xserver.xtour.Vehicle;
import com.ptvag.xserver.xtour.VehicleCosts;
import com.ptvag.xserver.xtour.VehicleRestrictions;
import com.ptvag.xserver.xtour.XTourException;
import com.ptvag.xserver.xtour.XTourRemoteInterface;

public class TourPlanningBasic extends JPanel {
	private static final double[][] tourPointsXY = new double[11][3];

	public static void main(String[] args) {
		// add xTour
		Plan plan = null;
		plan = runTourSample(plan);

		// add tour object
		Vector<Object[][]> jTourTables = new Vector<Object[][]>();
		jTourTables = runTourOutput(plan, jTourTables);

		// call xMap to draw both routings
		ImageIcon icon = null;
		icon = runMappingSample(plan, icon, jTourTables);

		// view tour result
		JTable tableTourResult = new JTable(jTourTables.lastElement(), new Object[] { "Vehicle Id",
				"Driving period [h:m:s]" });
		JScrollPane scrollPaneTour = new JScrollPane(tableTourResult);
		JFrame tourOutput = new JFrame("Tour Output");
		tourOutput.getContentPane().add(scrollPaneTour);
		tourOutput.setVisible(true);
		tourOutput.setSize(400, 100);
		tourOutput.setLocation(icon.getIconWidth() + 20, 0);
		tourOutput.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JTable tableTourPerChain1 = new JTable(jTourTables.firstElement(), new Object[] { "Vehicle-ID", "Tour-ID",
				"Order-ID", "Description", "Arrival", "Departure", "Service Period [h:m:s]", "Driving Distance [km]",
				"Driving Period [h:m:s]", "Quantity [kg]" });
		JScrollPane scrollPaneChain1 = new JScrollPane(tableTourPerChain1);
		JFrame tourOutputPerChain1 = new JFrame("Output per Chain");
		tourOutputPerChain1.getContentPane().add(scrollPaneChain1);
		tourOutputPerChain1.setVisible(true);
		tourOutputPerChain1.setSize(800, 200);
		tourOutputPerChain1.setLocation(icon.getIconWidth() + 20, 100);
		tourOutputPerChain1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JTable tableTourPerChain2 = new JTable(jTourTables.get(1), new Object[] { "Vehicle-ID", "Tour-ID", "Order-ID",
				"Description", "Arrival", "Departure", "Service Period [h:m:s]", "Driving Distance [km]",
				"Driving Period [h:m:s]", "Quantity [kg]" });
		JScrollPane scrollPaneChain2 = new JScrollPane(tableTourPerChain2);
		JFrame tourOutputPerChain2 = new JFrame("Output per Chain");
		tourOutputPerChain2.getContentPane().add(scrollPaneChain2);
		tourOutputPerChain2.setVisible(true);
		tourOutputPerChain2.setSize(800, 200);
		tourOutputPerChain2.setLocation(icon.getIconWidth() + 20, 300);
		tourOutputPerChain2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// draw route onto map
		JFrame frame = new JFrame("Basic Tour Planning");
		frame.setSize(icon.getIconWidth() + 20, icon.getIconHeight() + 50);
		frame.getContentPane().add(new JLabel(icon));
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private static ImageIcon runMappingSample(Plan plan, ImageIcon icon, Vector<Object[][]> tourOutput) {
		// Step 1: Instantiate xMap client. Use package utils to create java
		// client automatically.
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		// Step 2: Use bounding box to draw the calculated route.
		BoundingBox boundingBox = new BoundingBox();
		Point leftBottom = new Point(new PlainPoint(653000, 6356000, null));
		boundingBox.setLeftTop(leftBottom);
		Point rightTop = new Point(new PlainPoint(697000, 6391000, null));
		boundingBox.setRightBottom(rightTop);

		int maxTourPoints = tourPointsXY.length;
		Bitmap[] bitmapTourPoints = new Bitmap[maxTourPoints];

		for (int i = 0; i < maxTourPoints; i++) {
			Point point = new Point();
			point.setPoint(new PlainPoint(tourPointsXY[i][0], tourPointsXY[i][1], null));

			if (i == 0) {
				Bitmap flaggreen = new Bitmap();
				flaggreen.setDescr("depot flag");
				flaggreen.setName("flaggreen.bmp");
				flaggreen.setPosition(point);
				bitmapTourPoints[i] = flaggreen;
			} else if (0 < i && i < 5) {
				Bitmap flagblue = new Bitmap();
				flagblue.setDescr("transportDepot flag");
				flagblue.setName("flagblue.bmp");
				flagblue.setPosition(point);
				bitmapTourPoints[i] = flagblue;
			} else {
				Bitmap flagred = new Bitmap();
				flagred.setDescr("transportAB flag");
				flagred.setName("flagred.bmp");
				flagred.setPosition(point);
				bitmapTourPoints[i] = flagred;
			}
		}

		// Step 3.1: Define bitmap options
		BitmapOptions bitmapOptions = new BitmapOptions();
		bitmapOptions.setTransparencyColor(new Color(255, 255, 255));

		// Step 2.3: Define array and put bitmapReachableObjects and
		// bitmapOptions into
		// bitmaps
		Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps(bitmapTourPoints, bitmapOptions) };

		// Step 3: Define custom layer for bitmaps
		CustomLayer customLayerBitmaps = new CustomLayer();
		customLayerBitmaps.setName("Draw bitmaps");
		customLayerBitmaps.setBitmaps(bitmaps);
		customLayerBitmaps.setDrawPriority(10000);
		customLayerBitmaps.setCenterObjects(true);
		customLayerBitmaps.setObjectInfos(ObjectInfoType.NONE);
		customLayerBitmaps.setVisible(true);

		// Step 3: define an array of lines
		int maxTour1 = tourOutput.get(0).length;
		int maxTour2 = tourOutput.get(1).length;
		Lines[] lines1 = new Lines[maxTour1];
		Lines[] lines2 = new Lines[maxTour2];
		Object rows1[][] = tourOutput.get(0);
		Object rows2[][] = tourOutput.get(1);
		
		for (int i = 0; i < maxTour1-1; i++) {
			// Step 3.1: create plainLineString
			PlainLineString plainLineString = new PlainLineString();
			PlainPoint plainPointStart = new PlainPoint();
			PlainPoint plainPointDestination = new PlainPoint();
			int loc1 = (Integer) rows1[i][2];
			int loc2 = (Integer) rows1[i+1][2];
			double xy1[][] = getLocation(loc1);
			plainPointStart.setX(xy1[0][0]);
			plainPointStart.setY(xy1[0][1]);
			//System.out.println("Lokation-1: " + xy1[0][0] + "   " + xy1[0][1]  + "   " + loc1);
			double xy2[][] = getLocation(loc2);
			plainPointDestination.setX(xy2[0][0]);
			plainPointDestination.setY(xy2[0][1]);
			//System.out.println("Lokation-2: " + xy2[0][0] + "   " + xy2[0][1]  + "   " + loc2);
			plainLineString.setPoints(new PlainPoint[] {plainPointStart, plainPointDestination});
			
			// Step 3.2: set object into lineStrings[0]. Note: object lineStrings has
			// to be reinitialized in every loop.
			LineString[] lineStrings = new LineString[1];
			lineStrings[0] = new LineString();
			lineStrings[0].setLineString(plainLineString);
			
			// Step 3.3: create new lines[i] and fill array
			lines1[i] = new Lines();
			lines1[i].setLines(lineStrings);
			
			// Step 3.4: set lineOptions transparency
			LineOptions lineOptions = new LineOptions();
			lineOptions.setTransparent(true);
			
			// Step 3.5: set linePartOptions visible, define width of line and
			// specific color
			LinePartOptions linePartOptions = new LinePartOptions();
			linePartOptions.setVisible(true);
			linePartOptions.setWidth(3);
			linePartOptions.setColor(new Color(255,0,0));
			
			// Step 3.6: set main line
			lineOptions.setMainLine(linePartOptions);
			
			// Step 3.7: set all options for specific line
			lines1[i].setOptions(lineOptions);	
		}
		
		for (int i = 0; i < maxTour2-1; i++) {
			// Step 3.1: create plainLineString
			PlainLineString plainLineString = new PlainLineString();
			PlainPoint plainPointStart = new PlainPoint();
			PlainPoint plainPointDestination = new PlainPoint();
			int loc1 = (Integer) rows2[i][2];
			int loc2 = (Integer) rows2[i+1][2];
			double xy1[][] = getLocation(loc1);
			plainPointStart.setX(xy1[0][0]);
			plainPointStart.setY(xy1[0][1]);
			//System.out.println("Lokation-1: " + xy1[0][0] + "   " + xy1[0][1]  + "   " + loc1);
			double xy2[][] = getLocation(loc2);
			plainPointDestination.setX(xy2[0][0]);
			plainPointDestination.setY(xy2[0][1]);
			//System.out.println("Lokation-2: " + xy2[0][0] + "   " + xy2[0][1]  + "   " + loc2);
			plainLineString.setPoints(new PlainPoint[] {plainPointStart, plainPointDestination});
			
			// Step 3.2: set object into lineStrings[0]. Note: object lineStrings has
			// to be reinitialized in every loop.
			LineString[] lineStrings = new LineString[1];
			lineStrings[0] = new LineString();
			lineStrings[0].setLineString(plainLineString);
			
			// Step 3.3: create new lines[i] and fill array
			lines2[i] = new Lines();
			lines2[i].setLines(lineStrings);
			
			// Step 3.4: set lineOptions transparency
			LineOptions lineOptions = new LineOptions();
			lineOptions.setTransparent(true);
			
			// Step 3.5: set linePartOptions visible, define width of line and
			// specific color
			LinePartOptions linePartOptions = new LinePartOptions();
			linePartOptions.setVisible(true);
			linePartOptions.setWidth(3);
			linePartOptions.setColor(new Color(0,0,255));
			
			// Step 3.6: set main line
			lineOptions.setMainLine(linePartOptions);
			
			// Step 3.7: set all options for specific line
			lines2[i].setOptions(lineOptions);	
		}

		CustomLayer customLayerTourOutput1 = new CustomLayer();
		customLayerTourOutput1.setName("Draw lines1");
		customLayerTourOutput1.setLines(lines1);
		customLayerTourOutput1.setDrawPriority(500);
		customLayerTourOutput1.setCenterObjects(true);
		customLayerTourOutput1.setObjectInfos(ObjectInfoType.NONE);
		customLayerTourOutput1.setVisible(true);
		
		CustomLayer customLayerTourOutput2 = new CustomLayer();
		customLayerTourOutput2.setName("Draw lines2");
		customLayerTourOutput2.setLines(lines2);
		customLayerTourOutput2.setDrawPriority(500);
		customLayerTourOutput2.setCenterObjects(true);
		customLayerTourOutput2.setObjectInfos(ObjectInfoType.NONE);
		customLayerTourOutput2.setVisible(true);

		// Step 4: Define array and put customLayer into layers
		Layer[] layers = new Layer[] { customLayerBitmaps, customLayerTourOutput1, customLayerTourOutput2 };

		// Step 5: Create class imageInfo and set format and dimension.
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.PNG);
		imageInfo.setHeight(600);
		imageInfo.setWidth(800);

		// Step 6: Use mapParams for scale and distance unit
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(false);

		try {
			// Step 7: Call renderMapBoundingBox and return object map.
			Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, false);

			String imageUrl = "http://" + map.getImage().getUrl();
			try {
				icon = new ImageIcon(new URL(imageUrl));
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}

		} catch (SystemException se) {
			System.err.println("Network or URL problem (xmap): " + se);
		} catch (XMapException xme) {
			System.err.println("Problem with calculating a map" + xme);
		} catch (XServiceException xse) {
			System.err.println("Technical problem on xMap server");
		}

		return icon;
	}
	
	private static double[][] getLocation(int tourPoint){
		double xy[][] = new double[1][2];
		
		for (int i = 0; i < tourPointsXY.length; i++) {
			if (tourPoint == tourPointsXY[i][2]){
				xy[0][0] = tourPointsXY[i][0];
				xy[0][1] = tourPointsXY[i][1];
			}
		}
		
		//System.out.println("Lokation: " + xy[0][0] + "   " + xy[0][1]  + "   " + tourPoint);
		return xy;
	}

	private static Vector<Object[][]> runTourOutput(Plan plan, Vector<Object[][]> jTourTables) {
		int index = 0;
		Object[][] rowsA = new Object[plan.getChains().length][3];

		for (Chain chain : plan.getChains()) {
			rowsA[index][0] = chain.getVehicleId();
			rowsA[index][1] = second2hours(chain.getResult().getDrivingPeriod());
			int kindex = 0;
			for (Tour tour : chain.getTours()) {
				Object[][] rowsB = new Object[tour.getTourPoints().length][10];
				for (TourPoint tourPoint : tour.getTourPoints()) {
					rowsB[kindex][0] = chain.getVehicleId();
					rowsB[kindex][1] = tour.getId();
					rowsB[kindex][2] = tourPoint.getId();
					rowsB[kindex][3] = tourPoint.getDescription();
					rowsB[kindex][4] = second2hours(tourPoint.getResult().getArrivalTime());
					rowsB[kindex][5] = second2hours(tourPoint.getResult().getDepartureTime());
					rowsB[kindex][6] = second2hours(tourPoint.getResult().getServicePeriod());
					rowsB[kindex][7] = String.format("%.1f", tourPoint.getResult().getDrivingDistance() / 1000.0);
					rowsB[kindex][8] = second2hours(tourPoint.getResult().getDrivingPeriod());
					rowsB[kindex][9] = tourPoint.getResult().getCurrentQuantities().getQuantities()[0];
					kindex++;
				}
				jTourTables.addElement(rowsB);
			}

			index++;
		}

		jTourTables.addElement(rowsA);

		return jTourTables;
	}

	private static String second2hours(int second) {
		String time = "";
		String hours = "";
		String minutes = "";
		String seconds = "";

		int h = (second / 3600);
		if (h < 10) {
			hours = "0" + h;
		} else {
			hours = "" + h;
		}

		int rest1 = (second % 3600);
		int min = (rest1 / 60);
		if (min < 10) {
			minutes = "0" + min;
		} else {
			minutes = "" + min;
		}

		int sec = (rest1 % 60);
		if (sec < 10) {
			seconds = "0" + sec;
		} else {
			seconds = "" + sec;
		}

		time = (hours + ":" + minutes + ":" + seconds);
		return time;
	}

	private static Plan runTourSample(Plan plan) {

		// Step 1.1: instantiate xTour client
		String URL = "http://127.0.0.1:50090/xtour/ws/XTour";
		XTourRemoteInterface xTourClient = XTourUtils.makeClient(URL);
		// Step 1.2: Set a new CallerContext
		CallerContext cxt = new CallerContext();
		CallerContextProperty ccp = new CallerContextProperty();
		ccp.setKey("ResponseGeometry");
		ccp.setValue("WKT");
		ccp.setKey("Profile");
		ccp.setValue("truck");
		CallerContextProperty[] properties = { ccp };
		cxt.setProperties(properties);
		// Step 1.3: Add CallerContext to xTour client.
		xTourClient.setCallerContext(cxt);

		// Step 2.1: Create object depot
		Depot depotLux = new Depot();
		depotLux.setDescription("Depot Luxembourg");
		int depotID = 9999;
		depotLux.setId(depotID);

		// Step 2.2: Set location
		PlainPoint plainPointDepot = new PlainPoint();
		tourPointsXY[0][0] = 681812.0;
		tourPointsXY[0][1] = 6375071.0;
		tourPointsXY[0][2] = 9999;
		plainPointDepot.setX(tourPointsXY[0][0]);
		plainPointDepot.setY(tourPointsXY[0][1]);
		Point location = new Point();
		location.setPoint(plainPointDepot);
		depotLux.setLocation(location); // Rue Munchen-Tesch, Luxembourg

		// Step 2.3: Define openingInterval for depot
		Interval openingInterval = new Interval();
		openingInterval.setFrom(28800); // from 8:00 until 18:00
		openingInterval.setTill(64800);
		Interval[] openingIntervals = { openingInterval };
		depotLux.setOpeningIntervals(openingIntervals);

		// Step 2.4: Put depotLux into the array depots
		Depot[] depots = { depotLux };

		// Step 3.1: Define transport depot 100 and set transport point 301
		TransportDepot transportDepot100 = new TransportDepot();
		transportDepot100.setId(100);
		transportDepot100.setDepotIds(new int[] { depotID });
		//transportDepot100.setVehicleId(1);
		transportDepot100.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportDepot100.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportDepot100.setPriority(1);

		// Step 3.2: Define transportPoint 301
		PlainPoint plainPoint301 = new PlainPoint();
		tourPointsXY[1][0] = 682000.0;
		tourPointsXY[1][1] = 6382000.0;
		tourPointsXY[1][2] = 301;
		plainPoint301.setX(tourPointsXY[1][0]);
		plainPoint301.setY(tourPointsXY[1][1]);
		Point point301 = new Point();
		point301.setPoint(plainPoint301);
		TransportPoint transportPoint301 = new TransportPoint();
		transportPoint301.setLocation(point301); // Rue Baron de Reinach,
		// Heisdorf
		transportPoint301.setId(301);
		transportPoint301.setServicePeriod(5800); // 1 hour
		transportPoint301.setDescription("shipment order 301: transport goods to customer 1");
		transportDepot100.setTransportPoint(transportPoint301);

		// Step 3.3: Define quantities for delivery of shipment 301
		Quantities deliveryQuantities301 = new Quantities();
		int[] quantities301 = new int[] { 10, 0, 0, 0, 0, 0 };
		deliveryQuantities301.setQuantities(quantities301);
		transportDepot100.setDeliveryQuantities(deliveryQuantities301);

		// Step 4.1: Define transport depot 200 and set transport point 401
		TransportDepot transportDepot200 = new TransportDepot();
		transportDepot200.setId(200);
		transportDepot200.setDepotIds(new int[] { depotID });
		//transportDepot200.setVehicleId(1);
		transportDepot200.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportDepot200.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportDepot200.setPriority(1);

		// Step 4.2: Define transportPoint 401
		PlainPoint plainPoint401 = new PlainPoint();
		tourPointsXY[2][0] = 671006.0;
		tourPointsXY[2][1] = 6382294.0;
		tourPointsXY[2][2] = 401;
		plainPoint401.setX(tourPointsXY[2][0]);
		plainPoint401.setY(tourPointsXY[2][1]);
		Point point401 = new Point();
		point401.setPoint(plainPoint401);
		TransportPoint transportPoint401 = new TransportPoint();
		transportPoint401.setLocation(point401); // Rue de Nospelt, Kehlen
		transportPoint401.setId(401);
		transportPoint401.setServicePeriod(5900);
		transportPoint401.setDescription("shipment order 401: transport goods to customer 2");
		transportDepot200.setTransportPoint(transportPoint401);

		// Step 4.3: Define quantities for delivery of shipment 401
		Quantities deliveryQuantities401 = new Quantities();
		int[] quantities401 = new int[] { 10, 0, 0, 0, 0, 0 };
		deliveryQuantities401.setQuantities(quantities401);
		transportDepot200.setDeliveryQuantities(deliveryQuantities401);

		// Step 5.1: Define transport depot 300 and set transport point 501
		TransportDepot transportDepot300 = new TransportDepot();
		transportDepot300.setId(300);
		transportDepot300.setDepotIds(new int[] { depotID });
		//transportDepot300.setVehicleId(1);
		transportDepot300.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportDepot300.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportDepot300.setPriority(1);

		// Step 5.2: Define transportPoint 501
		PlainPoint plainPoint501 = new PlainPoint();
		tourPointsXY[3][0] = 682812.0;
		tourPointsXY[3][1] = 6388220.0;
		tourPointsXY[3][2] = 501;
		plainPoint501.setX(tourPointsXY[3][0]);
		plainPoint501.setY(tourPointsXY[3][1]);
		Point point501 = new Point();
		point501.setPoint(plainPoint501);
		TransportPoint transportPoint501 = new TransportPoint();
		transportPoint501.setLocation(point501); // Rue Eugene Nickels,
		// Lorentzweiler
		transportPoint501.setId(501);
		transportPoint501.setServicePeriod(6000);
		transportPoint501.setDescription("shipment order 501: transport goods to customer 3");
		transportDepot300.setTransportPoint(transportPoint501);

		// Step 5.3: Define quantities for delivery of shipment 501
		Quantities deliveryQuantities501 = new Quantities();
		int[] quantities501 = new int[] { 10, 0, 0, 0, 0, 0 };
		deliveryQuantities501.setQuantities(quantities501);
		transportDepot300.setDeliveryQuantities(deliveryQuantities501);

		// Step 6.1: Define transport depot 400 and set transport point 601
		TransportDepot transportDepot400 = new TransportDepot();
		transportDepot400.setId(400);
		transportDepot400.setDepotIds(new int[] { depotID });
		//transportDepot400.setVehicleId(1);
		transportDepot400.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportDepot400.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportDepot400.setPriority(1);

		// Step 6.2: Define transportPoint 601
		PlainPoint plainPoint601 = new PlainPoint();
		tourPointsXY[4][0] = 678665.0;
		tourPointsXY[4][1] = 6356256.0;
		tourPointsXY[4][2] = 601;
		plainPoint601.setX(tourPointsXY[4][0]);
		plainPoint601.setY(tourPointsXY[4][1]);
		Point point601 = new Point();
		point601.setPoint(plainPoint601);
		TransportPoint transportPoint601 = new TransportPoint();
		transportPoint601.setLocation(point601); // Rue des Cheminots,
		// Bettembourg
		transportPoint601.setId(601);
		transportPoint601.setServicePeriod(6200);
		transportPoint601.setDescription("shipment order 601: transport goods to customer 4");
		transportDepot400.setTransportPoint(transportPoint601);

		// Step 6.3: Define quantities for delivery of shipment 501
		Quantities deliveryQuantities601 = new Quantities();
		int[] quantities601 = new int[] { 10, 0, 0, 0, 0, 0 };
		deliveryQuantities601.setQuantities(quantities601);
		transportDepot400.setDeliveryQuantities(deliveryQuantities601);

		// Step 7.1: Define shipment AB 101
		TransportAB transportAB101 = new TransportAB();
		transportAB101.setId(101);
		//transportAB101.setVehicleId(2);
		transportAB101.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportAB101.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportAB101.setPriority(1);

		// Step 7.2: Define transportPoint for location A1001 (pickup) and B1001
		// (delivery).
		PlainPoint plainPointA1001 = new PlainPoint();
		tourPointsXY[5][0] = 681000.0;
		tourPointsXY[5][1] = 6391000.0;
		tourPointsXY[5][2] = 1001;
		plainPointA1001.setX(tourPointsXY[5][0]);
		plainPointA1001.setY(tourPointsXY[5][1]);
		Point pointA1001 = new Point();
		pointA1001.setPoint(plainPointA1001);
		TransportPoint transportPointA1001 = new TransportPoint();
		transportPointA1001.setId(1001);
		transportPointA1001.setLocation(pointA1001); // Rue de Diekirch, Lintgen
		transportPointA1001.setServicePeriod(3000);
		transportPointA1001.setDescription("Pickup goods from location 1001");
		transportAB101.setTransportPointA(transportPointA1001);

		PlainPoint plainPointB1002 = new PlainPoint();
		tourPointsXY[6][0] = 675566.0;
		tourPointsXY[6][1] = 6380559.0;
		tourPointsXY[6][2] = 1002;
		plainPointB1002.setX(tourPointsXY[6][0]);
		plainPointB1002.setY(tourPointsXY[6][1]);
		Point pointB1002 = new Point();
		pointB1002.setPoint(plainPointB1002);
		TransportPoint transportPointB1002 = new TransportPoint();
		transportPointB1002.setId(1002);
		transportPointB1002.setLocation(pointB1002); // Rue des Bouleaux, Kopstal
		transportPointB1002.setServicePeriod(3100);
		transportPointB1002.setDescription("Deliver goods at location 1002");
		transportAB101.setTransportPointB(transportPointB1002);

		// Step 7.3: Define quantities for pickup and delivery A1001/B1002
		Quantities quantitiesA1001B1002 = new Quantities();
		int[] quantitiesAB100 = new int[] { 10, 0, 0, 0, 0, 0 };
		quantitiesA1001B1002.setQuantities(quantitiesAB100);
		transportAB101.setQuantities(quantitiesA1001B1002);

		// Step 8.1: Define shipment AB 201
		TransportAB transportAB201 = new TransportAB();
		transportAB201.setId(201);
		//transportAB201.setVehicleId(2);
		transportAB201.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportAB201.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportAB201.setPriority(1);

		// Step 8.2: Define transportPoint for location A2001 (pickup) and B2001
		// (delivery).
		PlainPoint plainPointA2001 = new PlainPoint();
		tourPointsXY[7][0] = 668207.0;
		tourPointsXY[7][1] = 6390449.0;
		tourPointsXY[7][2] = 2001;
		plainPointA2001.setX(tourPointsXY[7][0]);
		plainPointA2001.setY(tourPointsXY[7][1]);
		Point pointA2001 = new Point();
		pointA2001.setPoint(plainPointA2001);
		TransportPoint transportPointA2001 = new TransportPoint();
		transportPointA2001.setId(2001);
		transportPointA2001.setLocation(pointA2001); // Rue de Brouch, Tuntange
		transportPointA2001.setServicePeriod(3250);
		transportPointA2001.setDescription("Pickup goods from location 2001");
		transportAB201.setTransportPointA(transportPointA2001);

		PlainPoint plainPointB2002 = new PlainPoint();
		tourPointsXY[8][0] = 679108.0;
		tourPointsXY[8][1] = 6396522.0;
		tourPointsXY[8][2] = 2002;
		plainPointB2002.setX(tourPointsXY[8][0]);
		plainPointB2002.setY(tourPointsXY[8][1]);
		Point pointB2002 = new Point();
		pointB2002.setPoint(plainPointB2002);
		TransportPoint transportPointB2002 = new TransportPoint();
		transportPointB2002.setId(2002);
		transportPointB2002.setLocation(pointB2002); // Rue Bouvart,
		// Mersch
		transportPointB2002.setServicePeriod(3300);
		transportPointB2002.setDescription("Deliver goods at location 2002");
		transportAB201.setTransportPointB(transportPointB2002);

		// Step 8.3: Define quantities for pickup and delivery A2001/B2002
		Quantities quantitiesA2001B2002 = new Quantities();
		int[] quantitiesAB200 = new int[] { 10, 0, 0, 0, 0, 0 };
		quantitiesA2001B2002.setQuantities(quantitiesAB200);
		transportAB201.setQuantities(quantitiesA2001B2002);

		// Step 9.1: Define shipment AB 301
		TransportAB transportAB301 = new TransportAB();
		transportAB301.setId(301);
		//transportAB301.setVehicleId(2);
		transportAB301.setVehicleCategories(new int[] { 1, 2, 3, 4, 5 });
		transportAB301.setVehicleRequirements(new int[] { 1, 2, 3, 4, 5 });
		transportAB301.setPriority(1);

		// Step 9.2: Define transportPoint for location A3001 (pickup) and B3001
		// (delivery).
		PlainPoint plainPointA3001 = new PlainPoint();
		tourPointsXY[9][0] = 653966.0;
		tourPointsXY[9][1] = 6369424.0;
		tourPointsXY[9][2] = 3001;
		plainPointA3001.setX(tourPointsXY[9][0]);
		plainPointA3001.setY(tourPointsXY[9][1]);
		Point pointA3001 = new Point();
		pointA3001.setPoint(plainPointA3001);
		TransportPoint transportPointA3001 = new TransportPoint();
		transportPointA3001.setId(3001);
		transportPointA3001.setLocation(pointA3001); // Rue de Fingig, Clemency
		transportPointA3001.setServicePeriod(3400);
		transportPointA3001.setDescription("Pickup goods from location 3001");
		transportAB301.setTransportPointA(transportPointA3001);

		PlainPoint plainPointB3002 = new PlainPoint();
		tourPointsXY[10][0] = 696741.0;
		tourPointsXY[10][1] = 6373860.0;
		tourPointsXY[10][2] = 3002;
		plainPointB3002.setX(tourPointsXY[10][0]);
		plainPointB3002.setY(tourPointsXY[10][1]);
		Point pointB3002 = new Point();
		pointB3002.setPoint(plainPointB3002);
		TransportPoint transportPointB3002 = new TransportPoint();
		transportPointB3002.setId(3002);
		transportPointB3002.setLocation(pointB3002); // Rue de Neuhausen,
		// Schuttrange
		transportPointB3002.setServicePeriod(3600);
		transportPointB3002.setDescription("Deliver goods at location 3002");
		transportAB301.setTransportPointB(transportPointB3002);

		// Step 9.3: Define quantities for pickup and delivery A3001/B3002
		Quantities quantitiesA3001B3002 = new Quantities();
		int[] quantitiesAB300 = new int[] { 10, 0, 0, 0, 0, 0 };
		quantitiesA3001B3002.setQuantities(quantitiesAB300);
		transportAB301.setQuantities(quantitiesA3001B3002);

		// Step 9.4: Create object transportOrders and fill with all
		// transportDepot
		// and transportAB objects.
		// This is required for planBasicTours().
		TransportOrder[] transportOrders = { transportDepot100, transportDepot200, transportDepot300, transportDepot400,
				transportAB101, transportAB201, transportAB301 };

		// Step 10.1: Create two new instances for the specified vehicle types
		// which should be used.
		Fleet fleet = new Fleet();
		// Step 10.2: Define properties for vehicle one.
		Vehicle vehicle1 = new Vehicle();
		vehicle1.setId(1);
		vehicle1.setDepotIdStart(depotID);
		vehicle1.setDepotIdEnd(depotID);
		vehicle1.setIsPreloaded(false);
		vehicle1.setDescription("Truck Type A");
		vehicle1.setCategory(1);
		VehicleRestrictions vehRestrictions1 = new VehicleRestrictions();
		vehRestrictions1.setMaxTourCountPerChain(1);
		vehRestrictions1.setMaxDrivingPeriodPerChain(28800);
		vehRestrictions1.setMaxTransportPointCountPerTour(10);
		vehicle1.setRestrictions(vehRestrictions1);
		vehicle1.setTransportEquipments(new int[] { 1, 2, 3, 4, 5 });
		Capacities capacities1 = new Capacities();
		Quantities quantity1 = new Quantities();
		quantity1.setQuantities(new int[] { 80, 0, 0, 0, 0, 0, 0 });
		Quantities[] quantities1 = new Quantities[] { quantity1 };
		capacities1.setCapacities(quantities1);
		vehicle1.setCapacities(capacities1);
		VehicleCosts costs1 = new VehicleCosts();
		costs1.setCostsPerHourDriverOnly(80.0);
		costs1.setCostsPerKm(20.0);
		vehicle1.setCosts(costs1);
		vehicle1.setDimaId(1000);

		// Step 10.3: Define properties for vehicle two.
		Vehicle vehicle2 = new Vehicle();
		vehicle2.setId(2);
		vehicle2.setDepotIdStart(depotID);
		vehicle2.setDepotIdEnd(depotID);
		vehicle2.setIsPreloaded(false);
		vehicle2.setDescription("Truck Type B");
		vehicle2.setCategory(2);
		VehicleRestrictions vehRestrictions2 = new VehicleRestrictions();
		vehRestrictions2.setMaxTourCountPerChain(1);
		vehRestrictions2.setMaxDrivingPeriodPerChain(28800);
		vehRestrictions2.setMaxTransportPointCountPerTour(10);
		vehicle2.setRestrictions(vehRestrictions2);
		vehicle2.setTransportEquipments(new int[] { 1, 2, 3, 4, 5 });
		Capacities capacities2 = new Capacities();
		Quantities quantity2 = new Quantities();
		quantity2.setQuantities(new int[] { 40, 0, 0, 0, 0, 0, 0 });
		Quantities[] quantities2 = new Quantities[] { quantity2 };
		capacities2.setCapacities(quantities2);
		vehicle2.setCapacities(capacities2);
		VehicleCosts costs2 = new VehicleCosts();
		costs2.setCostsPerHourDriverOnly(75.0);
		costs2.setCostsPerKm(18.0);
		vehicle2.setCosts(costs2);
		vehicle2.setDimaId(1000);

		// Step 10.4: Add vehicles to the specified array. Set array to fleet.
		Vehicle[] vehicles = { vehicle1, vehicle2 };
		fleet.setVehicles(vehicles);

		// Step 11.1: Define planning parameters
		StandardParams planningParams = new StandardParams();

		// Step 11.2: Add restrictions. See complete list in our API.
		Restrictions restrictions = new Restrictions();
		restrictions.setMaxTransportPointCountPerTour(10);
		restrictions.setMaxTourCount(2);
		planningParams.setRestrictions(restrictions);

		// Step 11.3: Set properties for distance matrix.
		DistanceMatrixByRoad distanceMatrixByRoad = new DistanceMatrixByRoad();
		distanceMatrixByRoad.setDimaId(1000);
		distanceMatrixByRoad.setDeleteBeforeUsage(true);
		distanceMatrixByRoad.setDeleteAfterUsage(true);
		DistanceMatrixByRoad[] dimas = new DistanceMatrixByRoad[] { distanceMatrixByRoad };
		planningParams.setDistanceMatrixCalculation(dimas);

		// Step 11.4: Set standard properties.
		planningParams.setConstructionStep(true);
		planningParams.setImprovementStep(true);
		planningParams.setTourReductionStep(false);

		// Step 12: we don't add an input plan

		try {
			// Step 13: call method planBsicTours and add the arguments as
			// defined above.
			plan = xTourClient.planBasicTours(transportOrders, depots, fleet, planningParams, null);
		} catch (SystemException se) {
			System.err.println("Network or URL problem\n" + se);
		} catch (XTourException xte) {
			System.err.println("Problem tour planning\n\n" + xte);
			System.out.println(xte.getMessage());
		} catch (XServiceException xse) {
			System.err.println("Technical problem on server\n" + xse);
		}

		return plan;
	}
}
