package utils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.ClientFactory;
import com.ptvag.jabba.service.baseservices.RemoteType;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.BasicBitmap;
import com.ptvag.xserver.xmap.Bitmap;
import com.ptvag.xserver.xmap.BitmapOptions;
import com.ptvag.xserver.xmap.Bitmaps;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.MapSection;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;

/**
 * this utility class contains static methods for xmap convenience
 * 
 * @author RN
 */
public final class XMapUtils {

	private XMapUtils() {
		// this is a utility class!
	}
	
	public static String rgbToColorString(int red, int green, int blue) {
		int color = (blue<<16)+(green<<8)+(red);
		return ""+color;
	}

	public static XMapRemoteInterface makeClient(String url) {
		XMapRemoteInterface client = (XMapRemoteInterface) ClientFactory.createClient(XMapRemoteInterface.class, RemoteType.DOCSTYLE_CXF, "","",url);
		CallerContext cc = CommonUtils.makeDefaultContext();
		client.setCallerContext(cc);
        client.setUsername("user");
        client.setPassword("pwd");
		return client;
	}
	
	public static Color makeColor(int red, int green, int blue) {
		Color res = new Color();
		res.setRed(red);
		res.setGreen(green);
		res.setBlue(blue);
		return res;
	}
	
	public static Color makeColor(java.awt.Color color) {
		Color res = new Color();
		res.setRed(color.getRed());
		res.setGreen(color.getGreen());
		res.setBlue(color.getBlue());
		return res;
	}
	
	public static MapSection makeMapSectionByCenterAndScale(Point center, int scale) {
		MapSection res = new MapSection();
		res.setCenter(center);
		res.setScale(scale);
		return res;
	}
	
	public static MapSection makeRelativeMapSection(MapSection current, int zoom, int scrollHorizontal, int scrollVertical) {
		MapSection res = new MapSection();
		res.setCenter(current.getCenter());
		res.setScale(current.getScale());
		res.setScrollHorizontal(scrollHorizontal);
		res.setScrollVertical(scrollVertical);
		res.setZoom(zoom);
		return res;
	}

	public static CustomLayer makeRouteLayer(LineString route, Color color, Point[] stations)  {
		CustomLayer routeLayer = new CustomLayer();
		LinePartOptions lpo = new LinePartOptions();
		lpo.setColor(color);
		lpo.setWidth(-3);
		lpo.setVisible(true);
		LineOptions lineOpts = new LineOptions();
		lineOpts.setShowFlags(false);
		if (stations != null) {
			Bitmaps bitmaps = new Bitmaps();
			BasicBitmap[] bms = new BasicBitmap[stations.length];
			for (int i = 0; i < stations.length; i++) {
				Bitmap bm = new Bitmap();
				bm.setName("FlagRed.bmp");
				bm.setPosition(stations[i]);
				bm.setDescr("waypoint " + i);
				bms[i] = bm;
			}
			BitmapOptions bos = new BitmapOptions();
			bitmaps.setOptions(bos);
			bitmaps.setBitmaps(bms);
			routeLayer.setBitmaps(new Bitmaps[] {bitmaps});

		}
		if (route != null) {
			lineOpts.setMainLine(lpo);
			lineOpts.setTransparent(true);
			Lines routeLines = new Lines(new LineString[] {route}, lineOpts);
			routeLayer.setLines(new Lines[] { routeLines });
		}
		routeLayer.setVisible(true);
		routeLayer.setDrawPriority(25000);
		return routeLayer;
	}
	
	public static Map renderCenteredRoute(XMapRemoteInterface client, ImageFileFormat format, int imgWidth, int imgHeight, LineString route, Color color, Point[] stations, boolean includeImageInResponse) throws SystemException, XServiceException, XMapException {
		MapSection ms = makeMapSectionByCenterAndScale(CommonUtils.makePoint(1479530, 6889308), 100); // this is just a dummy point
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(false);
		mapParams.setUseMiles(false);
		mapParams.setLanguage("x-ptv-DFT");
		
		ImageInfo ii = new ImageInfo(format, imgWidth, imgHeight, "");
		Layer[] layers = new Layer[] {};
		CustomLayer routeLayer = makeRouteLayer(route, color, stations);
		routeLayer.setName("route");
		routeLayer.setVisible(true);
		routeLayer.setCenterObjects(true);
		layers = new Layer[] { routeLayer };
		return client.renderMap(ms, mapParams, ii, layers, includeImageInResponse);
	}
	
}
